//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	BasicSkill.cpp
 *  \brief	abstraktna trieda, pre vsetky zakladne schopnosti
 *  \author Rado Otipka 
 *  \date 4. 12. 2001 21:49:11
 *  \version 
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_SKILL_MAP(CBasicSkill)
END_SKILL_MAP

//------------------------------------------------------
/*! \fn	CBasicSkill::CBasicSkill(
	const int iType,
	const int iPriority,
	CWorldModel &cWorld,
	CGameCommandSeq &cActions
	)
 *  \brief Konstruktor
 *  \return
 *  \author KPDL
 *  \date 2.5.2002 9:41:58
 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CBasicSkill::CBasicSkill(
	const int iType,
	const int iPriority,
	CWorldModel &cWorld,
	CGameCommandSeq &cActions
)
:	iType(iType),
	iPriority(iPriority),
	m_cWorld(cWorld),
	m_cActions(cActions),
	m_cSituation(cWorld),
	m_dAcceleration(0.0),
	m_dPower(0.0),
	m_dDirection(0.0),
	m_dMoment(0.0),
	m_iPlayerNumber(0),
	m_eWidth(VW_None),
	m_eQuality(VQ_None),
	m_cPosition(),
	m_sMessage(),
	m_eEffort(EFF_Minimum)
{
}

//------------------------------------------------------
/*! \fn	CBasicSkill::CBasicSkill(
	TSkillParameterVector &cParameters,
	const int iType,
	const int iPriority,
	CWorldModel &cWorld,
	CGameCommandSeq &cActions
	)
 *  \brief konstruktor
 *  \return
 *  \author KPDL
 *  \date 2.5.2002 9:42:13
 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CBasicSkill::CBasicSkill(
	TSkillParameterVector &cParameters,
	const int iType,
	const int iPriority,
	CWorldModel &cWorld,
	CGameCommandSeq &cActions
)
:	iType(iType),
	iPriority(iPriority),
	m_cWorld(cWorld),
	m_cSituation(cWorld),
	m_cActions(cActions),
	m_dAcceleration(0.0),
	m_dPower(0.0),
	m_dDirection(0.0),
	m_dMoment(0.0),
	m_iPlayerNumber(0),
	m_eWidth(VW_None),
	m_eQuality(VQ_None),
	m_cPosition(),
	m_sMessage(),
	m_eEffort(EFF_Minimum)
{
	TSkillParameterVectorIterator pIter = cParameters.begin();
	TSkillParameterVectorIterator pLast = cParameters.end();

	for(; pIter != pLast; pIter++ )
	{
		if(pIter->sName == BSKILL_PARAM_TYPE)
			this->iType = atoi(pIter->sValue.c_str());
		else if(pIter->sName == BSKILL_PARAM_X)
			m_cPosition.SetX(atof(pIter->sValue.c_str())).Validate();
		else if(pIter->sName == BSKILL_PARAM_Y) 
			m_cPosition.SetY(atof(pIter->sValue.c_str())).Validate();
		else if(pIter->sName == BSKILL_PARAM_ANGLE)
			m_dDirection = atof(pIter->sValue.c_str());
		else if(pIter->sName == BSKILL_PARAM_MOMENT) 
			m_dMoment = atof(pIter->sValue.c_str());
		else if(pIter->sName == BSKILL_PARAM_ACCEL)
			m_dAcceleration = atof(pIter->sValue.c_str());
		else if(pIter->sName == BSKILL_PARAM_WIDTH)
			m_eWidth = (EViewWidthType) atoi(pIter->sValue.c_str());
		else if(pIter->sName == BSKILL_PARAM_QUALITY)
			m_eQuality = (EViewQualityType) atoi(pIter->sValue.c_str());
		else if(pIter->sName == BSKILL_PARAM_PLAYER)
			m_iPlayerNumber = atoi(pIter->sValue.c_str());
		else if(pIter->sName == BSKILL_PARAM_EFFORT) 
		{
			if(pIter->sValue == "Maximum") 
				m_eEffort = EFF_Maximum;
			else if(pIter->sValue == "Medium") 
				m_eEffort = EFF_Medium;
			else if(pIter->sValue == "Minimum") 
				m_eEffort = EFF_Minimum;
		}	
	}
}

//------------------------------------------------------
/*! \fn	CBasicSkill::CBasicSkill(const CBasicSkill &cSource) 
 *  \brief kopirovaci konstruktor
 *  \param const CBasicSkill &cSourc
 *  \return
 *  \author KPDL
 *  \date 2.5.2002 9:42:30
 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CBasicSkill::CBasicSkill(const CBasicSkill &cSource) 
:	iType(cSource.iType),
	iPriority(cSource.iPriority),
	m_cWorld(cSource.m_cWorld),
	m_cSituation(m_cWorld),
	m_cActions(cSource.m_cActions),
	m_dAcceleration(cSource.m_dAcceleration),
	m_dPower(cSource.m_dPower),
	m_dDirection(cSource.m_dDirection),
	m_dMoment(cSource.m_dMoment),
	m_iPlayerNumber(cSource.m_iPlayerNumber),
	m_eWidth(cSource.m_eWidth),
	m_eQuality(cSource.m_eQuality),
	m_cPosition(cSource.m_cPosition),
	m_sMessage(cSource.m_sMessage),
	m_eEffort(m_eEffort)
{
}

//------------------------------------------------------
/*! \fn	KPDL_RESULT CBasicSkill::Execute()
 *  \brief vykona zakladnu schopnost
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:43:24
 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
KPDL_RESULT CBasicSkill::Execute()
{
	SSkillMap *pEntry = GetSkillMap();
	
	while(pEntry->fnSkill != NULL)
	{
		if(pEntry->iType == iType)
			return (this->*(pEntry->fnSkill))();
		pEntry++;

	}

	return KPDL_ERROR;
};

//------------------------------------------------------
/*! \fn	CBasicSkill::operator =(const CBasicSkill &cSource) 
 *  \brief operator porovnania
 *  \param const CBasicSkill &cSourc
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:43:41
 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CBasicSkill::operator =(const CBasicSkill &cSource) 
{
	iType = cSource.iType;
	iPriority = cSource.iPriority;
	m_dAcceleration = cSource.m_dAcceleration;
	m_dPower = cSource.m_dPower;
	m_dDirection = cSource.m_dDirection;
	m_dMoment = cSource.m_dMoment;
	m_iPlayerNumber = cSource.m_iPlayerNumber;
	m_eWidth = cSource.m_eWidth;
	m_eQuality = cSource.m_eQuality;
	m_cPosition = cSource.m_cPosition;
	m_sMessage = cSource.m_sMessage;
	m_eEffort  = cSource.m_eEffort;
} 
