//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	BasicSkill.h
 *	\brief  Deklaracia triedy  abstraktnej triedy CBasiSkill, 
 *	\brief  od ktorej su odvodene vsetky triedy 
 *	\brief  implementujuce zakladnych schopnosti
 *  \author Rado Otipka
 *  \date 2. 12. 2001 15:23:28
 *  \version 2.52.1
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#ifndef _BASIC_SKILL_
#define _BASIC_SKILL_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CBasicSkill;
class CBasicSkillList;

typedef vector<CBasicSkill *> TBasicSkillVector;
typedef TBasicSkillVector::iterator TBasicSkillVectorIterator;
typedef KPDL_RESULT (CBasicSkill::*TSkillFunction)();

struct SSkillMap
{
	int iType;
	TSkillFunction fnSkill;
};

#define DECLARE_SKILL_MAP \
	protected: \
		static SSkillMap m_tSkillMap[]; \
		virtual SSkillMap *GetSkillMap() { return m_tSkillMap; }
#define BEGIN_SKILL_MAP(class) SSkillMap class::m_tSkillMap[] = {
#define ADD_SKILL(type, skill) {(type), (TSkillFunction)&skill},
#define END_SKILL_MAP {BSKILL_NO_TYPE, (TSkillFunction)NULL}};

class CBasicSkill  
{
	DECLARE_SKILL_MAP

public:
	int iType;
	int iPriority;
	string sRemoveRule;
	CBasicSkillList *m_pListSkill;

protected:
	CWorldModel &m_cWorld;
	CGameCommandSeq &m_cActions;
	CEstimateLocalSituation	m_cSituation;
	// sada standardnych vnutornych parametrov dualneho rozrania zakladne schopnosti
	double m_dAcceleration;
	double m_dPower;
	double m_dDirection;
	double m_dMoment;
	double m_dStaminaFactor; 
	EEffort m_eEffort;
	int m_iPlayerNumber;
	EViewWidthType m_eWidth;
	EViewQualityType m_eQuality;
	C2DVector m_cPosition;
	string m_sMessage;

public:
	// dualne rozhranie schopnosti
	CBasicSkill(
		TSkillParameterVector &cParameters,
		const int iType,
		const int iPriority,
		CWorldModel &cWorld,
		CGameCommandSeq &cActions
	);

	// priame rozhranie schopnosti
	CBasicSkill(
		const int iType,
		const int iPriority,
		CWorldModel &cWorld,
		CGameCommandSeq &cActions
	);

	CBasicSkill(const CBasicSkill &cSource);
	virtual ~CBasicSkill() {}

	KPDL_RESULT Execute();

	bool IsTypeOf(const int iTestType) const
	{
		return 
			iType >= iTestType && 
			iType < iTestType + BSKILL_SUBTYPE_LIMIT;
	}
	operator =(const CBasicSkill &cSkill); 
};

// pomocne operatory
inline bool operator <(
	const CBasicSkill &cLeft,
	const CBasicSkill &cRight
)
{
	return(cLeft.iPriority < cRight.iPriority);
}

inline bool operator ==(
	const CBasicSkill &cLeft, 
	const CBasicSkill &cRight
) 
{
	return(cLeft.iPriority == cRight.iPriority);
}

inline bool operator <=(
	const CBasicSkill &cLeft, 
	const CBasicSkill &cRight
) 
{
	return(cLeft.iPriority <= cRight.iPriority);
}

#endif // _BASIC_SKILL_
