//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	Dribble.cpp
 *  \brief  Implementacia triedy CDribble, 
 *  \brief  zakladnej schopnost vedenia lopty
 *  \author Rado Otipka 
 *  \date 10. 12. 2001 18:14:09
 *  \version 
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

////////////////////////////////CDribble/////////////////////////////////////////////

//------------------------------------------------------
/*! \fn	CDribble::CDribble(
	TSkillParameterVector &cParameters,
	const int iPriority,
	CWorldModel &cWorld,
	CGameCommandSeq &cActions
	)
 *  \brief Konstruktor
 *  \return
 *  \author Radoslav Otipka 
 *  \date 7.5.2002 11:43:05

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CDribble::CDribble(
	TSkillParameterVector &cParameters,
	const int iPriority,
	CWorldModel &cWorld,
	CGameCommandSeq &cActions
)
:	CBasicSkill(cParameters, BSKILL_DRIBBLE, iPriority, cWorld, cActions)
{
}

//------------------------------------------------------
/*! \fn	CDribble::CDribble(
	const int iDribleType,
	const C2DVector &cDestination,
	const double dAcceleration,
	const int iPriority,
	CWorldModel &cWorld, 
	CGameCommandSeq &cActions
	)
 *  \brief konstruktor
 *  \return
 *  \author Radoslav Otipka 
 *  \date 7.5.2002 11:44:46

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CDribble::CDribble(
	const int iDribleType,
	const C2DVector &cDestination,
	const double dAcceleration,
	const int iPriority,
	CWorldModel &cWorld, 
	CGameCommandSeq &cActions
)
:	CBasicSkill(iDribleType, iPriority, cWorld, cActions) 
{
	m_cPosition = cDestination;
	m_dAcceleration = dAcceleration;
}

BEGIN_SKILL_MAP(CDribble)
	ADD_SKILL(BSKILL_DRIBLE_SMART, SmartDribble)
END_SKILL_MAP

//------------------------------------------------------
/*! \fn	const double CDribble::GetBallVelocity(
						C2DVector &cOutVelocityBall, 
						C2DVector &cPosition, 
						const double dAccel
		)
 *  \brief vrati rychlost lopty, vyhladom na poziciu lopty a hraca a zrychlenia
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:47:31

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
const double CDribble::GetBallVelocity(
						C2DVector &cOutVelocityBall, 
						C2DVector &cPosition, 
						const double dAccel
				)
{
    C2DVector v = cPosition - m_cWorld.cBall.cPosition;
    // uhol relativne k hracovemu natoceniu
    double ang = v.GetAngle() - m_cWorld.cThis.dBodyDirection;

	CMovingObject &cBall = m_cWorld.cBall;
    C2DVector vball = m_cWorld.cThis.RelVectorTo(
							cBall.cPosition
					  );
    double dist_ball = vball.GetDistance();
    double dir_diff = fabs(RadToDeg(vball.GetAngle()));
    double factor = 
		gpConfig->kick_power_rate * 
		(1.0 - 0.25 * dir_diff / 180.0 - 0.25 * 
			(dist_ball - gpConfig->player_size - gpConfig->ball_size)
			/ gpConfig->kickable_margin
		);
	C2DVector vkick;
    vkick.SetAngleDistance(ang, dAccel);
	vkick.SetDistance(vkick.GetDistance() * factor);
//    vkick += pBall->GetSpeed();
	cOutVelocityBall = vkick;
	return vkick.GetDistance();
}

//------------------------------------------------------
/*! \fn	const double CDribble::GetBallVelocityDir(
							C2DVector &cOutVelocityBall, 
							const double dDirection, 
							const double dAccel
							)
 *  \brief vrati smer lopty vzhladom na poziciu lopty hraca a zrychlenia
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:48:11

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
const double CDribble::GetBallVelocityDir(
							C2DVector &cOutVelocityBall, 
							const double dDirection, 
							const double dAccel
							)
{
    //C2DVector v = pos - (C2DVector)GetAgent()->GetWorldModel()->GetBall()->GetPosition();
    // uhol relativne k hracovemu natoceniu
    double ang = dDirection;

	CMovingObject &cBall = m_cWorld.cBall;
    C2DVector vball = m_cWorld.cThis.RelVectorTo( cBall.cPosition );
    double dist_ball = vball.GetDistance();
    double dir_diff = fabs(RadToDeg(vball.GetAngle()));
    double factor = 
		gpConfig->kick_power_rate * 
		(1.0 - 0.25 * dir_diff / 180.0 - 0.25 * 
		(dist_ball - gpConfig->player_size - gpConfig->ball_size) /
		gpConfig->kickable_margin
	);
    
	C2DVector vkick;
    vkick.SetAngleDistance(ang, dAccel);
	//vkick.SetDistance(vkick.GetDistance() * factor);
//    vkick += pBall->GetSpeed();
	cOutVelocityBall = vkick;
	return vkick.GetDistance();
}

//------------------------------------------------------
/*! \fn	const double CDribble::GetBoundary() 
 *  \brief vzdialenost od ktorej sa bude viest lopta
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:48:42

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
const double CDribble::GetBoundary() 
{
	C2DVector ballPos = m_cWorld.cBall.cPosition;
	double boundary = 
		fabs(((m_cWorld.cThis.cPosition - ballPos).GetDistance() 
		- gpConfig->kickable_area)
		- m_cWorld.cThis.cSpeed.GetDistance() 
		- m_cWorld.cBall.cSpeed.GetDistance()
	);
	//boundary *= 0.5;
	return boundary;
}

//------------------------------------------------------
/*! \fn	void CDribble::Dribble(const C2DVector cDestPos, double dAccel)
 *  \brief realizuje zakladnu schopnosti vedenia lopty
 *  \param const C2DVector cDestPos
 *  \param  double dAcce
 *  \return
 *  \author KPDL
 *  \date 1.5.2002 20:34:08
 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
void CDribble::Dribble(const C2DVector cDestPos, double dAccel)
{
	//dAccel = 100;
	/*C2DVector ballPos = GetAgent()->GetWorldModel()->GetBall()->GetPosition();
	C2DVector nextBallPos;*/
	
	//zisti ako dlho si uz nevidel loptu
/*	if( GetAgent()->GetTime().m_nPlayTime - GetAgent()->GetWorldModel()->GetBall()->m_cLastVisible.m_nPlayTime > 4) {
		 CTurn( BSKILL_TURN_TO_BALL,0,GetAgent(),101 ).Execute(); //hladaj loptu
		 return;
	}
	
	double boundary = GetBoundary();
	if (m_wasKick && m_oldKickCount+1 == agent->GetKickCount()) {
		C2DVector topos = m_localSituation->RelVectorTo(GetAgent(), destPos);
		if( topos.GetAngle() > 7 || topos.GetAngle() <7 ) {
			CTurn( BSKILL_TURN_2,topos.GetAngle(),GetAgent(),0 ).Execute();
			agent->listSkl->AddSkill(new CDribble(BSKILL_DRIBLE_SMART ,destPos, 0, agent, 100));
			return;
		}
		CMove( BSKILL_MOVE_DASH , (m_nextPos - GetAgent()->GetPosition()).GetDistance(), GetAgent(),101).Execute();	
//		CMove( BSKILL_MOVE_GOTO, m_nextPos, 0, GetAgent(),101).Execute();
		cBSkillLog.Write(true,"Choj za loptou po kopnuti: %lf %lf",m_nextPos.m_x,m_nextPos.m_y);
		m_wasKick = 1;
	} else {
		if (m_oldKickCount == agent->GetKickCount()) {
			//kopaj ak mas do coho
			if(IsBallOnDribblePos()) {
				double new_vel = (GetAgent()->bskl->KickRealToWanted(GetAgent(),boundary,0)).GetDistance();
				CKick(BSKILL_KICK_TO_POSITION,  boundary, destPos,GetAgent(),0).Execute();
				m_wasKick = 1;
				//naplnanujem pohyb na poziciu
				C2DVector v = destPos - (C2DVector)GetAgent()->GetWorldModel()->GetBall()->GetPosition();
				// uhol relativne k hracovemu natoceniu
				double ang = v.GetAngle() - GetAgent()->GetDirection();
				//
				NextMove(boundary, nextBallPos, ang);
				agent->listSkl->AddSkill(new CDribble(BSKILL_DRIBLE_SMART ,destPos, 0, m_wasKick, agent->GetKickCount(),nextBallPos,agent, 100));
				return;
			} else { //ak nemas do coho kopat tak sa nastav tak aby si mal
				//skus ist tesne pred loptu
				double distance = 0.1;
				CMove(BSKILL_MOVE_GOTO, ballPos, 0, distance,GetAgent(),101).Execute();

				cBSkillLog.Write(true,"Choj za loptou: %lf %lf",destPos.m_x,destPos.m_y);
			}			
		}
	}
	m_wasKick = 0;
	agent->listSkl->AddSkill(new CDribble(BSKILL_DRIBLE_SMART ,destPos, 0, agent, 100));*/
	C2DVector topos = m_cWorld.cThis.RelVectorTo( cDestPos);
	//C2DVector vBall = GetAgent()->GetWorldModel()->GetBall()->GetSpeed();
	//C2DVector vPa  = GetAgent()->GetSpeed();
	//topos.GetAngle
	//C2DVector  final = vBall + vPa;
	C2DVector ballPos = m_cWorld.cBall.cPosition;
	C2DVector agentPos = m_cWorld.cThis.cPosition;
	if(!ballPos.IsValid()) 
	{
		//hladaj loptu
		CTurn(
			BSKILL_TURN_TO_BALL,
			0,
			BSKILL_DEFAULT_PRIORITY, 
			m_cWorld,
			m_cActions
		).Execute(); 
//		cBSkillLog.Write(true,"otoc sa k lopte");
		return;
	}
   // if (m_localSituation->BallKickable(GetAgent())) {
	double dDirection=0;

	if(IsBallOnDribblePos()) 
	{

		/*	if(!IsBallInWriteDirection(dDirection)) {
				CTurn( BSKILL_TURN_2,
						dDirection,
						GetAgent(),
						101).Execute();
				return;
			}*/

		if( (cDestPos - agentPos).GetDistance() < 
			(cDestPos - ballPos).GetDistance() ) 
		{
			CKick(BSKILL_KICK_WITH_ACCEL_BYPASS, 
					35, 
					cDestPos ,
					BSKILL_NO_PRIORITY,
					m_cWorld,
					m_cActions
	    	).Execute();
			return;
		}
		else 
		{
			CKick(BSKILL_KICK_WITH_ACCEL_BYPASS, 
					20, 
					cDestPos ,
					BSKILL_NO_PRIORITY,
					m_cWorld,
					m_cActions
			).Execute();
		}

		if( topos.GetAngle() > 0.7 || topos.GetAngle() < 0.7 )
			m_pListSkill->AddSkill(	CTurn(
										BSKILL_TURN_2,
										topos.GetAngle(),
							 			101,
										m_cWorld,
										m_cActions
			));

			m_pListSkill->AddSkill(	CMove( 
										BSKILL_MOVE_DASH, 
										0,
										BSKILL_DEFAULT_PRIORITY,	
										m_cWorld,
										m_cActions
			));	
	} 
	else 
	{
		C2DVector nextBallPos = m_cSituation.BallPosition(3); /// ???
		
		CMove( 
			BSKILL_MOVE_GOTO , 
			ballPos, -100, 
			BSKILL_NO_PRIORITY,
			m_cWorld, 
			m_cActions
		).Execute();
    }
	if( (cDestPos - m_cWorld.cThis.cPosition).GetDistance() > MIN_DIST ) 
		m_pListSkill->AddSkill( CDribble( 
								BSKILL_DRIBLE_SMART, 
								cDestPos,
								dAccel, 
								BSKILL_DEFAULT_PRIORITY,
								m_cWorld, 
								m_cActions

		));
}

//------------------------------------------------------
/*! \fn	const bool CDribble::IsBallOnDribblePos()
 *  \brief otestuje ci je lopta na pozicii 
 *  \brief vhodnej pre vedenie lopty
 *  \return
 *  \author KPDL
 *  \date 1.5.2002 20:35:40

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
const bool CDribble::IsBallOnDribblePos()
{
	C2DVector deltaPos = m_cWorld.cBall.cPosition  - m_cWorld.cThis.cPosition;
	double accel = gpConfig->kickable_area;
	if (deltaPos.GetDistance()<accel) return true; else return false;
}


//------------------------------------------------------
/*! \fn	const bool CDribble::IsBallInWriteDirection(double &dDirection)
 *  \brief je lopta na spravnej pozicii na vedenie lopty
 *  \param double &dDirectio
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:49:29

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
const bool CDribble::IsBallInWriteDirection(double &dDirection)
{
	dDirection = m_cWorld.cThis.RelVectorTo(
		m_cWorld.cBall.cPosition
		).GetAngle();
	if(RadToDeg(dDirection)>5) return false ;
		else return true;	
}

//------------------------------------------------------
/*! \fn	void CDribble::NextMove(
		C2DVector &cNextPos, 
		const double dBoundary, 
		const double dAngle
	)
 *  \brief vypocitaj dalsiu poziciu, kam ma ist hrac za loptou
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:50:07

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
void CDribble::NextMove(
		C2DVector &cNextPos, 
		const double dBoundary, 
		const double dAngle
	)
{
	C2DVector kickVectordest;
	GetBallVelocityDir(kickVectordest, dAngle, dBoundary);
	cNextPos = kickVectordest + m_cWorld.cBall.cPosition;
	cNextPos += kickVectordest*gpConfig->ball_decay;
}
//////////////////////stari dribble/////////hore novy vo viacerych funkciach//
/*KPDL_RESULT CDribble::SmartDribble() 
{
	return Dribble(destPos, accel);


/*	double dashAccel;
	//C2DVector vBall = GetAgent()->GetWorldModel()->GetBall()->GetSpeed();
	//C2DVector vPa  = GetAgent()->GetSpeed();
	//topos.GetAngle
	//C2DVector  final = vBall + vPa;
	C2DVector ballPos = GetAgent()->GetWorldModel()->GetBall()->GetPosition();
	C2DVector nextBallPos, kick;
	C2DVector agentPos = GetAgent()->GetPosition();
	
	
	
	double useAccel = 30; 
	//ak som mensej vzdialenosti ako jedna tak zastav
	if ((destPos-agent->GetPosition()-destPos).GetDistance() < 1) {
			return;
	}
	
	//C2DVector toposBall = m_localSituation->RelVectorTo(GetAgent(), ballPos);
	double disttoball = agent->bskl->RelVectorTo(agent, agent->GetWorldModel()->GetBall()->GetPosition()).GetDistance();
    double dist = GetAgent()->cfg->kickable_area - disttoball;
	
	
	C2DVector topos = m_localSituation->RelVectorTo(GetAgent(), ballPos);
	double dir = topos.GetAngle();
	kick.SetAngleDistance(dir, boundary);
	nextBallPos= ballPos + kick;


	//cBSkillLog.Write(true,"Boundary: %lf",boundary);
	//fprintf(m_ff, "Dribble current time: %d \n", GetAgent()->GetTime().m_nPlayTime);
	if (GetAgent()->GetTime().m_nPlayTime - GetAgent()->GetWorldModel()->GetBall()->m_cLastVisible.m_nPlayTime > 3) {
		//GetAgent()->listSkl->Empty(); // kym sa nezosinchronizujeme s timy hore
		//CTurn( BSKILL_TURN_TO_BALL,0,GetAgent(),101 ).Execute();
		//return;
	}

	accel = GetAgent()->cfg->kickable_area ;//* (1 - GetAgent()->cfg->ball_decay ) + 0.2;
//    if (disttoball < GetAgent()->cfg->kickable_area*0.9) {
	if (m_wasKick == 0) {
		if(agent->GetSpeed().GetDistance() < accel)
			//agent->GetWorldModel()->GetBall()->GetSpeed().GetDistance() < 0.1
			 {
			if(GetAgent()->GetWorldModel()->GetBall()->GetSpeed().GetDistance() < 0.1) 
			{			
				double new_vel = (GetAgent()->bskl->KickRealToWanted(GetAgent(),boundary,0)).GetDistance();
				CKick(BSKILL_KICK_TO_POSITION, boundary, destPos, GetAgent(),0).Execute();
				useAccel = boundary;
			} else {
				double new_vel = (GetAgent()->bskl->KickRealToWanted(GetAgent(),boundary,0)).GetDistance();
				CKick(BSKILL_KICK_TO_POSITION, boundary, destPos, GetAgent(),0).Execute();
				useAccel = boundary ;
			}
			C2DVector kickVectordest;
			GetBallVelocityDir(kickVectordest, 0, useAccel);
			nextBallPos = kickVectordest + ballPos;
			nextBallPos += kickVectordest*GetAgent()->cfg->ball_decay;
			dashAccel = GetAgent()->bskl->DashRealToWanted(GetAgent(),(nextBallPos-agent->GetPosition()).GetDistance());
			///
			double speed = GetAgent()->GetWorldModel()->GetBall()->GetSpeed().GetDistance();
	//		cBSkillLog.Write(true,"Rychlost lopty: %lf",speed);
			speed = GetAgent()->GetSpeed().GetDistance();
	//		cBSkillLog.Write(true,"Rychlost hraca: %lf",speed);
			///
			m_wasKick = 1;
			agent->listSkl->AddSkill(new CDribble(BSKILL_DRIBLE_SMART ,destPos, 0, m_wasKick, agent->GetKickCount(),nextBallPos,agent, 100));
			return;
			//GetAgent()->listSkl->AddSkill(new CMove( BSKILL_MOVE_GOTO,nextBallPos ,0, GetAgent(),101));	
			//BSKILL_MOVE_DASH
		}	else {
  	      /*nextBallPos = ballPos + agent->GetWorldModel()->GetBall()->GetSpeed() * agent->cfg->ball_decay;
			nextBallPos += agent->GetWorldModel()->GetBall()->GetSpeed() * agent->cfg->ball_decay *agent->cfg->ball_decay;
			dashAccel = GetAgent()->bskl->DashRealToWanted(GetAgent(),(nextBallPos-agent->GetPosition()).GetDistance());*/
//			nextBallPos = agent->GetWorldModel()->GetBall()->GetSpeed()*agent->cfg->ball_decay + ballPos;
//			agent->listSkl->AddSkill(new CDribble(BSKILL_DRIBLE_SMART ,destPos, 0, m_wasKick, agent->GetKickCount(),nextBallPos,agent, 100));
			//GetAgent()->listSkl->AddSkill(new CMove( BSKILL_MOVE_GOTO, nextBallPos, 0, GetAgent(),101));
//			return;
//		}
  //  } else {
	//	Daco();
		//nextBallPos = agent->GetWorldModel()->GetBall()->GetSpeed()*GetAgent()->cfg->ball_decay*GetAgent()->cfg->ball_decay + ballPos;
		//CMove( BSKILL_MOVE_GOTO , nextBallPos, 0, GetAgent(), 0).Execute();	
		//GetAgent()->listSkl->AddSkill(new CDribble(BSKILL_DRIBLE_SMART,destPos,10,GetAgent(),102));
    // }
    // agent->listSkl->AddSkill(new CDribble(BSKILL_DRIBLE_SMART ,destPos, 0, m_wasKick, agent->GetKickCount(),nextBallPos,agent, 100));
//};




KPDL_RESULT CDribble::SmartDribble() 
{

/*	KPDL_RESULT eResult = KPDL_SUCCESS;

	C2DVector cToPos = m_cWorld.cThis.RelVectorTo(m_cPosition);
	//C2DVector cBallSpeed = m_cWorld.cBall.cSpeed;
	//C2DVector cThisSpeed = m_cWorld.cThis.cSpeed;
	//C2DVector cFinalSpeed = cBallSpeed + cThisSpeed;
	C2DVector cBallPosition = m_cWorld.cBall.cPosition;
	C2DVector cThisPosition = m_cWorld.cThis.cPosition;
	C2DVector cToPosBall = m_cWorld.cThis.RelVectorTo(cBallPosition);

	if(m_cWorld.cBall.iLastSeen > 3)
	{
		//GetAgent()->listSkl->Empty(); // kym sa nezosinchronizujeme s timy hore
		return CTurn(BSKILL_TURN_2, 0.7, 101, m_cWorld, m_cActions).Execute();
	}
	
	double dAcceleration = gpConfig->kickable_area * (1 - gpConfig->ball_decay) + 0.2;
    if(m_cSituation.BallKickable())
	{
		if((m_cPosition - cThisPosition).GetDistance() < 
			(m_cPosition - cBallPosition).GetDistance()) 
		{
			eResult = CKick(
				BSKILL_KICK_WITH_ACCEL_BYPASS, 
				dAcceleration + 20, 
				m_cPosition, 
				0, 
				m_cWorld, 
				m_cActions
			).Execute();
			listSkl->AddSkill(new CTurn(
				BSKILL_TURN_2, 
				cToPos.GetAngle(), 
				101, 
				m_cWorld, 
				m_cActions
			));
			return eResult;
		} 

		CKick(
			BSKILL_KICK_WITH_ACCEL_BYPASS, 
			dAcceleration + 20, 
			m_cPosition, 
			0, 
			m_cWorld, 
			m_cActions
		).Execute();

		if(cToPosBall.GetAngle() > -DegToRad(7) && 
			cToPosBall.GetAngle() < DegToRad(7))
			m_pListSkill->AddSkill(*(new CTurn(
				BSKILL_TURN_2,
				cToPosBall.GetAngle(),
				101,
				m_cWorld,
				m_cActions
			)));

		m_pListSkill->AddSkill(*(new CMove(
			BSKILL_MOVE_DASH, 
			100, 
			100, 
			m_cWorld, 
			m_cActions
		)));
    }
	else
	{
		//C2DVector cNextBallPosition = m_cSituation.BallPosition(m_cWorld.cTime.m_nPlayTime + 1);
		//CMove(BSKILL_MOVE_GOTO, cNextBallPosition, 100, 0, m_cWorld, m_cActions).Execute();
	CMove(BSKILL_MOVE_GOTO, cBallPosition, 100, 0, m_cWorld, m_cActions).Execute();	
		if(cToPosBall.GetAngle() > - DegToRad(7) && 
			cToPosBall.GetAngle() < DegToRad(7))
		{
			eResult = m_pListSkill->AddSkill(*(new CTurn(
				BSKILL_TURN_2,
				cToPos.GetAngle(),
				101,
				m_cWorld,
				m_cActions
			)));
			//return eResult;
		}

		//CMove(BSKILL_MOVE_DASH, 100, 101, m_cWorld, m_cActions).Execute();
    }

	if((m_cPosition - cThisPosition).GetDistance() > MIN_DIST) 
		listSkl->AddSkill(new CDribble(
			BSKILL_DRIBLE_SMART,
			m_cPosition,
			dAcceleration, 
			99,
			m_cWorld,
			m_cActions
		));


	return eResult;
*/	
	return KPDL_SUCCESS;
};
