//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	EstimateSituation.cpp
 *  \brief	ohodnotenie situacie pre zakladne schopnosti
 *  \author Rado Otipka 
 *  \date 4. 12. 2001 21:49:11
 *  \version 
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//------------------------------------------------------
/*! \fn	CEstimateLocalSituation::CEstimateLocalSituation(CWorldModel &cWorld)
 *  \brief konstruktor
 *  \param CWorldModel &cWorl
 *  \return
 *  \author Radoslav Otipka 
 *  \date 7.5.2002 11:46:42

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CEstimateLocalSituation::CEstimateLocalSituation(CWorldModel &cWorld)
:	m_cWorld(cWorld)
{
}

//------------------------------------------------------
/*! \fn	bool CEstimateLocalSituation::BallKickable()
 *  \brief mozem kopnut do lopty
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:50:38

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
bool CEstimateLocalSituation::BallKickable()
{
    double dDistToBall = m_cWorld.cThis.RelVectorTo(
		m_cWorld.cBall.cPosition
	).GetDistance();
    return dDistToBall <= gpConfig->kickable_area;
}

bool CEstimateLocalSituation::NearGoal(const EGoalType eGoalType) 
{
	return false;
}

//------------------------------------------------------
/*! \fn	CAgent *CEstimateLocalSituation::PlayerInWay(const double dDirection)
 *  \brief vrati hraca v ceste
 *  \param const double dDirectio
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:50:53

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CAgent *CEstimateLocalSituation::PlayerInWay(const double dDirection)
{
    C2DVector cToPlayer;
    double dPlayerDirection;
    double dPlayerDistance;
	double dPlayerDistance2;
    double dMinDirection;
    CAgent *pResult = NULL;

    // hladame medzi identifikovanymi hracmi
	TAgentVectorIterator pIter = m_cWorld.cIdentifiedAgents.begin();
	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();

	for(; pIter != pLast; pIter ++)
	{
        // vektor na hraca
        cToPlayer = m_cWorld.cThis.RelVectorTo((*pIter)->cPosition);
        dPlayerDirection = cToPlayer.GetAngle();
        dPlayerDistance = cToPlayer.GetDistance();
        dMinDirection = MinimalDirection(dPlayerDistance);
        if(fabs(dPlayerDirection - dDirection) < dMinDirection)
		{
            if(pResult != NULL)
			{
                dPlayerDistance2 = m_cWorld.cThis.RelVectorTo(
					pResult->cPosition
				).GetDistance();
                if(dPlayerDistance < dPlayerDistance2) 
	                // vratime toho co je blizsie
					pResult = *pIter;
            }
            else
                pResult = *pIter;
        }
    }

    // hladame medzi neidentifikovanymi hracmi
	pIter = m_cWorld.cUnIdentifiedAgents.begin();
	pLast = m_cWorld.cUnIdentifiedAgents.end();

	for(; pIter != pLast; pIter ++)
	{
        // vektor na hraca
        cToPlayer = m_cWorld.cThis.RelVectorTo( (*pIter)->cPosition);
        dPlayerDirection = cToPlayer.GetAngle();
        dPlayerDistance = cToPlayer.GetDistance();
        dMinDirection = MinimalDirection(dPlayerDistance);
        if(fabs(dPlayerDirection - dDirection) < dMinDirection)
		{
            if(pResult != NULL)
			{
                dPlayerDistance2 = m_cWorld.cThis.RelVectorTo(
					pResult->cPosition
				).GetDistance();
                if(dPlayerDistance < dPlayerDistance2) 
	                // vratime toho co je blizsie
					pResult = *pIter;
            }
            else
                pResult = *pIter;
        }
    }

    return pResult;
}


//------------------------------------------------------
/*! \fn	CAgent *CEstimateLocalSituation::NearestOpponentToBall()
 *  \brief vracia najblizsieho oponnenta k lopte
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:51:31

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CAgent *CEstimateLocalSituation::NearestOpponentToBall()
{
    C2DVector cToBall = m_cWorld.cThis.RelVectorTo(m_cWorld.cBall.cPosition);
    C2DVector cToPlayer;
    double dMinDistance = MAX_DISTANCE;
    CAgent *pResult = NULL;
    double dDistToBall;

    // hladame medzi identifikovanymi hracmi
	TAgentVectorIterator pIter = m_cWorld.cIdentifiedAgents.begin();
	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();

	for(; pIter != pLast; pIter ++)
	{
        if(!(*pIter)->IsValid() || (*pIter)->eSide == S_Home)
			continue;

        // vektor na hraca
        cToPlayer = m_cWorld.cThis.RelVectorTo((*pIter)->cPosition);
        dDistToBall = (cToBall - cToPlayer).GetDistance();
        
        if (dDistToBall < dMinDistance) {
            pResult = *pIter;
            dMinDistance = dDistToBall;
        }
    }

    return pResult;
}

//------------------------------------------------------
/*! \fn	CAgent *CEstimateLocalSituation::FreestFriend()
 *  \brief vracia volneho spoluhraca
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:51:53

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CAgent *CEstimateLocalSituation::FreestFriend()
{
	double dClosestOpponent;
	double dMinimum = MAX_DISTANCE;
    CAgent *pResult = NULL;

    // hladame medzi identifikovanymi hracmi
	TAgentVectorIterator pIter = m_cWorld.cIdentifiedAgents.begin();
	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();

	for(; pIter != pLast; pIter ++)
	{
        if((*pIter)->eSide == S_Home)
		{
			dClosestOpponent = DistToClosestOpponent((*pIter)->cPosition);
			if(dClosestOpponent <= dMinimum) 
			{
				dMinimum = dClosestOpponent;
				pResult = *pIter;
			}
        }
    }
	return pResult;
}

//------------------------------------------------------
/*! \fn	bool CEstimateLocalSituation::BallCatchable()
 *  \brief moze brankar zachytit loptu
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:52:11

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
bool CEstimateLocalSituation::BallCatchable()
{    
	if(m_cWorld.cThis.iLastCatch <= gpConfig->catch_ban_cycle)
        return false;
        
    // a zistit ci sa moze chytat - da sa iba raz za nejaky cas
    C2DVector cToBall = m_cWorld.cThis.RelVectorTo(m_cWorld.cBall.cPosition);
    bool bCatchArea = cToBall.GetDistance() < gpConfig->catchable_area_l;

//! t0n0 -> pri tomto teste sa nesmie nastavovat, ze bol aplikovany catch
//  az, ked je to potvrdene v sense body !!!
//	if (bCatchArea)
//        pa->m_last_catch = pa->cTime.m_nAllCycles;

    return bCatchArea;
}

//------------------------------------------------------
/*! \fn	const C2DVector CEstimateLocalSituation::BallPosition(
		const int iTime, 
		const C2DVector &cNewBallSpeed
	)
 *  \brief vrati poziciu lopty, v urcitom case a s urcitou rychlostou
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:52:46

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
const C2DVector CEstimateLocalSituation::BallPosition(
		const int iTime, 
		const C2DVector &cNewBallSpeed
	)
{
    C2DVector cBallPos = m_cWorld.cBall.cPosition;

    // geometricky rad - postupne spomalovanie lopty
    double x;
    if (gpConfig->ball_decay != 1)
        x = (pow(gpConfig->ball_decay, (double)iTime) - 1) / (gpConfig->ball_decay - 1);
    else
        x = iTime * gpConfig->ball_decay;

    return cBallPos + cNewBallSpeed * x;
}

//------------------------------------------------------
/*! \fn	const C2DVector CEstimateLocalSituation::BallPosition(const int iTime)
 *  \brief pozicia lopty o iTime cyklov
 *  \param const int iTim
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:53:15

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
const C2DVector CEstimateLocalSituation::BallPosition(const int iTime)
{
	  C2DVector cBallPos = m_cWorld.cBall.cPosition;
    C2DVector cNewBallSpeed = m_cWorld.cBall.cSpeed;

/*
    double x;
    if (gpConfig->ball_decay != 1)
        x = (pow(gpConfig->ball_decay, (double)nTime) - 1) / (gpConfig->ball_decay - 1);
    else
        x = nTime * gpConfig->ball_decay;

    return cBallPos + cNewBallSpeed * x;
*/

	for(int i = 0; i < iTime; i++)
	{
		cBallPos += cNewBallSpeed;
        cNewBallSpeed *= gpConfig->ball_decay;
	}

    return cBallPos;
}


//------------------------------------------------------
/*! \fn	const C2DVector CEstimateLocalSituation::GetBallInterceptPosition(
		const C2DVector &cNewBallSpeed, 
		CAgent &cAgent
	)
 *  \brief vrati poziciu na zachytenie lopty
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:53:35

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
const C2DVector CEstimateLocalSituation::GetBallInterceptPosition(
		const C2DVector &cNewBallSpeed, 
		CAgent &cAgent
	)
{
    C2DVector v;


    // ratame s tym, ze sa musi najprv otocit do spravneho uhlu.
    // keby sa dalo nejako zistit ci sa musi otocit alebo nie, bolo by to presnejsie
    int nTurn = 1;

    // - ratanie polohy playera
    C2DVector cNewPos, cSpd, cAccel, cu;
//    cNewPos = pa->cPosition;
    cNewPos.SetX(0.0).SetY(0.0).Validate();


    cSpd.SetX(0.0).SetY(0.0).Validate();



    //for (int i=0; i < 32; i++)    // pokial to ma ist ???   // Rado Otipka 8. 12. 2001 12:22:23
	for (int i=0; i < 120; i++)     // pokial to ma ist ???   
    {
        // - ratanie polohy playera
        cAccel.SetAngleDistance(0, gpConfig->maxpower * gpConfig->dash_power_rate); // todo: zapocitat staminu!
        // cAccel.SetDistance(gpConfig->maxpower * gpConfig->dash_power_rate); // todo: zapocitat staminu!
        if (cAccel.GetDistance() > gpConfig->player_accel_max) cAccel.SetDistance(gpConfig->player_accel_max);
        cu = cSpd + cAccel;
        cNewPos += cu;
        cSpd = cu * gpConfig->player_decay;
        if (cSpd.GetDistance() > gpConfig->player_speed_max) cSpd.SetDistance(gpConfig->player_speed_max);
        // poloha lopty
        v = BallPosition( i + nTurn, cNewBallSpeed);

        if (cAgent.RelVectorTo( v).GetDistance() < cNewPos.GetDistance())
        {
            break;
        }
        // FIX: posledna poloha
		// v.SetXY(NOT_EXISTING, NOT_EXISTING);
    }
	return v;
}

//------------------------------------------------------
/*! \fn	CAgent *CEstimateLocalSituation::NearestPlayerToBall()
 *  \brief vrati najblizsieho hraca k lopte
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:54:13

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CAgent *CEstimateLocalSituation::NearestPlayerToBall()
{
    TAgentVector *pia = &(m_cWorld.cIdentifiedAgents);

    int i;
    C2DVector toplr, toplr2;
    double mindist;
    CAgent *pnow = NULL;
    CAgent *pret = NULL;

    ESideType myside = m_cWorld.cThis.eSide;

    C2DVector ballpos = GetBallInterceptPosition(
		m_cWorld.cBall.cSpeed, 
		m_cWorld.cThis
	);
    C2DVector toball = m_cWorld.cThis.RelVectorTo(ballpos);

    mindist = MAX_DISTANCE;
    pret = NULL;

    double distplr2ball;

    // medzi identifikovanymi hracmi
	for(i=0; i < pia->size(); i++)
	{
        pnow = pia->at(i);
        // vektor na hraca
        toplr = m_cWorld.cThis.RelVectorTo(pnow->cPosition);

        distplr2ball = (toball - toplr).GetDistance();

		if (distplr2ball < mindist) {
			pret = pnow;
			mindist = distplr2ball;
		}
    }

    return pret;
}

//------------------------------------------------------
/*! \fn	CAgent *CEstimateLocalSituation::NearestOppPlayer()
 *  \brief vrati najblizsieho protihraca 
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:54:04

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CAgent *CEstimateLocalSituation::NearestOppPlayer()
{
    TAgentVector *pia = &(m_cWorld.cIdentifiedAgents);

    int i;
	C2DVector plrpos, plrspd;
    C2DVector toplr, toplr2;
    double mindist;
    CAgent *pnow = NULL;
    CAgent *pret = NULL;

    ESideType myside = m_cWorld.cThis.eSide;

    mindist = MAX_DISTANCE;
    pret = NULL;

    double distplr2me;

	for(i=0; i < pia->size(); i++)
	{
        pnow = pia->at(i);
        if (pnow->eSide != myside) {
            // predpokladany vektor na hraca v dalsom kroku
			// todo: pridat rychlost a smer kam bezi
			plrpos = pnow->cPosition;

			toplr = m_cWorld.cThis.RelVectorTo(plrpos);
			
            distplr2me = toplr.GetDistance();

			if (distplr2me < mindist) {
				pret = pnow;
				mindist = distplr2me;
			}
        }
    }
	
    return pret;
}

//------------------------------------------------------
/*! \fn	CAgent *CEstimateLocalSituation::NearestFriendToPos(const C2DVector &pos)
 *  \brief vrati najblizsieho spoluhraca k urcitej pozicii
 *  \param const C2DVector &po
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:54:00

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CAgent *CEstimateLocalSituation::NearestFriendToPos(const C2DVector &pos)
{
    TAgentVector *pia = &(m_cWorld.cIdentifiedAgents);

    int i;
    C2DVector topos, toplr;
    double mindist;
    CAgent *pnow = NULL;
    CAgent *pret = NULL;

    ESideType myside = m_cWorld.cThis.eSide;

    topos = m_cWorld.cThis.RelVectorTo(pos);

    mindist = MAX_DISTANCE;
    pret = NULL;

    double distplr2pos;

    // hladame medzi identifikovanymi hracmi
	for(i=0; i < pia->size(); i++)
	{
        pnow = pia->at(i);
        if (pnow->eSide == myside) {
            // vektor na hraca
            toplr = m_cWorld.cThis.RelVectorTo(pnow->cPosition);

            distplr2pos = (topos - toplr).GetDistance();

			if (distplr2pos < mindist) {
				pret = pnow;
				mindist = distplr2pos;
			}
        }
    }

    return pret;
}

//------------------------------------------------------
/*! \fn	double CEstimateLocalSituation::MinimalDirection(double dDistance)
 *  \brief 
 *  \param double dDistanc
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:53:55

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
double CEstimateLocalSituation::MinimalDirection(double dDistance)
{
    const double dFix = 1.0;
    double dSize = gpConfig->player_size * (1.0 + dFix) / dDistance;

    if(dSize <= 1.0)
		return asin(dSize);

	return 0.5 * PI;
}

//------------------------------------------------------
/*! \fn	double CEstimateLocalSituation::DistToClosestOpponent(const C2DVector &cFriendPos)
 *  \brief 
 *  \param const C2DVector &cFriendPo
 *  \return
 *  \author Radoslav Otipka 
 *  \date 2.5.2002 9:53:50

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
double CEstimateLocalSituation::DistToClosestOpponent(const C2DVector &cFriendPos)
{
	// todo: mozno by bolo dobre neidentifikovaneho playera chapat ako oponenta

	C2DVector tofriend = m_cWorld.cThis.RelVectorTo(cFriendPos);

    int i;
    CAgent *pnow = NULL;
	double dOppDist, dRet = MAX_DISTANCE;

    ESideType myside = m_cWorld.cThis.eSide;

	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();
	TAgentVectorIterator pIter = m_cWorld.cIdentifiedAgents.begin();

	for(; pIter != pLast; pIter ++)
	{
        pnow = (*pIter);
        if (pnow->eSide != myside) {
			dOppDist = (C2DVector(
					m_cWorld.cThis.RelVectorTo(pnow->cPosition)
				) - tofriend).GetDistance();
			if (dOppDist < dRet) dRet = dOppDist;
        }
    }
    return dRet;
}
