//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	Pass.cpp
 *  \brief CPass, trieda implementujuca zakladnu schopnost prihravky
 *  \author Rado Otipka 
 *  \date 10. 12. 2001 18:14:44
 *  \version 

 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

///////////////////////////////CPas/////////////////////////////////////////////
#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//------------------------------------------------------
/*! \fn	CPass::CPass(		
		TSkillParameterVector &cParameters, 
		const int iPriority,
		CWorldModel &cWorld,
		CGameCommandSeq &cActions
	)
 *  \brief Konstruktor
 *  \return
 *  \author Radoslav Otipka 
 *  \date 7.5.2002 12:58:50

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CPass::CPass(
		TSkillParameterVector &cParameters, 
		const int iPriority,
		CWorldModel &cWorld,
		CGameCommandSeq &cActions
	)
:	CBasicSkill(cParameters, BSKILL_PASS, iPriority, cWorld, cActions) 
{}

BEGIN_SKILL_MAP(CPass)
	ADD_SKILL(BSKILL_PASS_TO, PassToPlayer)
END_SKILL_MAP


//------------------------------------------------------
/*! \fn	KPDL_RESULT CPass::FillRelParamMaps(
								const double dDistToReceiver, 
								const double dAngleToReceiver
								) 
 *  \brief Vypln vetory vzdialenosti k jednotlivym protihracom a uhly, 
 *  \brief k jednotlivym protihracom
 *  \return
 *  \author Radoslav Otipka 
 *  \date 7.5.2002 13:01:12

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
KPDL_RESULT CPass::FillRelParamMaps(
								const double dDistToReceiver, 
								const double dAngleToReceiver
								) 
{
	double delta; 

	//if( number == m_cWorld.cThis.iNumber ) return KPDL_SUCCESS; 
	C2DVector posAgent(0,0);
	int agentsCount = m_cWorld.cIdentifiedAgents.size();
	ESideType tMySide = m_cWorld.cThis.eSide;
	for(int i=0; i < agentsCount; i++) 
	{
		CAgent *agent = m_cWorld.cIdentifiedAgents[i];
		///!!! nejaka funkcia sa robila ze je to oponent
		if( agent->eSide != tMySide ) 
		{
			delta = (agent->cPosition - 
						  m_cWorld.cThis.cPosition).GetDistance();
			if(delta > dDistToReceiver) continue;
			m_mRelDist.insert(
				VMapDoubleInt(dDistToReceiver - delta, agent->iNumber)
			);
			delta = dAngleToReceiver - 
					(m_cWorld.cThis.RelVectorTo(agent->cPosition)).GetAngle();
			if(delta >= 0) 
				m_mRelAnglePlus.insert(VMapDoubleInt(delta,agent->iNumber));
			else
				//su tam vlozene znegovane zaporne hodnoty
				m_mRelAngleMinus.insert(VMapDoubleInt(-delta,agent->iNumber));
		}
	}		
	return KPDL_SUCCESS;
}



//------------------------------------------------------
/*! \fn	KPDL_RESULT CPass::SayPassMessage(const double dAngle)
 *  \brief Zkric spravu, ktoremu hracovi sa ma nahrat
 *  \param const double dAngl
 *  \return
 *  \author Radoslav Otipka 
 *  \date 7.5.2002 15:55:55

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
KPDL_RESULT CPass::SayPassMessage(const double dAngle)
{
	static char sBuffer[MAX_SAY_STRING_SIZE];
	sprintf(sBuffer,"nahravka: %d %.2f",m_iPlayerNumber,dAngle);
	CSay(
		BSKILL_SAY_IT,
		string(sBuffer),
		BSKILL_NO_PRIORITY, 
		m_cWorld, 
		m_cActions
	).Execute();
	return KPDL_SUCCESS;
}

//------------------------------------------------------
/*! \fn	KPDL_RESULT CPass::PassToPlayer()
 *  \brief Prihra spoluhracovi loptu, 
 *  \brief tak aby uhol prihravky bol 
 *  \brief od protihracov priblizne rovnaky
 *  \return
 *  \author Radoslav Otipka 
 *  \date 7.5.2002 13:04:26

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
KPDL_RESULT CPass::PassToPlayer()
{
	int number = m_iPlayerNumber;
	if( number == m_cWorld.cThis.iNumber ) return KPDL_SUCCESS; 
	C2DVector posAgent(0,0);
	int agentsCount = m_cWorld.cIdentifiedAgents.size();
	ESideType tMySide = m_cWorld.cThis.eSide;
	for(int i=0; i < agentsCount; i++) 
	{
		CAgent *agent = m_cWorld.cIdentifiedAgents[i];
		if( agent->eSide == tMySide ) 
		{
			if( agent->iNumber == number ) 
			{
				posAgent = agent->cPosition;
				break;
			}
		}
	}
	//svojho agenta s takym cislom nevidim 
	if( i == agentsCount ) return KPDL_SUCCESS; 
	//natocim sa ked som v prilis velkom uhle
	

	FillRelParamMaps((m_cWorld.cThis.cPosition - posAgent).GetDistance(),
					 (m_cWorld.cThis.RelVectorTo(posAgent)).GetAngle()	
	);
	
	IMapDoubleInt itAnglePlus = m_mRelAnglePlus.begin();
	IMapDoubleInt itAngleMinus = m_mRelAnglePlus.begin();
	if(itAnglePlus->first <= DegToRad(MIN_PASS_ANGLE) || 
		itAngleMinus->first <= DegToRad(MIN_PASS_ANGLE))
		return KPDL_SUCCESS;
	if(itAnglePlus->first >= DegToRad(MAX_PASS_ANGLE)
		&& itAngleMinus->first >= DegToRad(MAX_PASS_ANGLE)
	)
		m_dAngle = 0;		
	else 
		m_dAngle = -itAngleMinus->first+
			((itAnglePlus->first + itAngleMinus->first) /2.0);
	SayPassMessage(m_dAngle);
	//skusit doplnit kruznicu, teda viacero bodov

	CKick(
		BSKILL_KICK_WITH_ACCEL, 
		0, 
		m_dAngle, 
		BSKILL_NO_PRIORITY, 
		m_cWorld, 
		m_cActions
	).Execute();
	return KPDL_SUCCESS;
}


/*KPDL_RESULT CPass::PassToPlayer() 
{
	return Pass2();
  int number = m_iPlayerNumber;

	if( number == m_cWorld.cThis.iNumber ) return KPDL_SUCCESS; 
	C2DVector posAgent(0,0);
	int agentsCount = m_cWorld.cIdentifiedAgents.size();
	ESide tMySide = m_cWorld.cThis.eSide;
	for(int i=0; i < agentsCount; i++) 
	{
		CAgent *agent = m_cWorld.cIdentifiedAgents[i];
		if( agent->eSide == tMySide ) 
		{
			if( agent->iNumber == number ) 
			{
				posAgent = agent->cPosition;
				break;
			}
		}
	}
	if( i == agentsCount ) return KPDL_SUCCESS; //svojho agenta s takym cislom nevidim 
	else  
	{
		
		CKick(
			BSKILL_KICK_TO_POSITION, 
			0, 
			posAgent, 
			BSKILL_NO_PRIORITY, 
			m_cWorld, 
			m_cActions
		).Execute();
	}
	return KPDL_SUCCESS;

} */
