//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	Types.h
 *  \brief  globalne fukcie pouzivane v module zakladnych schopnosti
 *  \author KPDL
 *  \date 17.4.2002 21:57:01
 *  \version 
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#ifndef _TYPES_
#define _TYPES_


//------------------------------------------------------
/*! \fn	inline double Abs(const double dValue)
 *  \brief Absolutna hodnota
 *  \param const double dValu
 *  \return
 *  \author Radoslav Otipka 
 *  \date 7.5.2002 15:58:58
 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
inline double Abs(const double dValue)
{
	return (dValue >= 0.0) ? dValue : -dValue;
}

//------------------------------------------------------
/*! \fn	inline int Round(const double dValue)
 *  \brief Zaokruhlenie cisla
 *  \param const double dValu
 *  \return
 *  \author Radoslav Otipka 
 *  \date 7.5.2002 15:59:11

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
inline int Round(const double dValue)
{
	return (int) ((dValue >= 0.0) ? dValue + 0.5 : dValue - 0.5);
}

//------------------------------------------------------
/*! \fn	inline int Lcm(const int iValue1, const int iValue2)
 *  \brief Vrati najvacsiehi spolocneho delitela
 *  \param const int iValue1
 *  \param  const int iValue
 *  \return
 *  \author Radoslav Otipka 
 *  \date 7.5.2002 16:01:53

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
inline int Lcm(const int iValue1, const int iValue2)
{
	int iLarger = max(iValue1, iValue2);
	int iResult = iLarger;

	while(iResult % iValue1 != 0 || iResult % iValue2 != 0)
		iResult += iLarger;

	return iResult;
}

// standardne navratove hodnoty funkcii
enum KPDL_RESULT
{
	KPDL_ERROR = -1,
	KPDL_SUCCESS,
	KPDL_UNSUCCESS,
	KPDL_CANCEL
};

// dalsie typy
typedef int TSystemTime;

// standarnde hodnoty
#define PI                          3.14159265359
// neplatne hodnoty
#define INVALID_TIME	            -1
#define INVALID_COUNT               -1
#define INVALID_ANGLE               1000.0
#define INVALID_AMOUNT              -1.0
#define INVALID_COORD               1000.0

// testy na platne hodnoty
#define IsValidTime(iTime) (iTime > INVALID_TIME)
#define IsValidCount(iCount) (iCount > INVALID_COUNT)
#define IsValidAngle(dAngle) (Abs(dAngle) < INVALID_ANGLE)
#define IsValidAmount(dAmount) (dAmount >= 0.0)
#define IsValidCoord(dCoord) (Abs(dCoord) < INVALID_COORD)

// praca s uhlami
#define RadToDeg(dAngle) ((dAngle) * 180.0 / PI)
#define DegToRad(dAngle) ((dAngle) * PI / 180.0)

inline double AngNormalise(const double dAngle)
{
	if(!IsValidAngle(dAngle))
		return INVALID_ANGLE;

	int iPeriod = Round(dAngle / (2.0 * PI));
	return dAngle - iPeriod * 2 * PI;

/*! t0n0 - stara verzia
	while(dAngle < -PI)
		dAngle += 2 * PI;
	while(dAngle > PI)
		dAngle -= 2 * PI;

	return dAngle;
*/
}

#define QUERY_NOT_AVAILABLE "N/A"
#define QUERY_MAX_BUFFER    30

#endif  // _TYPES_
