//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	CommunicationMain.h
 *  \brief Hlavny hlavickovy subor modulu Communication
 *  \author KPDL
 *  \date
 *  \version
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#ifndef _COMMUNICATION_MAIN_
#define _COMMUNICATION_MAIN_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#pragma warning (disable : 4786)

/////////////////////////////////////////////////////////////////////
// External libraries
#include "logmain.h"
#include "eventmain.h"

/////////////////////////////////////////////////////////////////////
// Platform dependent header
#define VC_EXTRALEAN
#include <afx.h>
#include <afxcmn.h>
#include <afxsock.h>
#include <mmsystem.h>

/////////////////////////////////////////////////////////////////////
// Standard header files
#include <math.h>
#include <vector>
#include <queue>
#include <algorithm>
#include <string>
#include <map>

using namespace std;

/////////////////////////////////////////////////////////////////////
// Library settings

#define ANGLE(dAngle) (DegToRad(dAngle))
#define SEND_ANGLE(dAngle) (RadToDeg(dAngle))

#define COMMUNICATE_MAX_BUFFER      65467 //! t0n0: v serveri je nastavena maximalna dlzka spravy 8192 bytov ??

enum EPlayModeType
{
	PM_None = -1,
	PM_BeforeKickOff,
	PM_TimeOver,
	PM_PlayOn,
	PM_KickOff_Home,
	PM_KickOff_Opponent,
	PM_KickIn_Home,
	PM_KickIn_Opponent,
	PM_FreeKick_Home,
	PM_FreeKick_Opponent,
	PM_CornerKick_Home,
	PM_CornerKick_Opponent,
	PM_GoalKick_Home,
	PM_GoalKick_Opponent,
	PM_AfterGoal_Home,
	PM_AfterGoal_Opponent,
	PM_DropBall,
	PM_OffSide_Home,
	PM_OffSide_Opponent,
	PM_PK_Home,
	PM_PK_Opponent,
	PM_FirstHalfOver,
	PM_Pause,
	PM_Human,
	PM_Foul_Charge_Home,
	PM_Foul_Charge_Opponent,
	PM_Foul_Push_Home,
	PM_Foul_Push_Opponent,
	PM_Foul_MultipleAttacker_Home,
	PM_Foul_MultipleAttacker_Opponent,
	PM_Foul_BallOut_Home,
	PM_Foul_BallOut_Opponent,	
	PM_Count
};

#define IsStopedPlayMode(ePlayMode) \
	((ePlayMode == PM_BeforeKickOff) || (ePlayMode == PM_OffSide_Home) || \
	(ePlayMode == PM_OffSide_Opponent) || (ePlayMode == PM_None))

enum EMsgType
{
    M_None = -1,
    M_See,
    M_Hear,
    M_SenseBody,
    M_Init,
	M_Count
};

enum EFlagType
{
    F_None = -1,
    F_Center,
    F_CenterTop,
    F_CenterBottom,
    F_Home0,
    F_HomeTop,
    F_HomeTop10,
    F_HomeTop20,
    F_HomeTop30,
    F_HomeBottom,
    F_HomeBottom10,
    F_HomeBottom20,
    F_HomeBottom30,
    F_Opponent0,
    F_OpponentTop,
    F_OpponentTop10,
    F_OpponentTop20,
    F_OpponentTop30,
    F_OpponentBottom,
    F_OpponentBottom10,
    F_OpponentBottom20,
    F_OpponentBottom30,
    F_PenaltyHomeTop,
    F_PenaltyHomeCenter,
    F_PenaltyHomeBottom,
    F_PenaltyOpponentTop,
    F_PenaltyOpponentCenter,
    F_PenaltyOpponentBottom,
    F_GoalHomeTop,
    F_GoalHomeBottom,
    F_GoalOpponentTop,
    F_GoalOpponentBottom,
    F_Top0,
    F_TopHome10,
    F_TopHome20,
    F_TopHome30,
    F_TopHome40,
    F_TopHome50,
    F_TopOpponent10,
    F_TopOpponent20,
    F_TopOpponent30,
    F_TopOpponent40,
    F_TopOpponent50,
    F_Bottom0,
    F_BottomHome10,
    F_BottomHome20,
    F_BottomHome30,
    F_BottomHome40,
    F_BottomHome50,
    F_BottomOpponent10,
    F_BottomOpponent20,
    F_BottomOpponent30,
    F_BottomOpponent40,
    F_BottomOpponent50,
	F_Count
};

enum ELineType
{
    L_None = -1,
    L_Home,
    L_Opponent,
    L_Top,
    L_Bottom,
	L_Count
};

enum EGoalType
{
	G_None = -1,
	G_Home,
	G_Opponent,
	G_Count
};

enum EHearMsgType
{
    HM_None = -1,
    HM_Referee,
    HM_Self,
	HM_OnlineCoachHome,
	HM_OnlineCoachOpponent,
    HM_Other,
	HM_Count
};

enum EViewQualityType
{
    VQ_None = -1,
    VQ_Low,
    VQ_High,
	VQ_Count
};

enum EViewWidthType
{
    VW_None = -1,
    VW_Narrow,
    VW_Normal,
    VW_Wide,
	VW_Count
};

enum ESideType
{
    S_None = -1,
	S_Home,
	S_Opponent,
	S_Count
};

enum ERealSideType
{
	RS_None = -1,
	RS_Left,
	RS_Right,
	RS_Count
};

enum EGameCommandType
{
	GC_None = -1,
	GC_Dash,
	GC_Turn,
	GC_TurnNeck,
	GC_Catch,
	GC_Kick,
	GC_Move,
	GC_ChangeView,
	GC_Say,
	GC_Init,		// zatial nevyuzivame
	GC_Bye,			//
	GC_ReConnect,	//
	GC_SenseBody,	//
	GC_Count	
};

/////////////////////////////////////////////////////////////////////
// Library header files
#include "config.h"
#include "types.h"
#include "gametime.h"
#include "systemclock.h"

#include "parsestring.h"
#include "gamecommand.h"
#include "gamecommandseq.h"
#include "servermsg.h"
#include "translator.h"

#include "seenobject.h"
#include "seenflag.h"
#include "seengoal.h"
#include "seenline.h"
#include "seenplayer.h"
#include "perception.h"
#include "initmsg.h"
#include "hearmsg.h"
#include "seemsg.h"
#include "sensebody.h"

#include "actevent.h"
#include "receiveevent.h"
#include "seeevent.h"
#include "sensebodyevent.h"
#include "initevent.h"
#include "hearevent.h"

#include "connection.h"
#include "communicator.h"

/////////////////////////////////////////////////////////////////////
// Global objects

extern CLog *gpConnectionLog;

#endif // _COMMUNICATION_MAIN_