//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	Communicator.cpp
 *  \brief Implementacia triedy komunikatorov
 *  \author KPDL
 *  \date
 *  \version 
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

BEGIN_EVENT_MAP(CCommunicator)
	ADD_EVENT(E_SrvConnect, OnConnect)
	ADD_EVENT(E_SrvDisconnect, OnDisconnect)
//	ADD_EVENT(E_ReceiveSrvMsg, OnReceiveMsg)
	ADD_EVENT(E_Act, OnAct)
END_EVENT_MAP

//******************************************************
/*! \fn	CCommunicator::CCommunicator(CCoordinator &cCoordinator)
 *  \brief Konstruktor triedy
 *  \param CCoordinator &cCoordinator
 *  \return
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
CCommunicator::CCommunicator(
	CCoordinator &cCoordinator
)
:	CEventThread(cCoordinator),
	m_cConnection(m_cSystemClock, *this), 
	m_cTranslator(),
	m_cSystemClock(),
	m_cGameTime(),
	m_iForceSystemTime(INVALID_TIME),
    m_bSynchWasStop(false)
{
}

//******************************************************
/*! \fn	KPDL_RESULT CCommunicator::OnConnect(CEvent &cEvent)
 *  \brief Spracovanie udalosti na pripojenie k serveru
 *	\param CEvent &cEvent
 *  \return KPDL_SUCCESS / KPDL_ERROR
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
KPDL_RESULT CCommunicator::OnConnect(CEvent &cEvent) 
{
    m_cSystemClock.Initialize();
	return m_cConnection.Connect(
		gpConfig->app_connect_server, 
		gpConfig->app_connect_port
	);
}

//******************************************************
/*! \fn	KPDL_RESULT CCommunicator::OnDisconnect(CEvent &cEvent)
 *  \brief Spracovanie udalosti na odpojenie od servera
 *	\param CEvent &cEvent
 *  \return KPDL_SUCCESS
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
KPDL_RESULT CCommunicator::OnDisconnect(CEvent &cEvent) 
{
	m_cConnection.Close();
	return KPDL_SUCCESS;
}

//******************************************************
/*! \fn	KPDL_RESULT CCommunicator::OnReceiveMsg(CEvent &cEvent)
 *  \brief Spracovanie udalosti na prijem spravy od servera
 *	\param CEvent &cEvent
 *  \return KPDL_SUCCESS
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
KPDL_RESULT CCommunicator::OnReceiveMsg(CEvent &cEvent) 
{
	CPerception *pPerception = m_cTranslator.ParseServerMsg(
		*((CReceiveEvent &)cEvent).m_pSrvMsg
	);

	if(pPerception != NULL)
	{
		UpdateGameTime(*pPerception);

		CEvent *pEvent = 
			(pPerception->eMsgType == M_See) ? 
				(CEvent *) new CSeeEvent((CSeeMsg *)pPerception) :
			(pPerception->eMsgType == M_SenseBody) ? 
				(CEvent *) new CSenseBodyEvent((CSenseBody *)pPerception) :
			(pPerception->eMsgType == M_Init) ?
				(CEvent *) new CInitEvent((CInitMsg *)pPerception) :
			(pPerception->eMsgType == M_Hear) ?
				(CEvent *) new CHearEvent((CHearMsg *)pPerception) :
			NULL;

		if(pEvent != NULL)
		{
			GenerateEvent(pEvent);
			pEvent->Release();
		}
	}

	return KPDL_SUCCESS;
}

//******************************************************
/*! \fn	KPDL_RESULT CCommunicator::OnAct(CEvent &cEvent)
 *  \brief Spracovanie udalosti na poslanie spravy serveru
 *	\param CEvent &cEvent
 *  \return KPDL_SUCCESS
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
KPDL_RESULT CCommunicator::OnAct(CEvent &cEvent) 
{
	CGameCommand *pSendCommand;
    CGameCommandSeq &cSendCommandSeq = ((CActEvent &)cEvent).cActions;
	CServerMsg cServerMsg;

    while(cSendCommandSeq.GetGameCommand(&pSendCommand) == KPDL_SUCCESS)
	{
    	// skontrolujeme, ci je povel urceny pre aktualny sim. krok
		//! radoK - docasne riesenie 
        //if(m_cGameTime.iPlayTime == pSendCommand->cGameTime)
		//{
			//! vygeneruje servermessage, ktory potom posle m_cConnection
			//! rado o.
			m_cTranslator.GenerateServerMsg(
				*pSendCommand, 
				cServerMsg
			);
			
			m_cConnection.Send(cServerMsg);
		//}
    }

	return KPDL_SUCCESS;
}

//******************************************************
/*! \fn	CCommunicator::UpdateGameState(const CPerception &cPerception)
 *  \brief Upravi stav hry
 *  \param const CPerception &cPerception
 *  \return
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
void CCommunicator::UpdateGameTime(const CPerception &cPerception)
{
	int iStopCycles = 0;

    // nastavi hraci mod podla vnemu 
    if(cPerception.eMsgType == M_Init)
    	m_cGameTime.ePlayMode = ((CInitMsg &)cPerception).ePlayMode;
    else if(cPerception.eMsgType == M_Hear) 
    	m_cGameTime.ePlayMode = ((CHearMsg &)cPerception).ePlayMode;

	if(IsStopedPlayMode(m_cGameTime.ePlayMode)){

		if(cPerception.iPlayTime == m_cGameTime.iPlayTime){

			TSystemTime tForceTime = 
				cPerception.iReceivedTime - m_iForceSystemTime;

			if((cPerception.eMsgType == M_SenseBody) || 
				(tForceTime > 3 * gpConfig->simulator_step))
			{
				iStopCycles = m_cGameTime.iStopCycles + 1;
				m_iForceSystemTime = cPerception.iReceivedTime;
			} 
			else
				iStopCycles = m_cGameTime.iStopCycles;
		}
    } 
	else{        

        if(cPerception.iPlayTime == m_cGameTime.iPlayTime){
			if(m_bSynchWasStop)
				iStopCycles = m_cGameTime.iStopCycles + 1;
			else
				iStopCycles = m_cGameTime.iStopCycles;
        }
	}

	m_cGameTime.iAllCycles += (m_cGameTime.iPlayTime == cPerception.iPlayTime) ? 
		(iStopCycles - m_cGameTime.iStopCycles) : 
		(cPerception.iPlayTime - m_cGameTime.iPlayTime);
	m_cGameTime.iPlayTime = cPerception.iPlayTime;
    m_cGameTime.iStopCycles = iStopCycles;
    
	m_bSynchWasStop = IsStopedPlayMode(m_cGameTime.ePlayMode);
}