//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	Connection.cpp
 *  \brief Implementacia triedy spojovacov so serverom
 *  \author KPDL
 *  \date
 *  \version 
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CLog *gpConnectionLog = NULL;

//******************************************************
/*! \fn	CConnection::CConnection(CSystemClock &cSystemClock, CCommunicator &cCommunicator)
 *  \brief Konstruktor triedy
 *  \param CSystemClock &cSystemClock
 *  \param CCommunicator &cCommunicator
 *  \return
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
CConnection::CConnection(
	CSystemClock &cSystemClock,
	CCommunicator &cCommunicator
)
:	m_cSystemClock(cSystemClock),
	m_cCommunicator(cCommunicator)
{
}

//******************************************************
/*! \fn	KPDL_RESULT CConnection::Connect(const string &sHostAddress, const int iHostPort)
 *  \brief Pripojenie sa k serveru
 *  \param const string &sHostAddress
 *  \param const int iHostPort
 *  \return KPDL_SUCCESS / KPDL_ERROR
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
KPDL_RESULT CConnection::Connect(
	const string &sHostAddress, 
	const int iHostPort
)
{
	if(m_hSocket != INVALID_SOCKET || !Create(0, SOCK_DGRAM, FD_READ))
		return KPDL_ERROR;

	memset(&m_tSockAddr, 0, sizeof(m_tSockAddr));
	m_tSockAddr.sin_port = htons((u_short) iHostPort);
	m_tSockAddr.sin_family = AF_INET;
	m_tSockAddr.sin_addr.s_addr = inet_addr(sHostAddress.c_str());

	if(m_tSockAddr.sin_addr.s_addr == INADDR_NONE)
	{
		LPHOSTENT lpHost = gethostbyname(sHostAddress.c_str());
		if(lpHost == NULL)
		{
			WSASetLastError(WSAEINVAL);
			return KPDL_ERROR;
		}

		m_tSockAddr.sin_addr.s_addr = ((LPIN_ADDR) lpHost->h_addr)->s_addr;
	}

	string sInit = 
		string("(init ") + gpConfig->app_team_name +
		string(" (version ") + gpConfig->app_protocol + string(")");

	if(gpConfig->app_goalie)
		sInit += string(" (goalie)");

	sInit += string(")");
    return Send(CServerMsg(sInit));
}

//******************************************************
/*! \fn	KPDL_RESULT CConnection::Send(CServerMsg &cMsg)
 *  \brief Poslanie spravy serveru
 *  \param CServerMsg &cMsg
 *  \return KPDL_SUCCESS / KPDL_ERROR
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
KPDL_RESULT CConnection::Send(CServerMsg &cMsg)
{
   	if(m_hSocket == INVALID_SOCKET)
		return KPDL_ERROR;

	// pre odosielanu spravu si zapamatame cas odoslania
	cMsg.iReceivedTime = m_cSystemClock.GetTime();

	int iMsgSize = cMsg.sMessage.size();
	int iSentSize = CAsyncSocket::SendTo(
		cMsg.sMessage.c_str(), 
		iMsgSize, 
		(SOCKADDR*) &m_tSockAddr, 
		sizeof(m_tSockAddr)
	);
	
	return (iMsgSize == iSentSize) ? KPDL_SUCCESS : KPDL_ERROR;
}

//******************************************************
/*! \fn	void CConnection::OnReceive(int iErrorCode)
 *  \brief Prijatie spravy od servera
 *  \param int iErrorCode
 *  \return
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
void CConnection::OnReceive(int iErrorCode) 
{
	char sBuffer[COMMUNICATE_MAX_BUFFER];
	int	iSockAddrLen = sizeof(m_tSockAddr);
	int	iResult = CAsyncSocket::ReceiveFrom(
		(void *)sBuffer, 
		COMMUNICATE_MAX_BUFFER, 
		(SOCKADDR*) &m_tSockAddr,
		&iSockAddrLen
	);
	
	if(iResult == SOCKET_ERROR)
		return;

	sBuffer[iResult] = '\0';
	
	/*
	CEvent *pEvent = new CReceiveEvent(sBuffer, m_cSystemClock.GetTime());
	m_cCommunicator.GenerateEvent(pEvent);
	pEvent->Release();
	*/
	
	m_cCommunicator.OnReceiveMsg(
		CReceiveEvent(sBuffer, m_cSystemClock.GetTime())
	);
}