//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	SenseBody.cpp
 *  \brief Implementacia triedy vnemov typu "sense body"
 *  \author KPDL
 *  \date
 *  \version 
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//******************************************************
/*! \fn	CSenseBody::CSenseBody() 
 *  \brief Konstruktor triedy
 *  \return
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
CSenseBody::CSenseBody() 
:	CPerception(M_SenseBody),
	eViewQuality(VQ_None),
    dViewWidth(INVALID_ANGLE),
	dStamina(INVALID_AMOUNT), 
	dEffort(INVALID_AMOUNT), 
	dSpeed(INVALID_AMOUNT),
	dSpeedDirection(INVALID_ANGLE),
	dHeadDirection(INVALID_ANGLE),
	iKickCount(INVALID_COUNT),
	iDashCount(INVALID_COUNT),
	iTurnCount(INVALID_COUNT),
	iSayCount(INVALID_COUNT),
	iHeadTurnCount(INVALID_COUNT),
	iCatchCount(INVALID_COUNT),
	iMoveCount(INVALID_COUNT),
	iChangeViewCount(INVALID_COUNT)
{
}

//******************************************************
/*! \fn	CSenseBody::CSenseBody(const CSenseBody &cSource) 
 *  \brief Kopirovaci konstruktor triedy
 *  \param const CSenseBody &cSource
 *  \return
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
CSenseBody::CSenseBody(const CSenseBody &cSource) 
:	CPerception(cSource),
	eViewQuality(cSource.eViewQuality),
    dViewWidth(cSource.dViewWidth),
	dStamina(cSource.dStamina), 
	dEffort(cSource.dEffort), 
	dSpeed(cSource.dSpeed),
	dSpeedDirection(cSource.dSpeedDirection),
	dHeadDirection(cSource.dHeadDirection),
	iKickCount(cSource.iKickCount),
	iDashCount(cSource.iDashCount),
	iTurnCount(cSource.iTurnCount),
	iSayCount(cSource.iSayCount),
	iHeadTurnCount(cSource.iHeadTurnCount),
	iCatchCount(cSource.iCatchCount),
	iMoveCount(cSource.iMoveCount),
	iChangeViewCount(cSource.iChangeViewCount)
{
}

//******************************************************
/*! \fn	KPDL_RESULT CSenseBody::Parse(CTranslator &cTranslator, CParseString &cString)
 *  \brief Parsuje vnem
 *  \param CTranslator &cTranslator
 *  \param CParseString &cString
 *  \return KPDL_SUCCESS
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
KPDL_RESULT CSenseBody::Parse(
	CTranslator &cTranslator, 
	CParseString &cString
)
{
	CParseString cWord;
	CParseString cBar;

	if(cString.GetWord(cWord) == KPDL_SUCCESS)
		iPlayTime = cWord.GetInteger();

	while(cString.GetBar(cBar) == KPDL_SUCCESS)
	{
		if(cBar.GetWord(cWord) != KPDL_SUCCESS)
			continue;

		if(cWord == "view_mode")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				eViewQuality = cTranslator.CountViewQualityType(cWord);

			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				dViewWidth = cTranslator.CountViewWidth(cWord);
		}
		else if(cWord == "stamina")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				dStamina = cWord.GetDouble();

			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				dEffort = cWord.GetDouble();
		}
		else if(cWord == "speed")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				dSpeed = cWord.GetDouble();

			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				dSpeedDirection = ANGLE(cWord.GetDouble());
		}
		else if(cWord == "head_angle")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				dHeadDirection = ANGLE(cWord.GetDouble());
		}
		else if(cWord == "kick")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				iKickCount = cWord.GetInteger();
		}
		else if(cWord == "dash")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				iDashCount = cWord.GetInteger();
		}
		else if(cWord == "turn")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				iTurnCount = cWord.GetInteger();
		}
		else if(cWord == "say")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				iSayCount = cWord.GetInteger();
		}
		else if(cWord == "turn_neck")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				iHeadTurnCount = cWord.GetInteger();
		}
		else if(cWord == "catch")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				iCatchCount = cWord.GetInteger();
		}
		else if(cWord == "move")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				iMoveCount = cWord.GetInteger();
		}
		else if(cWord == "change_view")
		{
			if(cBar.GetWord(cWord) == KPDL_SUCCESS)
				iChangeViewCount = cWord.GetInteger();
		}
	}

	return KPDL_SUCCESS;
}

//******************************************************
/*! \fn	const CSenseBody &CSenseBody::operator =(const CSenseBody &cSource)
 *  \brief Operator priradenia
 *  \param const CSenseBody &cSource
 *  \return Referenciu na aktualny objekt
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
const CSenseBody &CSenseBody::operator =(const CSenseBody &cSource)
{
	CPerception::operator =(cSource);

	eViewQuality = cSource.eViewQuality;
	dViewWidth = cSource.dViewWidth;
	dStamina = cSource.dStamina;
	dEffort = cSource.dEffort;
	dSpeed = cSource.dSpeed;
	dSpeedDirection = cSource.dSpeedDirection;
	dHeadDirection = cSource.dHeadDirection;
	iKickCount = cSource.iKickCount; 
	iDashCount = cSource.iDashCount; 
	iTurnCount = cSource.iTurnCount; 
	iSayCount = cSource.iSayCount; 
	iHeadTurnCount = cSource.iHeadTurnCount;
	iCatchCount = cSource.iCatchCount;
	iMoveCount = cSource.iMoveCount;
	iChangeViewCount = cSource.iChangeViewCount;

	return *this;
}