//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	Translator.cpp
 *  \brief Implementacia triedy prekladacov sprav a prikazov hraca
 *  \author KPDL
 *  \date
 *  \version 
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//******************************************************
/*! \fn	CTranslator::CTranslator()
 *  \brief Konstruktor triedy
 *  \return
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
CTranslator::CTranslator()
:	m_eRealSide(RS_None),
	m_sTeamName(gpConfig->app_team_name)
{
}

//******************************************************
/*! \fn	CPerception *CTranslator::ParseServerMsg(const CServerMsg &cServerMsg)
 *  \brief Spravu od servera preklada na vnem (alokuje pamat pomocou "new" !)
 *  \param const CServerMsg &cServerMsg
 *  \return Ukazovatel na vytvoreny vnem
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
CPerception *CTranslator::ParseServerMsg(
	const CServerMsg &cServerMsg
)
{
	CParseString cParseString(cServerMsg.sMessage);
	CParseString cWord;

	if(cParseString.GetWord(cWord) != KPDL_SUCCESS)
		return NULL;

	CPerception *pPerception = 
		(cWord == "see") ? (CPerception *) new CSeeMsg() :
		(cWord == "sense_body") ? (CPerception *) new CSenseBody() :
		(cWord == "hear") ? (CPerception *) new CHearMsg() :
		(cWord == "init") ? (CPerception *) new CInitMsg() :
		NULL;

	if(pPerception != NULL)
	{
		// zanalyzuje spravu
		pPerception->Parse(*this, cParseString);

		// a ulozi cas jej prichodu
		pPerception->iReceivedTime = cServerMsg.iReceivedTime;
	}

    return pPerception;
}

//******************************************************
/*! \fn	KPDL_RESULT CTranslator::GenerateServerMsg(const CGameCommand &cGameCommand, CServerMsg &cServerMsg)
 *  \brief Z prikazu hraca generuje spravu pre server
 *  \param const CGameCommand &cGameCommand
 *  \param CServerMsg &cServerMsg
 *  \return KPDL_SUCCESS / KPDL_UNSUCCESS
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
#include "logmain.h"

KPDL_RESULT CTranslator::GenerateServerMsg(
	const CGameCommand &cGameCommand, 
	CServerMsg &cServerMsg
)
{
    char sBuffer[64];

    switch (cGameCommand.eGameCommand)
	{
		case GC_Dash: 
			sprintf(sBuffer, "(dash %.2f)", cGameCommand.dArg1);
			gpBSkillLog->Write(true,"Poslal som dash: %.2f", cGameCommand.dArg1);
			break;

		case GC_Turn: 
			sprintf(sBuffer, "(turn %.2f)", SEND_ANGLE(cGameCommand.dArg1));
			gpBSkillLog->Write(true,"Poslal som turn: %.2f", SEND_ANGLE(cGameCommand.dArg1));
			break;

		case GC_TurnNeck: 
			sprintf(sBuffer, "(turn_neck %.2f)", SEND_ANGLE(cGameCommand.dArg1));
			break;

		case GC_Catch: 
			sprintf(sBuffer, "(catch %.2f)", SEND_ANGLE(cGameCommand.dArg1));
			break;

		case GC_Kick:
			sprintf(
				sBuffer, 
				"(kick %.2f %.2f)",
				cGameCommand.dArg1,
				SEND_ANGLE(cGameCommand.dArg2)
			);
			break;

		case GC_Move:
			if(m_eRealSide == RS_Left)
				sprintf(
					sBuffer, 
					"(move %.2f %.2f)", 
					cGameCommand.dArg1, 
					cGameCommand.dArg2
				);
			else if(m_eRealSide == RS_Right)
				sprintf(
					sBuffer, 
					"(move %.2f %.2f)", 
					- cGameCommand.dArg1, 
					- cGameCommand.dArg2
				);
			else
				return KPDL_UNSUCCESS;
			break;

		case GC_ChangeView:
			sprintf(
				sBuffer, 
				"(change_view %s %s)",
				cGameCommand.sArg1.c_str(),
				cGameCommand.sArg2.c_str()
			);
			break;

		case GC_Say:
			sprintf(sBuffer, "(say %s)", cGameCommand.sArg1.c_str());
			break;

		default:
			return KPDL_UNSUCCESS;	// ak je prikaz ineho typu
    }

	cServerMsg.sMessage = sBuffer;
    return KPDL_SUCCESS;
}

//******************************************************
/*! \fn	ERealSideType CTranslator::CountRealSideType(const char cChar)
 *  \brief Urcenie skutocnej strany
 *  \param const char cChar
 *  \return Skutocnu stranu
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
ERealSideType CTranslator::CountRealSideType(
	const char cChar
)
{
	// stranu, na ktorej hrac hra, si rovno zapametame :)
	// kvoli dalsim vypoctom
	return m_eRealSide = (
		(cChar == 'l') ? RS_Left :
		(cChar == 'r') ? RS_Right :
		RS_None
	);
}

//******************************************************
/*! \fn	EHearMsgType CTranslator::CountHearMsgType(const CParseString &cString)
 *  \brief Urcenie povodcu pocutej spravy
 *  \param const CParseString &cString
 *  \return	Povodcu pocutej spravy
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
EHearMsgType CTranslator::CountHearMsgType(
	const CParseString &cString
)
{
	return
		(cString == "referee") ? HM_Referee :
		(cString == "self") ? HM_Self :
		(m_eRealSide == RS_Left) ? 
		(
			// ked hrac hra vlavo, left je home, right je opponent
			(cString == "online_coach_left") ? HM_OnlineCoachHome :
			(cString == "online_coach_right") ? HM_OnlineCoachOpponent :
			HM_Other	//! alebo HM_None ?
		) :
		(m_eRealSide == RS_Right) ?
		(
			// ked hrac hra vpravo, left je opponent, right je home
			(cString == "online_coach_left") ? HM_OnlineCoachOpponent :
			(cString == "online_coach_right") ? HM_OnlineCoachHome :
			HM_Other	//! alebo HM_None ?
		) :
		HM_Other;		//! alebo HM_None ?
}

//******************************************************
/*! \fn	EViewQualityType CTranslator::CountViewQualityType(const CParseString &cString)
 *  \brief Urcenie kvality videnia
 *  \param const CParseString &cString
 *  \return Kvalitu videnia
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
EViewQualityType CTranslator::CountViewQualityType(
	const CParseString &cString
)
{
	return
		(cString == "low") ? VQ_Low :
		(cString == "high") ? VQ_High :
		VQ_None;
}

//******************************************************
/*! \fn	double CTranslator::CountViewWidth(const CParseString &cString)
 *  \brief Urcenie sirky videnia
 *  \param const CParseString &cString
 *  \return Sirku videnia
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
double CTranslator::CountViewWidth(
	const CParseString &cString
)
{
	double dViewAngle = gpConfig->visible_angle;

	return
		(cString == "narrow") ? 0.5 * dViewAngle : 
		(cString == "normal") ? dViewAngle :
		(cString == "wide") ? 2.0 * dViewAngle :
		INVALID_ANGLE;
}

//******************************************************
/*! \fn	ESideType CTranslator::CountSideType(const CParseString &cString)
 *  \brief Urcenie relativnej strany
 *  \param const CParseString &cString
 *  \return Relativnu stranu
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
ESideType CTranslator::CountSideType(
	const CParseString &cString
)
{
	return
		(cString == m_sTeamName) ? S_Home : 
		(cString == "") ? S_None :
		S_Opponent;
}

//******************************************************
/*! \fn	EFlagType CTranslator::CountFlagTypeCountFlagType(const char cChar1, const char cChar2, const char cChar3)
 *  \brief Urcenie umiestnenia zastavky
 *  \param const char cChar1
 *  \param const char cChar2
 *  \param const char cChar3
 *  \return Umiestnenie zastavky
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
EFlagType CTranslator::CountFlagType(
	const char cChar1, 
	const char cChar2, 
	const char cChar3
)
{
	return
		// ked hra vlavo, left je home, right je opponent,
		// top je top, bottom je bottom
		(m_eRealSide == RS_Left) ?
		(
			(cChar1 == 'c') ?
			(		
				(cChar2 == '\0') ? F_Center :
				(cChar2 == 't') ? F_CenterTop :
				(cChar2 == 'b') ? F_CenterBottom :
				F_None	
			) :
			(cChar1 == 'l') ?
			(
				(cChar2 == 't') ? 
				(
					(cChar3 == '\0') ? F_HomeTop :
					(cChar3 == '1') ? F_HomeTop10 :
					(cChar3 == '2') ? F_HomeTop20 :
					(cChar3 == '3') ? F_HomeTop30 :
					F_None
				) :	
				(cChar2 == 'b') ?
				(
					(cChar3 == '\0') ? F_HomeBottom :
					(cChar3 == '1') ? F_HomeBottom10 :
					(cChar3 == '2') ? F_HomeBottom20 :
					(cChar3 == '3') ? F_HomeBottom30 :
					F_None
				) :
				(cChar2 == '0') ? F_Home0 :
				F_None
			) :
			(cChar1 == 'r') ?
			(
				(cChar2 == 't') ?
				(
					(cChar3 == '\0') ? F_OpponentTop :
					(cChar3 == '1') ? F_OpponentTop10 :
					(cChar3 == '2') ? F_OpponentTop20 :
					(cChar3 == '3') ? F_OpponentTop30 :
					F_None
				) :
				(cChar2 == 'b') ?
				(
					(cChar3 == '\0') ? F_OpponentBottom :
					(cChar3 == '1') ? F_OpponentBottom10 :
					(cChar3 == '2') ? F_OpponentBottom20 :
					(cChar3 == '3') ? F_OpponentBottom30 :
					F_None
				) :
				(cChar2 == '0') ? F_Opponent0 :
				F_None
			) :
			(cChar1 == 't') ?
			(
				(cChar2 == 'l') ? 
				(
					(cChar3 == '1') ? F_TopHome10 :
					(cChar3 == '2') ? F_TopHome20 :
					(cChar3 == '3') ? F_TopHome30 :
					(cChar3 == '4') ? F_TopHome40 :
					(cChar3 == '5') ? F_TopHome50 :
					F_None
				) :
				(cChar2 == 'r') ?
				(
					(cChar3 == '1') ? F_TopOpponent10 :
					(cChar3 == '2') ? F_TopOpponent20 :
					(cChar3 == '3') ? F_TopOpponent30 :
					(cChar3 == '4') ? F_TopOpponent40 :
					(cChar3 == '5') ? F_TopOpponent50 :
					F_None
				) :
				(cChar2 == '0') ? F_Top0 :
				F_None
			) :
			(cChar1 == 'b') ? 
			(
				(cChar2 == 'l') ?
				(
					(cChar3 == '1') ? F_BottomHome10 :
					(cChar3 == '2') ? F_BottomHome20 :
					(cChar3 == '3') ? F_BottomHome30 :
					(cChar3 == '4') ? F_BottomHome40 :
					(cChar3 == '5') ? F_BottomHome50 :
					F_None
				) :
				(cChar2 == 'r') ?
				(
					(cChar3 == '1') ? F_BottomOpponent10 :
					(cChar3 == '2') ? F_BottomOpponent20 :
					(cChar3 == '3') ? F_BottomOpponent30 :
					(cChar3 == '4') ? F_BottomOpponent40 :
					(cChar3 == '5') ? F_BottomOpponent50 :
					F_None
				) :
				(cChar2 == '0') ? F_Bottom0 :
				F_None
			) :
			(cChar1 == 'p') ?
			(
				(cChar2 == 'l') ?
				(
					(cChar3 == 't') ? F_PenaltyHomeTop :
					(cChar3 == 'c') ? F_PenaltyHomeCenter :
					(cChar3 == 'b') ? F_PenaltyHomeBottom :
					F_None
				) :
				(cChar2 == 'r') ?
				(
					(cChar3 == 't') ? F_PenaltyOpponentTop :
					(cChar3 == 'c') ? F_PenaltyOpponentCenter :
					(cChar3 == 'b') ? F_PenaltyOpponentBottom :
					F_None
				) :
				F_None
			) :
			(cChar1 == 'g') ? 
			(
				(cChar2 == 'l') ? 
				(
					(cChar3 == 't') ? F_GoalHomeTop :
					(cChar3 == 'b') ? F_GoalHomeBottom :
					F_None
				) :
				(cChar2 == 'r') ?
				(
					(cChar3 == 't') ? F_GoalOpponentTop : 
					(cChar3 == 'b') ? F_GoalOpponentBottom :
					F_None
				) :
				F_None
			) :
			F_None
		) :
		// ked hra vpravo, left je opponent, right je home, 
		// top je bottom, bottom je top
		(m_eRealSide == RS_Right) ? 
		(
			(cChar1 == 'c') ?
			(		
				(cChar2 == '\0') ? F_Center :
				(cChar2 == 't') ? F_CenterBottom :
				(cChar2 == 'b') ? F_CenterTop :
				F_None	
			) :
			(cChar1 == 'l') ?
			(
				(cChar2 == 't') ? 
				(
					(cChar3 == '\0') ? F_OpponentBottom :
					(cChar3 == '1') ? F_OpponentBottom10 :
					(cChar3 == '2') ? F_OpponentBottom20 :
					(cChar3 == '3') ? F_OpponentBottom30 :
					F_None
				) :	
				(cChar2 == 'b') ?
				(
					(cChar3 == '\0') ? F_OpponentTop :
					(cChar3 == '1') ? F_OpponentTop10 :
					(cChar3 == '2') ? F_OpponentTop20 :
					(cChar3 == '3') ? F_OpponentTop30 :
					F_None
				) :
				(cChar2 == '0') ? F_Opponent0 :
				F_None
			) :
			(cChar1 == 'r') ?
			(
				(cChar2 == 't') ?
				(
					(cChar3 == '\0') ? F_HomeBottom :
					(cChar3 == '1') ? F_HomeBottom10 :
					(cChar3 == '2') ? F_HomeBottom20 :
					(cChar3 == '3') ? F_HomeBottom30 :
					F_None
				) :
				(cChar2 == 'b') ?
				(
					(cChar3 == '\0') ? F_HomeTop :
					(cChar3 == '1') ? F_HomeTop10 :
					(cChar3 == '2') ? F_HomeTop20 :
					(cChar3 == '3') ? F_HomeTop30 :
					F_None
				) :
				(cChar2 == '0') ? F_Home0 :
				F_None
			) :
			(cChar1 == 't') ?
			(
				(cChar2 == 'l') ? 
				(
					(cChar3 == '1') ? F_BottomOpponent10 :
					(cChar3 == '2') ? F_BottomOpponent20 :
					(cChar3 == '3') ? F_BottomOpponent30 :
					(cChar3 == '4') ? F_BottomOpponent40 :
					(cChar3 == '5') ? F_BottomOpponent50 :
					F_None
				) :
				(cChar2 == 'r') ?
				(
					(cChar3 == '1') ? F_BottomHome10 :
					(cChar3 == '2') ? F_BottomHome20 :
					(cChar3 == '3') ? F_BottomHome30 :
					(cChar3 == '4') ? F_BottomHome40 :
					(cChar3 == '5') ? F_BottomHome50 :
					F_None
				) :
				(cChar2 == '\0') ? F_Bottom0 :
				F_None
			) :
			(cChar1 == 'b') ? 
			(
				(cChar2 == 'l') ?
				(
					(cChar3 == '1') ? F_TopOpponent10 :
					(cChar3 == '2') ? F_TopOpponent20 :
					(cChar3 == '3') ? F_TopOpponent30 :
					(cChar3 == '4') ? F_TopOpponent40 :
					(cChar3 == '5') ? F_TopOpponent50 :
					F_None
				) :
				(cChar2 == 'r') ?
				(
					(cChar3 == '1') ? F_TopHome10 :
					(cChar3 == '2') ? F_TopHome20 :
					(cChar3 == '3') ? F_TopHome30 :
					(cChar3 == '4') ? F_TopHome40 :
					(cChar3 == '5') ? F_TopHome50 :
					F_None
				) :
				(cChar2 == '0') ? F_Top0 :
				F_None
			) :
			(cChar1 == 'p') ?
			(
				(cChar2 == 'l') ?
				(
					(cChar3 == 't') ? F_PenaltyOpponentBottom :
					(cChar3 == 'c') ? F_PenaltyOpponentCenter :
					(cChar3 == 'b') ? F_PenaltyOpponentTop :
					F_None
				) :
				(cChar2 == 'r') ?
				(
					(cChar3 == 't') ? F_PenaltyHomeBottom :
					(cChar3 == 'c') ? F_PenaltyHomeCenter :
					(cChar3 == 'b') ? F_PenaltyHomeTop :
					F_None
				) :
				F_None
			) :
			(cChar1 == 'g') ? 
			(
				(cChar2 == 'l') ? 
				(
					(cChar3 == 't') ? F_GoalOpponentBottom :
					(cChar3 == 'b') ? F_GoalOpponentTop :
					F_None
				) :
				(cChar2 == 'r') ?
				(
					(cChar3 == 't') ? F_GoalHomeBottom : 
					(cChar3 == 'b') ? F_GoalHomeTop :
					F_None
				) :
				F_None
			) :
			F_None
		) :
		F_None;
}

//******************************************************
/*! \fn	EGoalType CTranslator::CountGoalType(const char cChar)
 *  \brief Urcenie relativnej strany branky
 *  \param const char cChar
 *  \return Relativnu stranu branky
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
EGoalType CTranslator::CountGoalType(
	const char cChar
)
{
	return
		// ked hra vlavo, left je home, right je opponent !
		(m_eRealSide == RS_Left) ?
		(
			(cChar == 'l') ? G_Home : 
			(cChar == 'r') ? G_Opponent :
			G_None
		) :
		// ked hra vpravo, left je opponent, right je home !
		(m_eRealSide == RS_Right) ?
		(
			(cChar == 'l') ? G_Opponent :
			(cChar == 'r') ? G_Home : 
			G_None
		) :
		G_None;
}

//******************************************************
/*! \fn	ELineType CTranslator::CountLineType(const char cChar)
 *  \brief Urcenie umiestnenia ciary
 *	\param const char cChar
 *  \return Umiestnenie ciary
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
ELineType CTranslator::CountLineType(
	const char cChar
)
{
	return
		// ked hra vlavo, left je home, right je opponent,
		// top je top, bottom je bottom
		(m_eRealSide == RS_Left) ?
		(
			(cChar == 'l') ? L_Home : 
			(cChar == 'r') ? L_Opponent :
			(cChar == 't') ? L_Top :
			(cChar == 'b') ? L_Bottom :
			L_None
		) :
		// ked hra vpravo, left je opponent, right je home, 
		// top je bottom, bottom je top
		(m_eRealSide == RS_Right) ?
		(
			(cChar == 'l') ? L_Opponent :
			(cChar == 'r') ? L_Home : 
			(cChar == 't') ? L_Bottom :
			(cChar == 'b') ? L_Top :
			L_None
		) :
		L_None;
}

//******************************************************
/*! \fn	EPlayModeType CTranslator::CountPlayModeType(const CParseString &cString)
 *  \brief Urcenie modu hry
 *  \param const CParseString &cString
 *  \return Mod hry
 *  \author KPDL
 *  \date
 *  \bug 
 */
//******************************************************
// Historia zmien:
//******************************************************
/*
 */
//******************************************************
EPlayModeType CTranslator::CountPlayModeType(
	const CParseString &cString
)
{
	return 
		(cString == "before_kick_off") ? PM_BeforeKickOff :
		(cString == "time_over") ? PM_TimeOver :
		(cString == "play_on") ? PM_PlayOn :
		(cString == "drop_ball") ? PM_DropBall :
		(cString == "first_half_over") ? PM_FirstHalfOver :
		(cString == "pause") ? PM_Pause :
		(cString == "human_judge") ? PM_Human :
		// ked hra vlavo, left je home, right je opponent
		(m_eRealSide == RS_Left) ?
		(
			(cString == "kick_off_l") ? PM_KickOff_Home :
			(cString == "kick_off_r") ? PM_KickOff_Opponent :
			(cString == "kick_in_l") ? PM_KickIn_Home :
			(cString == "kick_in_r") ? PM_KickIn_Opponent :
			(cString == "free_kick_l") ? PM_FreeKick_Home :
			(cString == "free_kick_r") ? PM_FreeKick_Opponent :
			(cString == "corner_kick_l") ? PM_CornerKick_Home :
			(cString == "corner_kick_r") ? PM_CornerKick_Opponent :
			(cString == "goal_kick_l") ? PM_GoalKick_Home :
			(cString == "goal_kick_r") ? PM_GoalKick_Opponent :
			(cString == "goal_l") ? PM_AfterGoal_Home :
			(cString == "goal_r") ? PM_AfterGoal_Opponent :
			(cString == "offside_l") ? PM_OffSide_Home :
			(cString == "offside_r") ? PM_OffSide_Opponent :
			(cString == "penalty_kick_l") ? PM_PK_Home :
			(cString == "penalty_kick_r") ? PM_PK_Opponent :
			(cString == "foul_charge_l") ? PM_Foul_Charge_Home :
			(cString == "foul_charge_r") ? PM_Foul_Charge_Opponent :
			(cString == "foul_push_l") ? PM_Foul_Push_Home :
			(cString == "foul_push_r") ? PM_Foul_Push_Opponent :
			(cString == "foul_multiple_attact_l") ? PM_Foul_MultipleAttacker_Home :
			(cString == "foul_multiple_attact_r") ? PM_Foul_MultipleAttacker_Opponent :
			(cString == "foul_ballout_l") ? PM_Foul_BallOut_Home :
			(cString == "foul_ballout_r") ? PM_Foul_BallOut_Opponent :
			PM_None
		) :		
		// ked hra vpravo, left je opponent, right je home
		(m_eRealSide == RS_Right) ? 
		(
			(cString == "kick_off_l") ? PM_KickOff_Opponent :
			(cString == "kick_off_r") ? PM_KickOff_Home :
			(cString == "kick_in_l") ? PM_KickIn_Opponent :
			(cString == "kick_in_r") ? PM_KickIn_Home :
			(cString == "free_kick_l") ? PM_FreeKick_Opponent :
			(cString == "free_kick_r") ? PM_FreeKick_Home :
			(cString == "corner_kick_l") ? PM_CornerKick_Opponent :
			(cString == "corner_kick_r") ? PM_CornerKick_Home :
			(cString == "goal_kick_l") ? PM_GoalKick_Opponent :
			(cString == "goal_kick_r") ? PM_GoalKick_Home :
			(cString == "goal_l") ? PM_AfterGoal_Opponent :
			(cString == "goal_r") ? PM_AfterGoal_Home :
			(cString == "offside_l") ? PM_OffSide_Opponent :
			(cString == "offside_r") ? PM_OffSide_Home :
			(cString == "penalty_kick_l") ? PM_PK_Opponent :
			(cString == "penalty_kick_r") ? PM_PK_Home :
			(cString == "foul_charge_l") ? PM_Foul_Charge_Opponent :
			(cString == "foul_charge_r") ? PM_Foul_Charge_Home :
			(cString == "foul_push_l") ? PM_Foul_Push_Opponent :
			(cString == "foul_push_r") ? PM_Foul_Push_Home :
			(cString == "foul_multiple_attact_l") ? PM_Foul_MultipleAttacker_Opponent :
			(cString == "foul_multiple_attact_r") ? PM_Foul_MultipleAttacker_Home :
			(cString == "foul_ballout_l") ? PM_Foul_BallOut_Opponent :
			(cString == "foul_ballout_r") ? PM_Foul_BallOut_Home :
			PM_None 
		) :
		PM_None;

	/*
	// skratene tvary
	return 
		(cString == "bko") ? PM_BeforeKickOff :
		(cString == "to") ? PM_TimeOver :
		(cString == "play") ? PM_PlayOn :
		(cString == "db") ? PM_DropBall :
		(cString == "first_half_over") ? PM_FirstHalfOver : //! nema skrateny tvar
		(cString == "ps") ? PM_Pause :
		(cString == "hj") ? PM_Human :
		// ked hra vlavo, left je home, right je opponent
		(m_eRealSide == RS_Left) ?
		(
			(cString == "kol") ? PM_KickOff_Home :
			(cString == "kor") ? PM_KickOff_Opponent :
			(cString == "kil") ? PM_KickIn_Home :
			(cString == "kir") ? PM_KickIn_Opponent :
			(cString == "fkl") ? PM_FreeKick_Home :
			(cString == "fkr") ? PM_FreeKick_Opponent :
			(cString == "ckl") ? PM_CornerKick_Home :
			(cString == "ckr") ? PM_CornerKick_Opponent :
			(cString == "gkl") ? PM_GoalKick_Home :
			(cString == "gkr") ? PM_GoalKick_Opponent :
			(cString == "gl") ? PM_AfterGoal_Home :
			(cString == "gr") ? PM_AfterGoal_Opponent :
			(cString == "ol") ? PM_OffSide_Home :
			(cString == "or") ? PM_OffSide_Opponent :
			(cString == "penalty_kick_l") ? PM_PK_Home :		//! nema skrateny tvar
			(cString == "penalty_kick_r") ? PM_PK_Opponent :	//! nema skrateny tvar
			(cString == "flcl") ? PM_Foul_Charge_Home :
			(cString == "flcr") ? PM_Foul_Charge_Opponent :
			(cString == "flpl") ? PM_Foul_Push_Home :
			(cString == "flpr") ? PM_Foul_Push_Opponent :
			(cString == "flml") ? PM_Foul_MultipleAttacker_Home :
			(cString == "flmr") ? PM_Foul_MultipleAttacker_Opponent :
			(cString == "flbl") ? PM_Foul_BallOut_Home :
			(cString == "flbr") ? PM_Foul_BallOut_Opponent :
			PM_None
		) :		
		// ked hra vpravo, left je opponent, right je home
		(m_eRealSide == RS_Right) ? 
		(
			(cString == "kol") ? PM_KickOff_Opponent :
			(cString == "kor") ? PM_KickOff_Home :
			(cString == "kil") ? PM_KickIn_Opponent :
			(cString == "kir") ? PM_KickIn_Home :
			(cString == "fkl") ? PM_FreeKick_Opponent :
			(cString == "fkr") ? PM_FreeKick_Home :
			(cString == "ckl") ? PM_CornerKick_Opponent :
			(cString == "ckr") ? PM_CornerKick_Home :
			(cString == "gkl") ? PM_GoalKick_Opponent :
			(cString == "gkr") ? PM_GoalKick_Home :
			(cString == "gl") ? PM_AfterGoal_Opponent :
			(cString == "gr") ? PM_AfterGoal_Home :
			(cString == "ol") ? PM_OffSide_Opponent :
			(cString == "or") ? PM_OffSide_Home :
			(cString == "penalty_kick_l") ? PM_PK_Opponent :	//! nema skrateny tvar
			(cString == "penalty_kick_r") ? PM_PK_Home :		//! nema skrateny tvar
			(cString == "flcl") ? PM_Foul_Charge_Opponent :
			(cString == "flcr") ? PM_Foul_Charge_Home :
			(cString == "flpl") ? PM_Foul_Push_Opponent :
			(cString == "flpr") ? PM_Foul_Push_Home :
			(cString == "flml") ? PM_Foul_MultipleAttacker_Opponent :
			(cString == "flmr") ? PM_Foul_MultipleAttacker_Home :
			(cString == "flbl") ? PM_Foul_BallOut_Opponent :
			(cString == "flbr") ? PM_Foul_BallOut_Home :
			PM_None 
		) :
		PM_None;
		*/
}