//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	StrategicSkills.cpp
 *  \brief Implementacia strategickeho rozhodovania hraca
 *  \author Rado Kostelnik
 *  \date 13.3.2002 21:56:16
 *  \version 

 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define DEFENSE_THRESHOLD -0.75
#define ATTACK_THRESHOLD 0.75


BEGIN_TERM_MAP(CStrategicSkills)
	ADD_TERM("ActionPosition", ActionPosition)
	ADD_TERM("StrategicPosition", StrategicPosition)
	ADD_TERM("playModeChanged", playModeChanged)
	ADD_TERM("playerType", playerType)
END_TERM_MAP


CStrategicSkills::CStrategicSkills(
	CWorldModel& cWorld,
	CBasicSkillList& cBasicSkillList,
	const string &sFileName
)
:	m_cWorld(cWorld),
	m_cBasicSkillList(cBasicSkillList),
	m_cFormationData()
{
	m_cRuleEngine.LoadXML(sFileName);
}

CStrategicSkills::~CStrategicSkills()
{
}

void CStrategicSkills::Deliberate()
{
	TActionVector cActionList;
	string sResult;
	//char sBuffer[100];
	int iPriority = 0;
	TSkillParameterVector cParamList;

	// Novy rozhodovaci cyklus
	gpParserLog->Write(
		true, 
		"------------------------------------------------------------"
	);

	UpdateVariables();

	ClearCache();
	m_cRuleEngine.Evaluate("default", *this, cActionList);

	TActionVectorIterator pActionIter = cActionList.begin();
	TActionVectorIterator pActionEnd = cActionList.end();

	for(; pActionIter != pActionEnd; pActionIter++ )
	{
		CAction *pAction = (*pActionIter);

		//
		// SetFormation
		//
		if(pAction->sName == "SetFormation")
		{
			CRuleParameter& cRuleParam = pAction->cParamList[0];

			m_pFormation = m_cFormationData.GetFormation( cRuleParam.sValue );
		}
		//
		// ClearPlan
		//
		if(pAction->sName == "ClearPlan")
		{
			m_cBasicSkillList.Empty();
		}
	}
}

string CStrategicSkills::GetAttribute(const string &sName)
{
	string sHeader = sName;
	string sTail = CQuery::SplitTail(sHeader);
	
	if(sName == "formation")
		return m_pFormation->GetAttribute(sTail);

//! zrejme stav hry bude ulozeny niekde inde (v modeli sveta ...)
//	if(sName == "state")
//		return string("winning");

	if(sHeader == "strategy")
		return CQuery::Evaluate(sTail);
	
	return m_cWorld.Evaluate(sName);
}

void CStrategicSkills::Variable( const string& sVarName, string& sVarValue )
{
	if ( m_VarMap.find( sVarName ) != m_VarMap.end() )
		sVarValue = m_VarMap[sVarName];
}

void CStrategicSkills::UpdateVariables()
{
	double dFrontLine = m_cWorld.cBall.cPosition.GetX();
	char sBuffer[100];

	sprintf(sBuffer, "%.2f", dFrontLine + 5.0);
	m_VarMap[string("frontLine")] = string(sBuffer);
}

///////////////////////////////////////////////////////
// Termy
///////////////////////////////////////////////////////
string CStrategicSkills::ActionPosition()
{
	return StrategicPosition();
}

//------------------------------------------------------
/*! \fn	string CStrategicSkills::StrategicPosition()
 *  \brief Vypocet strategickej pozicie hraca
 *  \return
 *  \author Rado Kostelnik
 *  \date 8.4.2002 0:05:16

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
string CStrategicSkills::StrategicPosition()
{
	if( m_pFormation == NULL || !IsValidCount(m_cWorld.cThis.iNumber) )
		return string(QUERY_NOT_AVAILABLE);

	// home position
	C2DVector cHomePos, cRelHomePos;
	CTeamPosition* pTeamPos = m_pFormation->GetPosition(m_cWorld.cThis.iNumber);
	double dFrontLine;
	double dFieldLength = m_cWorld.cField.dFieldLength;
	double dFieldWidth  = m_cWorld.cField.dFieldWidth;

	// Vypocet domacej pozicie
	// relativnej (-1,1)
	cRelHomePos = pTeamPos->GetHome();

	// absolutnej vzhladom na velkost ihriska
	cHomePos.SetXY( dFieldLength * cRelHomePos.GetX(),
				    dFieldWidth  * cRelHomePos.GetY() );

	//
	// Staticka formacia
	//
	if ( m_pFormation->sFormationType == "static" )
	{
		dFrontLine = 0.0;
	}
	//
	// Formacia pohybujuca sa s loptou
	//
	else if ( m_pFormation->sFormationType == "ball" )
	{
		C2DVector& cBallPos = m_cWorld.cBall.cPosition;
		C2DVector cRelBallPos( 2.0 * cBallPos.GetX() / dFieldLength, 2.0 * cBallPos.GetY() / dFieldWidth );

		// ak je lopta nebezpecne blizko brany
		if ( cRelBallPos.GetX() < DEFENSE_THRESHOLD )
		{
			cHomePos.SetX( cHomePos.GetX() - pTeamPos->dRadius * dFieldLength );
			if ( cHomePos.GetX() < 0.0 )
				cHomePos.SetX( 0.0 );
		}
		// ak rozohravame
		else if ( cRelBallPos.GetX() < ATTACK_THRESHOLD )
		{
			if ( pTeamPos->GetType() == PT_Forward )
			{
				cHomePos.SetX( cHomePos.GetX() + pTeamPos->dRadius * dFieldLength );
			}
		}
		// ak je lopta nebezpecne blizko brany supera
		else if ( cRelBallPos.GetX() > ATTACK_THRESHOLD )
		{
			cHomePos.SetX( cHomePos.GetX() + pTeamPos->dRadius * dFieldLength );
		}
	}

	return cHomePos.GetAttribute();
}

string CStrategicSkills::playModeChanged()
{
	if ( m_cWorld.cTime.ePlayMode != m_PlayMode )
	{
		m_PlayMode = m_cWorld.cTime.ePlayMode;
		return string("true");
	}

	return string("false");
}

string CStrategicSkills::playerType()
{
	if ( m_pFormation == NULL )
		return QUERY_NOT_AVAILABLE;

	switch( m_pFormation->GetActualPlayerType(m_cWorld.cThis.iNumber) )
	{
	case PT_Goaltender:
		return string("goalie");
	case PT_Defender:
		return string("defender");
	case PT_Midfielder:
		return string("midfielder");
	case PT_Forward:
		return string("forward");
	}

	return string("unknown");
}

void CStrategicSkills::ClearCache()
{
	m_cWorld.ClearCache();
	CQuery::ClearCache();
}
