// FormationData.cpp: implementation of the CFormationData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFormationData::CFormationData()
{
	InitializeFormations();
}

CFormationData::~CFormationData()
{
	TFormationMapIterator fmi = m_formations.begin();
	TFormationMapIterator fme = m_formations.end();

	for ( ; fmi != fme; fmi++ )
		delete (*fmi).second;

	m_formations.clear();
}

KPDL_RESULT CFormationData::InitializeFormations()
{

	KPDL_RESULT kpdlResult = KPDL_SUCCESS;
	
	IXMLDOMDocumentPtr pDom = NULL;
	IXMLDOMNodePtr pNode = NULL;
	IXMLDOMNodeListPtr pChildList = NULL;
	IXMLDOMNodePtr pChild = NULL;
	string sFileName = "formations.xml";

	_variant_t vFileName;
	_variant_t vRetVal;
	CFormation *pFormation;

	if(FAILED(CoInitializeEx(0, COINIT_MULTITHREADED)))
	{
		return KPDL_ERROR;
	}

	try
	{
		pDom.CreateInstance(DOM_DOCUMENT);
		vFileName = _bstr_t((LPCSTR)sFileName.c_str());
		vRetVal = pDom->load(vFileName);
		
		if(!(bool)vRetVal)
			return KPDL_ERROR;

		pNode = pDom->documentElement;
		pChildList = pNode->selectNodes(L"formation");
		pChildList->reset(); 

		while((pChild = pChildList->nextNode()) != NULL)
		{
			pFormation = new CFormation();
			pFormation->SetXML(pChild);
			
			m_formations.insert( TFormationMap::value_type( pFormation->GetName(), pFormation ) );
		}
	}
	catch(_com_error &E)
	{
		gpSystemLog->Write(
			true, 
			"Chyba %08X pri nahravani suboru formacii '%s'",
			E.Error(),
			sFileName.c_str()
		);
		kpdlResult = KPDL_ERROR;
	}

	if(pChild != NULL)
		pChild.Release();

	if(pChildList != NULL)
		pChildList.Release();

	if(pNode != NULL)
		pNode.Release();

	if(pDom != NULL)
		pDom.Release();

	CoUninitialize();

	return kpdlResult;

}

CFormation *CFormationData::GetFormation(const string& formation)
{
	CFormation* pFormation = NULL;
	TFormationMapIterator fmi;

	fmi = m_formations.find( formation );
	if ( fmi != m_formations.end() )
		pFormation = m_formations[formation];

	return pFormation;
}