//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file		HigherSkills.cpp
 *  \brief		Implementacia HigherSkills - vyssie schopnosti hraca
 *  \author		Rado Kostelnik
 *  \date		1.12.2001 17:28:47
 *  \version	0.1

 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CReactiveSkills::CReactiveSkills(
	CWorldModel &cWorld,
	CGameCommandSeq &cActions,
	CBasicSkillList& cBasicSkillList,
	CStrategicSkills &cStrategicSkills,
	const string &sFileName
)
:	m_cWorld(cWorld), 
	m_cActions(cActions),
	m_cListSkill(cBasicSkillList),
	m_cStrategicSkills(cStrategicSkills)
{
	m_cRuleEngine.LoadXML(sFileName);
}

CReactiveSkills::~CReactiveSkills()
{
}

//------------------------------------------------------
/*! \fn			void CReactiveSkills::Deliberate()
 *  \brief		Rozhodovanie o nasledujucej cinnosti na vyssej urovni
 *  \param		
 *  \return		nic
 *  \author		Rado Kostelnik
 *  \date		1.12.2001 17:16:09

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------

void CReactiveSkills::Deliberate()
{
	TActionVector cActionList;
	string sResult;
	char sBuffer[100];
	int iPriority = 0;
	TSkillParameterVector cParamList;
	CBasicSkill* pSkill = NULL;

	// Novy rozhodovaci cyklus
	if(gpParserLog != NULL)
		gpParserLog->Write(
			true, 
			"------------------------------------------------------------"
		);

	ClearCache();

	/////////////////////////////////////////////////////
	// Rado K.
	// planovanie akcii - vyberie prvu akciu z radu a
	// v pripade, ze jej remove-rule nie je ""
	// vyhodnoti ho a ak sa zisti, ze akcia je na vyhodenie,
	// tak sa vyhodi
	/////////////////////////////////////////////////////
	if ( !m_cListSkill.IsEmpty() )
	{
		// ziskaj prvy BasicSkill z radu
		if ( m_cListSkill.GetFirst( &pSkill ) == KPDL_SUCCESS )
		{
			//if ( pSkill->sRemoveRule != "" )
			//{
				CAction* pAction;
				m_cRuleEngine.Evaluate(pSkill->sRemoveRule, m_cWorld, cActionList);

				// ak je vysledkom rozhodovania nejaka akcia
				if ( cActionList.size() > 0 )
				{
					pAction = *(cActionList.begin() );
					// a je to Remove
					if ( pAction->sName == "Remove" )
					{
						// zmaz vsetky nasledne BasicSkills z radu
						m_cListSkill.Empty();
						pSkill = NULL;
					}
				}
				else
					return;
			//}
			//else
				// nechaj akciu v rade na opatovne vykonanie
			//	return;
		}
	}

	/////////////////////////////////////////////////////
	// Rozhodovanie na reaktivnej urovni
	/////////////////////////////////////////////////////
	
	m_cRuleEngine.Evaluate("default", *this, cActionList);

	TActionVectorIterator pActionIter = cActionList.begin();
	TActionVectorIterator pActionEnd = cActionList.end();

	for(; pActionIter != pActionEnd; pActionIter++ )
	{
		CAction *pAction = (*pActionIter);

		//
		// Vyhodnotenie realnych hodnot parametrov akcii
		//
		TRuleParameterVectorIterator pParamIter = pAction->cParamList.begin();
		TRuleParameterVectorIterator pParamEnd = pAction->cParamList.end();

		cParamList.clear();
		for(; pParamIter != pParamEnd; pParamIter++)
		{
			CRuleParameter& cParam = (*pParamIter);
			
			long lPos;

			// location
			if(cParam.sType == "location")
			{
				sResult = Evaluate(cParam.sValue);
				lPos = sResult.find(" ");
				cParamList.push_back(CSkillParameter(BSKILL_PARAM_X, sResult.substr(0, lPos)));
				cParamList.push_back(CSkillParameter(BSKILL_PARAM_Y, sResult.substr(lPos + 1)));
			}
			// playerid
			else if(cParam.sType == "playerid")
			{
				sResult = Evaluate(cParam.sValue);
				cParamList.push_back(CSkillParameter(BSKILL_PARAM_PLAYER, sResult));
			}
			// effort
			else if(cParam.sType == "effort")
			{
				/* if(cParam.sValue == "Maximum")
					iPriority = 3;
				else if(cParam.sValue == "Medium")
					iPriority = 2;
				else if(cParam.sValue == "Minimum")
					iPriority = 1; */
				cParamList.push_back(CSkillParameter(BSKILL_PARAM_EFFORT, cParam.sValue));

			}
			// angle
			else if(cParam.sType == "angle")
			{
				cParamList.push_back(CSkillParameter(BSKILL_PARAM_ANGLE, cParam.sValue));
			}
			else
			{
				if(gpSystemLog != NULL)
					gpSystemLog->Write(
						true, 
						"Chyba pri vyhodnocovani parametrov: neznamy typ '%s'" ,
						cParam.sType.c_str()
					);
			}
		}

		//
		// Kick
		//
		if(pAction->sName == "Kick")
		{
			itoa(BSKILL_KICK_TO_POSITION, sBuffer, 10);
			cParamList.push_back(CSkillParameter(BSKILL_PARAM_TYPE, sBuffer));
			pSkill = new CKick(cParamList, iPriority, m_cWorld, m_cActions);
		}
		else
		//
		// GoToPosition
		//
		if(pAction->sName == "GoToPosition")
		{
			itoa(BSKILL_MOVE_GOTO, sBuffer, 10);
			cParamList.push_back(CSkillParameter(BSKILL_PARAM_TYPE, sBuffer));
			pSkill = new CMove(cParamList, iPriority, m_cWorld, m_cActions);
		}
		//
		// Move
		//
		if(pAction->sName == "Move")
		{
			itoa(BSKILL_MOVE_TO, sBuffer, 10);
			cParamList.push_back(CSkillParameter(BSKILL_PARAM_TYPE, sBuffer));
			pSkill = new CMove(cParamList, iPriority, m_cWorld, m_cActions);
		}
		else 
		//
		// DribbleTo
		//
		if(pAction->sName == "DribbleTo")
		{
			itoa(BSKILL_DRIBLE_SMART, sBuffer, 10);
			cParamList.push_back(CSkillParameter(BSKILL_PARAM_TYPE, sBuffer));
			pSkill = new CDribble(cParamList, iPriority, m_cWorld, m_cActions);
		}
		else 
		//
		// TurnTo
		//
		if(pAction->sName == "Turn")
		{
			itoa(BSKILL_MOVE_GOTO_BALL, sBuffer, 10);
			cParamList.push_back(CSkillParameter(BSKILL_PARAM_TYPE, sBuffer));
			pSkill = new CTurn(cParamList, iPriority, m_cWorld, m_cActions);
		}
		else
		//
		// PassTo
		//
		if(pAction->sName == "PassTo")
		{
			itoa(BSKILL_TURN_2, sBuffer, 10);
			cParamList.push_back(CSkillParameter(BSKILL_PARAM_TYPE, sBuffer));
			pSkill = new CPass(cParamList, iPriority, m_cWorld, m_cActions);
		}
		else 
		//
		// Catch
		//
		if(pAction->sName == "Catch")
		{
			itoa(BSKILL_CATCH_BASIC, sBuffer, 10);
			cParamList.push_back(CSkillParameter(BSKILL_PARAM_TYPE, sBuffer));
			pSkill = new CCatch(cParamList, iPriority, m_cWorld, m_cActions);
		}
		else
		// rado 30.4.2002 19:56:30
		if(pAction->sName == "InterceptPass") 
		{
			itoa(BSKILL_KICK_PASS_INTERCEPT, sBuffer, 10);
			cParamList.push_back(CSkillParameter(BSKILL_PARAM_TYPE, sBuffer));
			pSkill = new CMove(cParamList, iPriority, m_cWorld, m_cActions);			
		}
		else 
		//rado 27. 12. 2001
		//
		// GoToBallPosition
		//
		if(pAction->sName == "GoToBallPosition") 
		{
			itoa(BSKILL_MOVE_GOTO_BALL, sBuffer, 10);
			cParamList.push_back(CSkillParameter(BSKILL_PARAM_TYPE, sBuffer));
			pSkill = new CMove(cParamList, iPriority, m_cWorld, m_cActions);
		}

		if(pSkill != NULL)
		{
			//pSkill->m_cListSkill = &m_cListSkill;
			pSkill->sRemoveRule = pAction->sRemoveRule;

			m_cListSkill.AddSkill(*pSkill);

			if(gpSystemLog != NULL)
				gpSystemLog->Write(
					true,
					"pridana akcia: '%s'", 
					pAction->sName.c_str()
				);
		}

		pSkill = NULL;
	}
}

string CReactiveSkills::GetAttribute(const string &sName)
{
	string sHeader = sName;
	string sTail = CQuery::SplitTail(sHeader);
	
	if(sHeader == "strategy")
		return m_cStrategicSkills.Evaluate(sTail);
	
	return m_cWorld.Evaluate(sName);
}

void CReactiveSkills::ClearCache()
{
	m_cWorld.ClearCache();
	m_cStrategicSkills.ClearCache();
	CQuery::ClearCache();
}
