#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BEGIN_MSG_MAP(CThread)
END_MSG_MAP

CThread::CThread()
:	m_ulThreadID(NULL),
	m_hThreadHnd(NULL),
	m_hInitEvent(NULL),
	m_eResult(KPDL_ERROR)
{
	m_hInitEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
}

CThread::~CThread()
{
	Stop();

	if(m_hInitEvent != NULL)
	{
		CloseHandle(m_hInitEvent);
		m_hInitEvent = NULL;
	}
}

KPDL_RESULT CThread::Start(const int iPriority /* = THREAD_PRIORITY_NORMAL */)
{
	if(m_hInitEvent == NULL || IsValid())
		return KPDL_ERROR;

	m_hThreadHnd = CreateThread(
		NULL,
		THREAD_STACK_SIZE,
		(LPTHREAD_START_ROUTINE) ThreadProc,
		this,
		0,
		&m_ulThreadID
	);
	
	if(m_hThreadHnd == NULL ||
		WaitForSingleObject(m_hInitEvent, INFINITE) == WAIT_FAILED)
		return KPDL_ERROR;

	return m_eResult;
}

KPDL_RESULT CThread::Stop()
{
	if(!IsValid())
		return KPDL_ERROR;

	if(!PostThreadMessage(m_ulThreadID, WM_QUIT, NULL, NULL) ||
		WaitForSingleObject(m_hThreadHnd, INFINITE) == WAIT_FAILED)
		return KPDL_ERROR;

	CloseHandle(m_hThreadHnd);
	m_hThreadHnd = NULL;
	m_ulThreadID = NULL;

	return m_eResult;
}

KPDL_RESULT CThread::SetMsg(
	const int iMsgID, 
	const WPARAM wParam /* = NULL */,
	const LPARAM lParam /* = NULL */
)
{
	if(IsValid() &&
		PostThreadMessage(m_ulThreadID, iMsgID, wParam, lParam))
		return KPDL_SUCCESS;

	return KPDL_ERROR;
}

DWORD WINAPI CThread::ThreadProc(CThread *pThis)
{
	MSG tMessage;
	SMsgMap *pEntry;

	if(pThis == NULL || pThis->m_hInitEvent == NULL)
		ExitThread(KPDL_ERROR);

	PeekMessage(&tMessage, NULL, 0, 0, PM_NOREMOVE);
	pThis->m_eResult = pThis->Init();

	if(!::SetEvent(pThis->m_hInitEvent))
		pThis->m_eResult = KPDL_ERROR;

	if(pThis->m_eResult == KPDL_SUCCESS)
	{
		while(GetMessage(&tMessage, NULL, 0, 0))
		{
			for(pEntry = pThis->GetMsgMap(); pEntry->fnMsg != NULL; pEntry++)
			{
				if(pEntry->uiMsgID == tMessage.message)
					(pThis->*pEntry->fnMsg)(
						tMessage.wParam, 
						tMessage.lParam
					);
			}

			TranslateMessage(&tMessage); 
			DispatchMessage(&tMessage); 
		}
	}
	
	pThis->Exit();
	ExitThread(pThis->m_eResult);
	// kvoli kompilatoru, program sa sem nikdy nedostane
	return 0;
}
