#ifndef _THREAD_
#define _THREAD_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define THREAD_STACK_SIZE (1048576ul)

class CThread;

typedef KPDL_RESULT (CThread::*TMsgFunction)( \
	const WPARAM wParam, \
	const LPARAM lParam \
);

struct SMsgMap
{
	UINT uiMsgID;
	TMsgFunction fnMsg;
};

#define DECLARE_MSG_MAP \
	protected: \
		static SMsgMap m_tMsgMap[]; \
		virtual SMsgMap *GetMsgMap() { return m_tMsgMap; }
#define BEGIN_MSG_MAP(class) SMsgMap class::m_tMsgMap[] = {
#define END_MSG_MAP {(UINT)0, (TMsgFunction)NULL}};
#define ADD_MSG(uiMsgID, fnMsg) \
	{(UINT)uiMsgID, (TMsgFunction)fnMsg},

class CThread
{
	DECLARE_MSG_MAP

private:
	ULONG m_ulThreadID;
	HANDLE m_hThreadHnd;
	HANDLE m_hInitEvent;
	KPDL_RESULT m_eResult;

public:
	CThread();
	virtual ~CThread();

	KPDL_RESULT Start(const int iPriority = THREAD_PRIORITY_NORMAL);
	KPDL_RESULT Stop();

	KPDL_RESULT SetMsg(
		const int iMsgID, 
		const WPARAM wParam = NULL,
		const LPARAM lParam = NULL		
	);
	
	bool IsValid()
	{
		return m_hThreadHnd != NULL;
	}

protected:
	virtual KPDL_RESULT Init()
	{
		return KPDL_SUCCESS;
	}

	virtual void Exit() {}

private:
	static DWORD WINAPI ThreadProc(CThread *pThis);

};

#endif // _THREAD_