// EventThread.h : header file
//

#ifndef _EVENT_THREAD_
#define _EVENT_THREAD_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CEventThread;

typedef KPDL_RESULT (CEventThread::*TEventFunction)(CEvent &cEvent);

struct SEventMap
{
	EEventType eType;
	TEventFunction fnEvent;
};

#define DECLARE_EVENT_MAP \
	protected: \
		static SEventMap m_tEventMap[]; \
		virtual SEventMap *GetEventMap() { return m_tEventMap; }
#define BEGIN_EVENT_MAP(class) SEventMap class::m_tEventMap[] = {
#define END_EVENT_MAP {E_None, (TEventFunction)NULL}};
#define ADD_EVENT(eType, fnEvent) \
	{(EEventType)eType, (TEventFunction)&fnEvent},

#ifdef EVENT_ASYNC_PLAYER

// Asynchronne spracovanie udalosti v systeme
class CEventThread : public CThread
{
	DECLARE_MSG_MAP
	DECLARE_EVENT_MAP

private:
	CCoordinator &m_cCoordinator;

public:
	CEventThread(
		CCoordinator &cCoordinator,
		const int iPriority = THREAD_PRIORITY_NORMAL
	)
	:	m_cCoordinator(cCoordinator)
	{
		// priamo spustime thread
		Start(iPriority);
		// a zaregistrujeme ho v koordinatore
		m_cCoordinator.RegisterEventThread(*this);
	}
	virtual ~CEventThread()
	{
		// odregistrujeme sa z koordinatora
		m_cCoordinator.UnregisterEventThread(*this);
	};

	// t0t0 sluzi iba koordinatorovi na odovzdanie udalosti
	// na spracovanie tomuto threadu
	KPDL_RESULT SetEvent(CEvent &cEvent)
	{
		cEvent.AddRef();
		return CThread::SetMsg(WM_EVENT, NULL, (LPARAM) &cEvent);
	}

	// t0t0 je metoda na generovanie udalosti
	KPDL_RESULT GenerateEvent(CEvent *pEvent)
	{
		if(pEvent == NULL)
			return KPDL_ERROR;
		return m_cCoordinator.DispatchEvent(*pEvent);
	}

private:
	KPDL_RESULT OnEventMessage(
		const WPARAM wParam, 
		const LPARAM lParam
	);

};

#else // EVENT_ASYNC_PLAYER

// Synchronne spracovanie udalosti v systeme (t0t0 vlastne ani nie je thread)
class CEventThread
{
	DECLARE_MSG_MAP
	DECLARE_EVENT_MAP

private:
	CCoordinator &m_cCoordinator;

public:
	CEventThread(
		CCoordinator &cCoordinator,
		const int iPriority = THREAD_PRIORITY_NORMAL
	)
	:	m_cCoordinator(cCoordinator)
	{
		// a zaregistrujeme ho v koordinatore
		m_cCoordinator.RegisterEventThread(*this);
	}
	virtual ~CEventThread()
	{
		// odregistrujeme sa z koordinatora
		m_cCoordinator.UnregisterEventThread(*this);
	};

	// t0t0 sluzi iba koordinatorovi na odovzdanie udalosti
	// na spracovanie tomuto threadu
	KPDL_RESULT SetEvent(CEvent &cEvent)
	{
		cEvent.AddRef();
		return OnEventMessage(NULL, (LPARAM) &cEvent);
	}

	// t0t0 je metoda na generovanie udalosti
	KPDL_RESULT GenerateEvent(CEvent *pEvent)
	{
		if(pEvent == NULL)
			return KPDL_ERROR;
		return m_cCoordinator.DispatchEvent(*pEvent);
	}

private:
	KPDL_RESULT OnEventMessage(
		const WPARAM wParam, 
		const LPARAM lParam
	);

};

#endif // EVENT_ASYNC_PLAYER

#endif // _EVENT_THREAD_
