// LogBuffer.cpp: implementation of the CLogBuffer class.
//////////////////////////////////////////////////////////////////////
//! /bug K metodam AddMsg a Clear resp. Show sa pristupuje z roznych
//       threadov, bolo by vhodne implementovat kontrolu pristupu
//       (zrejme pomocou kritickej sekcie)
#include "stdafx.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLogBuffer::CLogBuffer()
{
	InitializeCriticalSection(&m_tLock);
}

CLogBuffer::~CLogBuffer()
{
	DeleteCriticalSection(&m_tLock);
	m_cMsgList.clear();
}

KPDL_RESULT CLogBuffer::AddMsg(
	const string &sModule,
	const string &sTimeStamp,
	const string &sMessage
)
{
	EnterCriticalSection(&m_tLock);

	TMsgListIterator pMsgIter = m_cMsgList.insert(m_cMsgList.end());
	(*pMsgIter).sModule = sModule;
	(*pMsgIter).sTimeStamp = sTimeStamp;
	(*pMsgIter).sMessage = sMessage;

	if(m_cMsgList.size() > LOG_MAX_MSG)
		m_cMsgList.erase(m_cMsgList.begin());

	LeaveCriticalSection(&m_tLock);

	return KPDL_SUCCESS;
}

KPDL_RESULT CLogBuffer::Clear()
{
	EnterCriticalSection(&m_tLock);
	m_cMsgList.clear();
	LeaveCriticalSection(&m_tLock);

	return KPDL_SUCCESS;
}

KPDL_RESULT CLogBuffer::Show(
	CDC &cDevice, 
	const int iVSize, 
	const int iFontSize
)
{
	EnterCriticalSection(&m_tLock);

	TMsgListIterator pFirst = m_cMsgList.begin();
	TMsgListIterator pIter = m_cMsgList.end();
	int iVPos = iVSize;

	while(pIter != pFirst)
	{
		iVPos -= iFontSize;
		--pIter;

		SLogEntry &tEntry = *pIter;

		cDevice.TextOut(
			LOG_FILE_POS, 
			iVPos, 
			CString(tEntry.sModule.c_str())
		);
		cDevice.TextOut(
			LOG_TIME_POS, 
			iVPos, 
			CString(tEntry.sTimeStamp.c_str())
		);
		cDevice.TextOut(
			LOG_MSG_POS, 
			iVPos, 
			CString(tEntry.sMessage.c_str())
		);

		if(iVPos < 0)
			// plne okno
			break;
	}

	LeaveCriticalSection(&m_tLock);

	return KPDL_SUCCESS;
}
