//*****************************************************************************
// Forward.cpp: Implementation of the CForwardNetwork class
//*****************************************************************************

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//*****************************************************************************
// Construction/Destruction
//*****************************************************************************

CForwardNetwork::CForwardNetwork(
	const int iSizeIn,
	const int iSizeHid,
	const int iSizeOut,
	CActivate &cActivate,
	const double dTolerance, /* = 0.1 */
	const double dMomentum, /* = 0.0 */
	const double dAlpha /* = 0.0 */
)
:	CNeuralNetwork(dTolerance, dMomentum, dAlpha),
	// Limits used
	iSizeIn(iSizeIn),
	iSizeHid(iSizeHid),
	iSizeOut(iSizeOut),
	// Layers
	m_cHidLayer(iSizeIn, iSizeHid, cActivate),
	m_cOutLayer(iSizeHid, iSizeOut, cActivate),
	// Temporals
	m_cHidden(iSizeHid + 1)
{
}

//*****************************************************************************
// Methods/Operators
//*****************************************************************************

KPDL_RESULT CForwardNetwork::Process(
	CSymbol &cInput, 
	CSymbol &cOutput
)
{
	m_cHidLayer.Process(cInput, m_cHidden);
	m_cOutLayer.Process(m_cHidden, cOutput);
	cOutput.Interpret(dTolerance);
	return KPDL_SUCCESS;
}

double CForwardNetwork::Adapt(
	CSymbol &cInput,
	CSymbol &cDesired
)
{
	// Current output, output sigma, hidden sigma 
	CVector cOutput(iSizeOut);
	CVector cDeltaOut(iSizeOut);
	CVector cDeltaHid(iSizeHid);

	m_cHidLayer.Process(cInput, m_cHidden);
	m_cOutLayer.Process(m_cHidden, cOutput);

	// Delta for network output
	m_cOutLayer.OutputDelta(cDesired, cOutput, cDeltaOut);
	// Delta for the hidden layer
	m_cOutLayer.HiddenDelta(cDeltaOut, m_cHidden, cDeltaHid);

	// Update output weights
	m_cOutLayer.Adapt(dAlpha, dMomentum, cDeltaOut, m_cHidden);
	// Update hidden weights
	m_cHidLayer.Adapt(dAlpha, dMomentum, cDeltaHid, cInput);

	// Count the error
	return cOutput.Error(cDesired);
}
