//*****************************************************************************
// Layer.cpp: Implementation of the CLayer class
//*****************************************************************************

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//*****************************************************************************
// Construction/Destruction
//*****************************************************************************

CLayer::CLayer(
	const int iSizeIn,
	const int iSizeOut,
	CActivate &cActivate
)
:	// Limits used
	iSizeIn(iSizeIn),
	iSizeOut(iSizeOut),
	// Weights matrices (1 dummy input neuron :)
	m_cWeights(iSizeOut, iSizeIn + 1, -1.0, 1.0),
	m_cDiffW(iSizeOut, iSizeIn + 1),
	// Activation function
	m_cActivate(cActivate)
{
	if(gpNeuralLog != NULL)
	{
		gpNeuralLog->Write(true, " - Weights:");
		gpNeuralLog->WriteMatrix(true, " % 6.4f", m_cWeights);
	}
}

//*****************************************************************************
// Methods/Operators
//*****************************************************************************

KPDL_RESULT CLayer::OutputDelta(
	CVector &cDesired,
	CVector &cOutput,
	CVector &cDelta
)
{
	// Indexes
	int iNeuron;

	// Delta for network output
	for(iNeuron = 0; iNeuron < iSizeOut; iNeuron++)
		cDelta(iNeuron) = (cDesired(iNeuron) - cOutput(iNeuron)) *
			m_cActivate.DiffF(cOutput(iNeuron));

	return KPDL_SUCCESS;
}

KPDL_RESULT CLayer::HiddenDelta(
	CVector &cDeltaNext,
	CVector &cOutput,
	CVector &cDelta
)
{
	// Indexes
	int iNeuron;
	int iInput;

	// Delta for the hidden layer 
	for(iNeuron = 0; iNeuron < iSizeIn; iNeuron++)
	{
		cDelta(iNeuron) = 0.0;
		for(iInput = 0; iInput < iSizeOut; iInput++)
			cDelta(iNeuron) += cDeltaNext(iInput) * m_cWeights(iInput, iNeuron);
		cDelta(iNeuron) *= m_cActivate.DiffF(cOutput(iNeuron));
	}

	// Dummy neuron will never learn (it is always out)
	cDelta(iSizeIn + 1) = 0.0;

	return KPDL_SUCCESS;
}

KPDL_RESULT CLayer::Adapt(
	const double dAlpha,
	const double dMomentum,
	CVector &cDelta,
	CVector &cInput
)
{
	// Indexes
	int iNeuron;
	int iInput;

	// Update weights
	for(iNeuron = 0; iNeuron < iSizeOut; iNeuron++)
	{
		for(iInput = 0; iInput < iSizeIn + 1; iInput++)
		{
			m_cDiffW(iNeuron, iInput) *= dMomentum; 
			m_cDiffW(iNeuron, iInput) += 
				dAlpha * cDelta(iNeuron) * cInput(iInput);
			m_cWeights(iNeuron, iInput) += m_cDiffW(iNeuron, iInput);
		}
	}

	return KPDL_SUCCESS;
}

KPDL_RESULT CLayer::Process(
	CVector &cInput,
	CVector &cOutput
)
{
	// Indexes
	int iNeuron;
	int iInput;
	double dNet;
	
	// Threshold
	cInput(iSizeIn) = -1.0;
	// Ouptut from hidden layer
	for(iNeuron = 0; iNeuron < iSizeOut; iNeuron++)
	{
		dNet = 0.0;
		// Dummy neuron included, too
		for(iInput = 0; iInput < iSizeIn + 1; iInput++)
			dNet += cInput(iInput) * m_cWeights(iNeuron, iInput);
		cOutput(iNeuron) = m_cActivate.F(dNet);
	}

	return KPDL_SUCCESS;
}
