//*****************************************************************************
// FILE:         List.h
// DESCRIPTION:  Generic list & list item
// PROJECT:      Evolution as an avalanche process
// AUTHOR:       Anton Weissensteiner
// DATE:         5.11.2001
//*****************************************************************************

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//*****************************************************************************
// CItem Construction/Destruction
//*****************************************************************************

CItem::CItem()
:	pPrev(NULL),
	pNext(NULL)
{
}


CItem::~CItem()
{
	if(pPrev != NULL)
	{
		pPrev->pNext = pNext;
	}

	if(pNext != NULL)
	{
		pNext->pPrev = pPrev;
	}

	pPrev = pNext = NULL;
}

//*****************************************************************************
// CItem Methods/Operators
//*****************************************************************************

KPDL_RESULT CItem::AddPrev(CItem *pNew)
{
	if(pNew == NULL)
		return KPDL_ERROR;

	CItem *pNewFirst = pNew->GetFirst();
	CItem *pNewLast = pNew->GetLast();

	pNewLast->pNext= this;
	if((pNewFirst->pPrev = pPrev) != NULL)
	{
		pPrev->pNext = pNewFirst;
	}
	pPrev = pNewFirst;

	return KPDL_SUCCESS;
}

KPDL_RESULT CItem::AddNext(CItem *pNew)
{
	if(pNew == NULL)
		return KPDL_ERROR;

	CItem *pNewFirst = pNew->GetFirst();
	CItem *pNewLast = pNew->GetLast();

	pNewFirst->pPrev = this;
	if((pNewLast->pNext = pNext) != NULL)
	{
		pNext->pPrev = pNewLast;
	}
	pNext = pNewFirst;

	return KPDL_SUCCESS;
}

CItem *CItem::GetFirst()
{
	CItem *pFirst = this;

	while(!pFirst->IsFirst())
	{
		pFirst = pFirst->pPrev;
	}

	return pFirst;
}

CItem *CItem::GetLast()
{
	CItem *pLast = this;

	while(!pLast->IsLast())
	{
		pLast = pLast->pNext;
	}

	return pLast;
}

//*****************************************************************************
// CList Construction/Destruction
//*****************************************************************************

CList::CList()
:	m_pCurrent(NULL),
	m_iSize(0)
{
}

CList::~CList()
{
	Clear();
}

//*****************************************************************************
// CList Methods/Operators
//*****************************************************************************

KPDL_RESULT CList::Append(CItem *pNew)
{
	if(pNew == NULL)
		return KPDL_ERROR;

	if(m_pCurrent != NULL)
		m_pCurrent->AddNext(pNew);

	m_pCurrent = pNew;
	m_iSize ++;

	return KPDL_SUCCESS;
}

KPDL_RESULT CList::Insert(CItem *pNew)
{
	if(pNew == NULL)
		return KPDL_ERROR;

	if(m_pCurrent != NULL)
		m_pCurrent->AddPrev(pNew);

	m_pCurrent = pNew;
	m_iSize ++;

	return KPDL_SUCCESS;
}

KPDL_RESULT CList::Delete()
{
	CItem *pDelete = m_pCurrent;
	
	if(pDelete == NULL)
		return KPDL_ERROR;
	
	if(m_pCurrent->IsFirst())
		m_pCurrent = m_pCurrent->pNext;
	else 
		m_pCurrent = m_pCurrent->pPrev;

	delete pDelete;
	m_iSize --;
	return KPDL_SUCCESS;
}

KPDL_RESULT CList::Clear()
{
	if(m_pCurrent == NULL)
		return KPDL_ERROR;

	m_pCurrent = m_pCurrent->GetFirst();
	while(m_pCurrent != NULL)
	{
		CItem *pDelete = m_pCurrent;
		m_pCurrent = m_pCurrent->pNext;
		delete pDelete;
	}

	m_iSize = 0;
	return KPDL_SUCCESS;
}

KPDL_RESULT CList::MovePrev()
{
	if(m_pCurrent == NULL || m_pCurrent->IsFirst())
		return KPDL_UNSUCCESS;
	
	m_pCurrent = m_pCurrent->pPrev;
	return KPDL_SUCCESS;
}

KPDL_RESULT CList::MoveNext()
{
	if(m_pCurrent == NULL || m_pCurrent->IsLast())
		return KPDL_UNSUCCESS;

	m_pCurrent = m_pCurrent->pNext;
	return KPDL_SUCCESS;
}

KPDL_RESULT CList::MoveFirst()
{
	if(m_pCurrent == NULL)
		return KPDL_UNSUCCESS;

	m_pCurrent = m_pCurrent->GetFirst();
	return KPDL_SUCCESS;
}

KPDL_RESULT CList::MoveLast()
{
	if(m_pCurrent == NULL)
		return KPDL_UNSUCCESS;

	m_pCurrent = m_pCurrent->GetLast();
	return KPDL_SUCCESS;
}

bool CList::Contains(CItem *pItem)
{
	if(m_pCurrent == NULL)
		return FALSE;

	CItem *m_pItem = m_pCurrent->GetFirst();
	while(m_pItem != NULL)
	{
		if(m_pItem == pItem)
			return TRUE;

		m_pItem = m_pItem->pNext;
	}

	return FALSE;
}
