//*****************************************************************************
// Matrix.cpp: Implementation of the CMatrix class
//*****************************************************************************

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//*****************************************************************************
// Construction/Destruction
//*****************************************************************************

CRandom CMatrix::m_cInitVal;

CMatrix::CMatrix(
	const int iHeight, 
	const int iWidth,
	const double dValue // = 0.0
)
:	iHeight(iHeight),
	iWidth(iWidth),
	m_pdStorage(new double[iHeight * iWidth])
{
	int iSize = iHeight * iWidth;
	int iIndex;

	for(iIndex = 0; iIndex < iSize; iIndex++)
		m_pdStorage[iIndex] = dValue;
}

CMatrix::CMatrix(CMatrix &Source)
:	iHeight(Source.iHeight),
	iWidth(Source.iWidth),
	m_pdStorage(new double[iHeight * iWidth])
{
	int iSize = iHeight * iWidth;
	memcpy(m_pdStorage, Source.m_pdStorage, iSize * sizeof(double));
}

CMatrix::CMatrix(
	const int iHeight,
	const int iWidth,
	const double dBottom,
	const double dTop
)
:	iHeight(iHeight),
	iWidth(iWidth),
	m_pdStorage(new double[iHeight * iWidth])
{
	double dScope = dTop - dBottom;
	int iSize = iHeight * iWidth;
	int iIndex;

	for(iIndex = 0; iIndex < iSize; iIndex ++)
		m_pdStorage[iIndex] = dBottom + dScope * m_cInitVal.Random();
}

CMatrix::~CMatrix() 
{
	if(m_pdStorage != NULL)
	{
		delete m_pdStorage;
		m_pdStorage = NULL;
	}
	iWidth = iHeight = 0;
}

//*****************************************************************************
// Methods/Operators
//*****************************************************************************

double &CMatrix::operator ()(const int iRow, const int iColumn)
{
#ifdef NEURAL_CHECK_INDEX
	if(
		iRow < 0 || iRow >= iHeight || 
		iColumn < 0 || iColumn >= iWidth || 
		m_pdStorage == NULL
	)
	{
		if(gpNeuralLog != NULL)
			gpNeuralLog->Write(true, "Chyba: Index matice mimo rozsah!");
	}

#endif // NEURAL_CHECK_INDEX

	return m_pdStorage[iRow * iWidth + iColumn];
}

CMatrix &CMatrix::operator =(CMatrix &cSource)
{
#ifdef NEURAL_CHECK_INDEX
	if(
		m_pdStorage == NULL || cSource.m_pdStorage == NULL ||
		iHeight != cSource.iHeight || iWidth != cSource.iWidth
	)
	{
		if(gpNeuralLog != NULL)
			gpNeuralLog->Write(
				true,
				"Chyba: Nesedi rozmer matice so vzorom pri kopirovani!"
			);
	}

#endif // NEURAL_CHECK_INDEX

	int iSize = iHeight * iWidth;

	memcpy(m_pdStorage, cSource.m_pdStorage, iSize * sizeof(double));
	return(*this);
}
