//*****************************************************************************
// Vector.cpp: Implementation of the CVector class DBL_MAX
//*****************************************************************************

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//*****************************************************************************
// Construction/Destruction
//*****************************************************************************

CRandom CVector::m_cInitVal;

CVector::CVector(
	const int iSize,
	const double dValue /* = 0.0 */
)
:	CItem(),
	iSize(iSize),
	m_pdStorage(new double[iSize])
{
	int iIndex;

	for(iIndex = 0; iIndex < iSize; iIndex++)
		m_pdStorage[iIndex] = dValue;
}

CVector::CVector(CVector &Source)
:	CItem(),
	iSize(Source.iSize),
	m_pdStorage(new double[iSize])
{
	memcpy(m_pdStorage, Source.m_pdStorage, iSize * sizeof(double));
}

CVector::CVector(
	const int iSize,
	const double dBottom,
	const double dTop
)
:	CItem(),
	iSize(iSize),
	m_pdStorage(new double[iSize])
{
	double dScope = dTop - dBottom;
	int iIndex;

	for(iIndex = 0; iIndex < iSize; iIndex ++)
		m_pdStorage[iIndex] = dBottom + dScope * m_cInitVal.Random();
}

CVector::~CVector() 
{
	if(m_pdStorage != NULL)
	{
		delete m_pdStorage;
		m_pdStorage = NULL;
	}
	iSize = 0;
}

//*****************************************************************************
// Methods/Operators
//*****************************************************************************

double CVector::Error(CVector &cDesired)
{
#ifdef NEURAL_CHECK_INDEX
	if(m_pdStorage == NULL || cDesired.m_pdStorage == NULL || 
		iSize != cDesired.iSize)
	{
		if(gpNeuralLog != NULL)
			gpNeuralLog->Write(
				true, 
				"Chyba: Nesedi rozmer vektora so vzorom pri pocitani rozdielu!"
			);
	}

#endif // NEURAL_CHECK_INDEX

	int iIndex;
	double dDiff;
	double dNorm = 0.0;

	for(iIndex = 0; iIndex < iSize; iIndex ++)
	{
		// Count error from continuos network output
		dDiff = m_pdStorage[iIndex] - cDesired.m_pdStorage[iIndex];
		dNorm += dDiff * dDiff;
	}

	return 0.5 * dNorm;
}

double &CVector::operator ()(const int iIndex)
{
#ifdef NEURAL_CHECK_INDEX
	if(iIndex < 0 || iIndex >= iSize || m_pdStorage == NULL)
	{
		if(gpNeuralLog != NULL)
			gpNeuralLog->Write(true, "Chyba: Index vektora mimo rozsah!");
	}

#endif // NEURAL_CHECK_INDEX

	return m_pdStorage[iIndex];
}

CVector &CVector::operator =(CVector &cSource)
{
#ifdef NEURAL_CHECK_INDEX
	if(m_pdStorage == NULL || cSource.m_pdStorage == NULL || iSize != cSource.iSize)
	{
		if(gpNeuralLog != NULL)
			gpNeuralLog->Write(
				true, 
				"Chyba: Nesedi rozmer vektora so vzorom pri kopirovani!"
			);
	}

#endif // NEURAL_CHECK_INDEX

	memcpy(m_pdStorage, cSource.m_pdStorage, iSize * sizeof(double));
	return(*this);
}
