//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	RuleEngine.cpp
 *  \brief Implementuje engine pre spracovanie pravidiel
 *  \author Rado Kostelnik
 *  \date 1.12.2001 16:00:10
 *  \version 
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CLog *gpParserLog = NULL;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRuleEngine::CRuleEngine()
{
}

CRuleEngine::~CRuleEngine()
{
	TRuleVectorIterator pRuleIter = cRuleList.begin();
	TRuleVectorIterator pRuleLast = cRuleList.end();

	for(; pRuleIter != pRuleLast; pRuleIter++)
		delete (*pRuleIter);

	cRuleList.clear();
}

//------------------------------------------------------
/*! \fn		bool CRuleEngine::LoadXML(const string &sFileName)
 *  \brief	 
 *	\param	filename Nazov vstupneho suboru
 *  \return Vracia true, ak sa subor nacital v poriadku
 *  \author	Rado Kostelnik
 *  \date	30.11.2001 23:36:59
 */
//------------------------------------------------------
KPDL_RESULT CRuleEngine::LoadXML(const string &sFileName)
{
	// think positive :)
	KPDL_RESULT kpdlResult = KPDL_SUCCESS;
	
	IXMLDOMDocumentPtr pDom = NULL;
	IXMLDOMNodePtr pNode = NULL;
	IXMLDOMNodeListPtr pChildList = NULL;
	IXMLDOMNodePtr pChild = NULL;

	_variant_t vValue;
	string sTemp;
	_variant_t vFileName;
	_variant_t vRetVal;
	CRule *pRule;

	if(FAILED(CoInitializeEx(0, COINIT_MULTITHREADED)))
	{
		return KPDL_ERROR;
	}

	try
	{
		pDom.CreateInstance(DOM_DOCUMENT);
		vFileName = _bstr_t((LPCSTR)sFileName.c_str());
		vRetVal = pDom->load(vFileName);
		
		if(!(bool)vRetVal)
			return KPDL_ERROR;

		pNode = pDom->documentElement;

		try
		{
			// precitaj typ vyhodnocovania
			pNode->attributes->getNamedItem("type")->get_nodeValue(&vValue);
			sTemp = _bstr_t(vValue);

			if ( sTemp == "sequential" )
				m_EvaluationType = 0;
			else if ( sTemp == "priority" ) 
				m_EvaluationType = 1;
			else
				m_EvaluationType = 0;
		}
		catch(_com_error &E)
		{	
			m_EvaluationType = 0;
		}

		pChildList = pNode->selectNodes(L"rule");
		pChildList->reset(); 

		while((pChild = pChildList->nextNode()) != NULL)
		{
			pRule = new CRule(this);
			pRule->SetXML(pChild);
			cRuleList.push_back(pRule);
		}
	}
	catch(_com_error &E)
	{
		if(gpSystemLog != NULL)
			gpSystemLog->Write(
				true, 
				"Chyba %08X pri nahravani suboru pravidiel '%s'",
				E.Error(),
				sFileName.c_str()
			);
		kpdlResult = KPDL_ERROR;
	}

	if(pChild != NULL)
		pChild.Release();

	if(pChildList != NULL)
		pChildList.Release();

	if(pNode != NULL)
		pNode.Release();

	if(pDom != NULL)
		pDom.Release();

	CoUninitialize();

	return kpdlResult;
}

KPDL_RESULT CRuleEngine::Evaluate(
	const string &sRuleName,
	CQuery& cQuery,
	TActionVector &cResult
)
{
	if ( m_EvaluationType == 0 )
		return Evaluate1( sRuleName, cQuery, cResult );
	else if ( m_EvaluationType == 1 )
		return Evaluate2( sRuleName, cQuery, cResult );

	return KPDL_ERROR;
}

//------------------------------------------------------
/*! \fn		KPDL_RESULT CRuleEngine::Evaluate(const string &sRuleName, CQuery& cQuery, TActionVector &cResult)
 *  \brief	Zacina vyhodnocovanie stromu pravidiel 
 *	\param	sRuleName Nazov zaciatocneho pravidla
 *	\param	cQuery objekt, ktory vracia hodnoty premmenych
 *	\param	cResult vector akcii, ktore sa maju vykonat
 *  \return KPDL_SUCCESS vyhodnotenie pravidiel bolo uspesne
 *          KPDL_UNSUCCESS pravidlo sa v strome nenaslo
 *  \author	Rado Kostelnik
 *  \date	30.11.2001 23:36:59
 */
//------------------------------------------------------
KPDL_RESULT CRuleEngine::Evaluate1(
	const string &sRuleName,
	CQuery& cQuery,
	TActionVector &cResult
)
{
	TRuleVectorIterator pIter = cRuleList.begin();
	TRuleVectorIterator pLast = cRuleList.end();

	for(; pIter != pLast; pIter++)
	{
		CRule *pRule = (*pIter);
		if(pRule->sName == sRuleName)
		{
			// found a rule, situation is saved
			if(pRule->Evaluate(cQuery, cResult) == KPDL_SUCCESS)
			    return KPDL_SUCCESS;
		}
	}

	return KPDL_UNSUCCESS;
}

//------------------------------------------------------
/*! \fn		KPDL_RESULT CRuleEngine::Evaluate2(const string &sRuleName, CQuery& cQuery, TActionVector &cResult)
 *  \brief	Zacina vyhodnocovanie stromu pravidiel a vracia zoznam akcii
 *	\param	sRuleName Nazov zaciatocneho pravidla
 *	\param	cQuery objekt, ktory vracia hodnoty premmenych
 *	\param	cResult vector akcii, ktore sa maju vykonat
 *  \return KPDL_SUCCESS vyhodnotenie pravidiel bolo uspesne
 *          KPDL_UNSUCCESS pravidlo sa v strome nenaslo
 *  \author	Rado Kostelnik
 *  \date	24.04.2002 15:22
 */
//------------------------------------------------------
KPDL_RESULT CRuleEngine::Evaluate2(
	const string &sRuleName,
	CQuery& cQuery,
	TActionVector &cResult
)
{
	TRuleVectorIterator pIter = cRuleList.begin();
	TRuleVectorIterator pLast = cRuleList.end();
	string sMaxPriority = "";
	TActionVector tResult;
	bool bFound = false;

	for(; pIter != pLast; pIter++)
	{
		CRule *pRule = (*pIter);
		if(pRule->sName == sRuleName)
		{
			bFound = true;

			TActionVector tmpResult;
			if (pRule->Evaluate2(cQuery, tmpResult) == KPDL_SUCCESS && pRule->sPriority > sMaxPriority)
			{
				sMaxPriority = pRule->sPriority;
				tResult = tmpResult;
			}
		}
	}

	if ( bFound == false )
		return KPDL_SUCCESS;

	TActionVectorIterator pActionIter = tResult.begin();
	TActionVectorIterator pActionLast = tResult.end();

	for ( ;pActionIter != pActionLast; pActionIter++ )
	{
		CAction* pAction = (*pActionIter);
		TActionVector tmpResult;
		
		if ( Evaluate2( pAction->sName, cQuery, tmpResult) == KPDL_SUCCESS )
		{
			TActionVectorIterator pActionIter1 = tResult.begin();
			TActionVectorIterator pActionLast1 = tResult.end();

			for ( ;pActionIter1 != pActionLast1; pActionIter1++ )
			{
				CAction* pAction = (*pActionIter1);
				cResult.push_back( pAction );
			}
		}
	}

	return KPDL_UNSUCCESS;
}
