//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	config.cpp
 *  \brief Implementacia triedy CConfig, 
	ktora nacitava konfiguracne parametre hraca a servera zo suboru 
 *  \author KPDL
 *  \date 8.5.2002 14:39:29
 *  \version 

 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CConfig *gpConfig = NULL;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//------------------------------------------------------
/*! \fn	CConfig::CConfig()
 *  \brief Konstruktor, ktory nastavy 
	jednotlive konfiguracne parametre na standartnu hodnotu
 *  \return
 *  \author KPDL 
 *  \date 8.5.2002 14:49:22

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
CConfig::CConfig()
{
	// nastavenie na default hodnoty
    #define CFG_AMOUNT(Param, Value) Param = Value
    #define CFG_ANGLE(Param, Value) Param = Value
    #define CFG_COUNT(Param, Value) Param = Value
    #define CFG_BOOL(Param, Value) Param = Value
    #define CFG_STRING(Param, Value) Param = Value

    #include "Config_var.h"

    #undef CFG_AMOUNT
    #undef CFG_ANGLE
    #undef CFG_COUNT
    #undef CFG_BOOL
    #undef CFG_STRING

	goal_width *= 0.5;
    kickable_area = kickable_margin + ball_size + player_size;
}

//------------------------------------------------------
/*! \fn	void CConfig::Load(const string &sFilename, const char cSeparator)
 *  \brief nacita konfiguraciu zo suboru sFilename
 *  \return
 *  \author KPDL 
 *  \date 8.5.2002 14:51:08

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
void CConfig::Load(
	const string &sFilename, 
	const char cSeparator /* = ':' */
)
{
    string sKey;
	string sValue;
    char sBuffer[256];
	char sKeyBuffer[256];
	char *sValueBuffer;
	char *sEnd;
    FILE *hConfig;

    if((hConfig = fopen(sFilename.c_str(), "rt")) == NULL)
		return;

    while(fgets(sBuffer, 256, hConfig) != NULL)
    {
        if(sBuffer[0] == '#' || sBuffer[0] == '\n')
			continue;

        sValueBuffer = strchr(sBuffer, cSeparator);
        if(sValueBuffer != NULL) 
        {
            sscanf(sBuffer, "%s", sKeyBuffer);
            *sValueBuffer = '\0';
            sValueBuffer ++;
            
			while((*sValueBuffer==' ') || (*sValueBuffer=='\t'))
				sValueBuffer ++;

            sEnd = strchr(sValueBuffer, '\0');
            while(
				(*(sEnd-1) == ' ') || (*(sEnd-1) == '\t') ||
				(*(sEnd-1) == '\n')
			)
				sEnd--;

            *sEnd = '\0';
            sKey = sKeyBuffer;
            sValue = sValueBuffer;

            m_cConfigMap[sKey] = sValue;
        }
    }

    fclose(hConfig);
}

//------------------------------------------------------
/*! \fn	void CConfig::AssignValues()
 *  \brief 	priradenie hodnot podla konfiguracnej mapy vytvorenej
			zo spracovanych konfiguracnych subobrov
 *  \return
 *  \author KPDL 
 *  \date 8.5.2002 14:51:31

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
void CConfig::AssignValues()
{
	// priradenie hodnot podla konfiguracnej mapy vytvorenej
	// zo spracovanych konfiguracnych subobrov
    #define CFG_AMOUNT(Param, Value) Param = GetAmount(#Param, Value)
    #define CFG_ANGLE(Param, Value) Param = GetAngle(#Param, Value)
    #define CFG_COUNT(Param, Value) Param = GetCount(#Param, Value)
    #define CFG_BOOL(Param, Value) Param = GetBool(#Param, Value)
    #define CFG_STRING(Param, Value) Param = GetString(#Param, Value)

    #include "Config_var.h"

    #undef CFG_AMOUNT
    #undef CFG_ANGLE
    #undef CFG_COUNT
    #undef CFG_BOOL
    #undef CFG_STRING

	goal_width *= 0.5;
    kickable_area = kickable_margin + ball_size + player_size;
}

//------------------------------------------------------
/*! \fn	string CConfig::GetString(const string &sID,const string &sValue)
 *  \brief vrati hodnotu konfiguracnej premenej sID
 *  \return
 *  \author KPDL 
 *  \date 8.5.2002 14:52:07
 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
string CConfig::GetString(
	const string &sID, 
	const string &sValue /* = "" */
)
{
	TConfigMapIterator pItem = m_cConfigMap.find(sID);

    return 
		(pItem == m_cConfigMap.end()) ? 
			sValue : 
			(*pItem).second;
}

//------------------------------------------------------
/*! \fn	double CConfig::GetAmount(const string &sID,const double dValue)
 *  \brief 
 *  \return
 *  \author KPDL 
 *  \date 8.5.2002 14:52:14

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
double CConfig::GetAmount(
	const string &sID,
	const double dValue /* = INVALID_AMOUNT */
)
{
	TConfigMapIterator pItem = m_cConfigMap.find(sID);
    
	return 
		(pItem == m_cConfigMap.end()) ? 
			dValue : 
			atof((*pItem).second.c_str());
}

//------------------------------------------------------
/*! \fn	double CConfig::GetAngle(const string &sID, const double dValue)
 *  \brief 
 *  \return
 *  \author KPDL 
 *  \date 8.5.2002 14:52:21
 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
double CConfig::GetAngle(
	const string &sID, 
	const double dValue /* = INVALID_ANGLE */
)
{
	TConfigMapIterator pItem = m_cConfigMap.find(sID);
    return 
		(pItem == m_cConfigMap.end()) ? 
			dValue : 
			DegToRad(atof((*pItem).second.c_str()));
}

//------------------------------------------------------
/*! \fn	int CConfig::GetCount(const string &sID, const int iValue)
 *  \brief 
 *  \return
 *  \author KPDL 
 *  \date 8.5.2002 14:53:13
 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
int CConfig::GetCount(
	const string &sID, 
	const int iValue /* = INVALID_COUNT */
)
{
	TConfigMapIterator pItem = m_cConfigMap.find(sID);
    return
		(pItem == m_cConfigMap.end()) ?
			iValue : 
			atoi((*pItem).second.c_str());
}

//------------------------------------------------------
/*! \fn	bool CConfig::GetBool(const string &sID, const bool bValue) 
 *  \brief 
 *  \return
 *  \author KPDL 
 *  \date 8.5.2002 14:53:21

 *  \bug 
 */
//------------------------------------------------------
// Historia zmien:
//------------------------------------------------------
/*
 */
//------------------------------------------------------
bool CConfig::GetBool(
	const string &sID, 
	const bool bValue /* = false */
)
{
	TConfigMapIterator pItem = m_cConfigMap.find(sID);

	if(pItem == m_cConfigMap.end())
		return bValue;

	string sValue = (*pItem).second;

	return
		(sValue == "on" || sValue == "1" || sValue == "y") ? true :
		(sValue == "off" || sValue == "0" || sValue == "n") ? false :
		bValue;
}
