// PlayerApp.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CPlayerApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CPlayerApp
CPlayerApp::CPlayerApp()
:	// stav pripojenie na server
	m_bConnected(false),
	// konfiguracia ladiacich modulov
	m_eLogSetup(LOG_MASK_EMPTY),
	m_eVisSetup(VIS_SETUP_EMPTY),
	// jeho eminencia hrac
	m_pPlayer(NULL),
	// okienkove haluze
	m_hMDIMenu(NULL),
	m_hMDIAccel(NULL),
	m_pPlayerFrame(NULL),
	m_pLogFrame(NULL),
	m_pVisFrame(NULL)
{
}

CPlayerApp::~CPlayerApp()
{
	// resetneme nastavenie ladiacich modulov, just to be sure
	m_eVisSetup = VIS_SETUP_EMPTY;
	m_eLogSetup = LOG_MASK_EMPTY;
}

KPDL_RESULT CPlayerApp::InitGlobals(const string &sAgentCfgName)
{
	gpLogManager = new CLogManager(m_eLogSetup);
	gpSystemLog = new CLog("System.log", LOG_MASK_SYSTEM);
	gpParserLog = new CLog("Parser.log", LOG_MASK_PARSER);
 	gpConnectionLog = new CLog("Connection.log", LOG_MASK_CONNECT);
	gpBSkillLog = new CLog("BasicSkills.log", LOG_MASK_BSKILL);

    gpConfig = new CConfig;
    gpConfig->Load("server.conf");
    gpConfig->Load(sAgentCfgName);
    gpConfig->AssignValues();

	return KPDL_SUCCESS;
}

KPDL_RESULT CPlayerApp::DestroyGlobals()
{
	if(gpConfig != NULL)
	{
		delete gpConfig;
		gpConfig = NULL;
	}

	if(gpConnectionLog != NULL)
	{
		delete gpConnectionLog;
		gpConnectionLog = NULL;
	}

	if(gpParserLog != NULL)
	{
		delete gpParserLog;
		gpParserLog = NULL;
	}

	if(gpSystemLog != NULL)
	{
		delete gpSystemLog;
		gpSystemLog = NULL;
	}

	if(gpLogManager != NULL)
	{
		delete gpLogManager;
		gpLogManager = NULL;
	}
	
	return KPDL_SUCCESS;
}

BOOL CPlayerApp::InitInstance()
{
	if(!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	AfxEnableControlContainer();
	Enable3dControls();

    CCmdLineInfo cCmdInfo;
	ParseCommandLine(cCmdInfo);

	// inicializacia globalnych premennych
	InitGlobals(cCmdInfo.sAgentCfgName);

	// logovacia konfiguracia
	m_eLogSetup = ELogMask(
		((gpConfig->log_parser) ? LOG_MASK_PARSER : LOG_MASK_EMPTY) | 
		((gpConfig->log_system) ? LOG_MASK_SYSTEM : LOG_MASK_EMPTY) |
		((gpConfig->log_connect) ? LOG_MASK_CONNECT : LOG_MASK_EMPTY)
	);

	// vizualizacna konfiguracia
	m_eVisSetup = EVisSetup(
		((gpConfig->vis_percept) ? VIS_SETUP_PERCEPT : VIS_SETUP_EMPTY) | 
		((gpConfig->vis_ball) ? VIS_SETUP_BALL : VIS_SETUP_EMPTY) |
		((gpConfig->vis_teammate) ? VIS_SETUP_TEAMMATE : VIS_SETUP_EMPTY) |
		((gpConfig->vis_enemy) ? VIS_SETUP_ENEMY : VIS_SETUP_EMPTY) |
		((gpConfig->vis_player) ? VIS_SETUP_PLAYER : VIS_SETUP_EMPTY)
	);
	
	// hlavne okno aplikacie
	m_pMainWnd = m_pPlayerFrame = new CPlayerFrame();
	if(!m_pPlayerFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;

	HINSTANCE hInst = AfxGetResourceHandle();
	m_hMDIMenu  = ::LoadMenu(hInst, MAKEINTRESOURCE(IDR_MAINFRAME));
	m_hMDIAccel = ::LoadAccelerators(hInst, MAKEINTRESOURCE(IDR_MAINFRAME));

	m_pPlayerFrame->ShowWindow(m_nCmdShow);
	m_pPlayerFrame->UpdateWindow();

	// vytvorenie hraca
	m_pPlayer = new CPlayer();

    if(gpConfig->app_autoconnect)
	{
        if(gpConfig->app_goalie)
            OnServerConnectGoalie();
        else
            OnServerConnect();
		
        if(gpConfig->app_autoconnect_dummy)
		{
            Sleep(1000);
            OnServerDisconnect();
        }
    }

	return TRUE;
}

int CPlayerApp::ExitInstance()
{
	// pripojenie na server
	if(m_bConnected)
		OnServerDisconnect();

	// hrac
	if(m_pPlayer != NULL)
	{
		delete m_pPlayer;
		m_pPlayer = NULL;
	}

	// destrukcia globalnych premennych
	DestroyGlobals();

	// okienkove haluze
	if(m_hMDIMenu != NULL)
		DestroyMenu(m_hMDIMenu);
	if(m_hMDIAccel != NULL)
		DestroyAcceleratorTable(m_hMDIAccel);
	
	return CWinApp::ExitInstance();
}

BEGIN_MESSAGE_MAP(CPlayerApp, CWinApp)
	//{{AFX_MSG_MAP(CPlayerApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_SERVER_CONNECT, OnServerConnect)
	ON_UPDATE_COMMAND_UI(ID_SERVER_CONNECT, OnUpdateServerConnect)
	ON_COMMAND(ID_SERVER_CONNECT_GOALIE, OnServerConnectGoalie)
	ON_UPDATE_COMMAND_UI(ID_SERVER_CONNECT_GOALIE, OnUpdateServerConnectGoalie)
	ON_COMMAND(ID_SERVER_DISCONNECT, OnServerDisconnect)
	ON_UPDATE_COMMAND_UI(ID_SERVER_DISCONNECT, OnUpdateServerDisconnect)
	ON_COMMAND(ID_VIEW_LOG, OnViewLog)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LOG, OnUpdateViewLog)
	ON_COMMAND(ID_VIEW_VISUALIZATION, OnViewVisualization)
	ON_UPDATE_COMMAND_UI(ID_VIEW_VISUALIZATION, OnUpdateViewVisualization)
	ON_COMMAND(ID_SETUP_VISUALIZATION, OnSetupVisualization)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	//ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	//ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	//ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

void CPlayerApp::OnAppAbout()
{
	CAboutDlg cAbout;
	cAbout.DoModal();
}

void CPlayerApp::OnServerConnect() 
{
	// hrac vystupuje ako obranca alebo utocnik
	gpConfig->app_goalie = false;
	m_pPlayer->Start();
	m_bConnected = true;
}

void CPlayerApp::OnUpdateServerConnect(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_bConnected);
}

void CPlayerApp::OnServerConnectGoalie() 
{
	// hrac vystupuje ako brankar
	gpConfig->app_goalie = true;
	m_pPlayer->Start();
	m_bConnected = true;
}

void CPlayerApp::OnUpdateServerConnectGoalie(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_bConnected);
}

void CPlayerApp::OnServerDisconnect() 
{
    // pozastavime cinnost hraca
	m_pPlayer->Stop();

	// Zatvorenie vsetkych aktivnych okien
    if(gpVisualView != NULL)
		m_pVisFrame->MDIDestroy();

	m_bConnected = false;
}

void CPlayerApp::OnUpdateServerDisconnect(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bConnected);
}

void CPlayerApp::OnViewLog() 
{
	if(gpLogManager->pLogView != NULL)
	{
		if(m_pLogFrame != m_pPlayerFrame->MDIGetActive())
			m_pLogFrame->MDIActivate();
		else
			m_pLogFrame->MDIDestroy();

		return;
	}

	m_pLogFrame = new CLogFrame();
	m_pLogFrame->SetHandles(m_hMDIMenu, m_hMDIAccel);
	m_pLogFrame->Create(NULL, NULL);
}

void CPlayerApp::OnUpdateViewLog(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(gpLogManager->pLogView != NULL);
}

void CPlayerApp::OnViewVisualization() 
{
	if(gpVisualView != NULL)
	{
		if(m_pVisFrame != m_pPlayerFrame->MDIGetActive())
			m_pVisFrame->MDIActivate();
		else
			m_pVisFrame->MDIDestroy();

		return;
	}

	m_pVisFrame = new CVisualFrame();
	m_pVisFrame->SetHandles(m_hMDIMenu, m_hMDIAccel); 
	m_pVisFrame->Create(NULL, NULL);
	gpVisualView->eVisSetup = m_eVisSetup;
	gpVisualView->SetWorldModel(&m_pPlayer->pWorld->cWorld);
}

void CPlayerApp::OnUpdateViewVisualization(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bConnected);
	pCmdUI->SetCheck(gpVisualView != NULL);
}

void CPlayerApp::OnSetupVisualization()
{
	CVisualSetupDlg cVisSetupDlg(m_eVisSetup);
	
	cVisSetupDlg.DoModal();
	if(gpVisualView != NULL)
	{
		gpVisualView->eVisSetup = m_eVisSetup;
		gpVisualView->Invalidate();
	}
}

/////////////////////////////////////////////////////////////////////////////
// About dialog implementation
CAboutDlg::CAboutDlg()
:	CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	m_sVersion = PLAYER_VERSION;
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Text(pDX, IDC_VERSION_ID, m_sVersion);
	DDV_MaxChars(pDX, m_sVersion, 25);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
