// VisualView.cpp : implementation of the CVisualView class
//

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CVisualView *gpVisualView = NULL;

// Pomocne manipulacie s farbou
// Zmena jasu (neuvazuje pretecenie farby)
COLORREF Bright(const COLORREF tColor, double dModifier)
{
	BYTE bR = BYTE(GetRValue(tColor) * dModifier);
	BYTE bG = BYTE(GetGValue(tColor) * dModifier);
	BYTE bB = BYTE(GetBValue(tColor) * dModifier);
	return RGB(bR, bG, bB);
}

// Zmiesavanie farieb (alfa je z intervalu <0, 1>)
COLORREF Blend(const COLORREF tBack, const COLORREF tFront, double dAlpha)
{
	BYTE bBackR = GetRValue(tBack);
	BYTE bBackG = GetGValue(tBack);
	BYTE bBackB = GetBValue(tBack);
	BYTE bDiffR = BYTE((GetRValue(tFront) - bBackR) * dAlpha);
	BYTE bDiffG = BYTE((GetGValue(tFront) - bBackG) * dAlpha);
	BYTE bDiffB = BYTE((GetBValue(tFront) - bBackB) * dAlpha);
	return RGB(bBackR + bDiffR, bBackG + bDiffG, bBackB + bDiffB);
}

// Presunutie bodu v zadanom uhle o zadanu vzdialenost
inline void Relocate(
	const double dDistance,
	const double dAngle,
	double &dWorldX,
	double &dWorldY
)
{
	dWorldX = dDistance * cos(dAngle);
	dWorldY = dDistance * sin(dAngle);
}

/////////////////////////////////////////////////////////////////////////////
// CVisualView

CVisualView::CVisualView()
:	eVisSetup(VIS_SETUP_FULL),
	cSeeMsg(),
	m_pWorld(NULL),
	m_cDevice(),
	m_cBitmap(),
	m_cFont()
{
	m_iViewPixelRatio = m_iPgSizeX = m_iPgSizeY = m_iPgBorder = 0;
	m_iPgCenterX = m_iViewBorderX = VIS_LEFT_OFFSET;
	m_iPgCenterY = m_iViewBorderY = VIS_TOP_OFFSET;

	CreatePensAndBrushes();

	// Singleton
	gpVisualView = this;
}

CVisualView::~CVisualView()
{
	eVisSetup = VIS_SETUP_EMPTY;
	m_pWorld = NULL;

	gpVisualView = NULL;
}

KPDL_RESULT CVisualView::SetWorldModel(CWorldModel *pWorld)
{
	m_pWorld = pWorld;

	RECT tCoordinates;
	GetClientRect(&tCoordinates);

	return ViewResize(tCoordinates.right, tCoordinates.bottom);
}

BOOL CVisualView::Create(
	LPCTSTR lpszClassName, 
	LPCTSTR lpszWindowName, 
	DWORD dwStyle, 
	const RECT& rect, 
	CWnd* pParentWnd, 
	UINT nID, 
	CCreateContext* pContext
) 
{
	BOOL bRetVal = CWnd::Create(
		lpszClassName, 
		lpszWindowName, 
		dwStyle, 
		rect, 
		pParentWnd, 
		nID, 
		pContext
	);

	CClientDC cDC(this);
	m_cDevice.CreateCompatibleDC(&cDC);
	m_cBitmap.CreateCompatibleBitmap(
		&cDC, 
		::GetSystemMetrics(SM_CXSCREEN),
		::GetSystemMetrics(SM_CYSCREEN)
	); 
	m_cDevice.SetMapMode(MM_TEXT);
	m_cFont.CreateFont(
		VIS_TEXT_FONT_SIZE, 0, 0, 0, FW_DONTCARE,
		FALSE, FALSE, FALSE, ANSI_CHARSET, 
		OUT_DEVICE_PRECIS, CLIP_DEFAULT_PRECIS, 
		DEFAULT_QUALITY, DEFAULT_PITCH, VIS_TEXT_FONT_NAME
	);
	m_cDevice.SelectObject(m_cFont);
	return bRetVal;
}

BOOL CVisualView::PreCreateWindow(CREATESTRUCT& tCS) 
{
	if(!CWnd::PreCreateWindow(tCS))
		return FALSE;

	tCS.dwExStyle |= WS_EX_CLIENTEDGE;
	tCS.style &= ~WS_BORDER;
	tCS.lpszClass = AfxRegisterWndClass(
		CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), 
		HBRUSH(COLOR_WINDOW+1), 
		NULL
	);

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
// Temporal methods

KPDL_RESULT CVisualView::CreatePensAndBrushes()
{
	m_tField.pBackground = new CBrush(RGB(0, 140, 0));
	m_tField.pPlayground = new CBrush(RGB(0, 175, 0)); 
	m_tField.pFlagBrush = new CBrush(RGB(0, 0, 0));
	m_tField.pFlag = new CPen(PS_SOLID, 1, RGB(0, 0, 0));
	m_tField.pGoal = new CPen(PS_SOLID, VIS_FIELD_GOAL_LINE_WIDTH, RGB(0, 0, 0));
	m_tField.pLine = new CPen(PS_SOLID, VIS_FIELD_LINE_WIDTH, RGB(255, 255, 255));

	m_tBall.pBrush = new CBrush(RGB(0, 0, 0));
	m_tBall.pBase = new CPen(PS_SOLID, 1, RGB(240, 140, 0));
	m_tBall.pSpeed = new CPen(PS_SOLID, 1, RGB(0, 100, 255));
	m_tBall.pAccel = new CPen(PS_SOLID, 1, RGB(128, 128, 128));

	m_tAgent.pBrush = new CBrush(RGB(0, 240, 0));
	m_tAgent.pBase = new CPen(PS_SOLID, 1, RGB(0, 240, 0));
	m_tAgent.pSpeed = new CPen(PS_SOLID, 1, RGB(0, 0, 220));
	m_tAgent.pAccel = new CPen(PS_SOLID, 1, RGB(128, 128, 128));
	m_tAgent.pHeadBrush = new CBrush(RGB(0, 160, 255));
	m_tAgent.pHeadPen = new CPen(PS_SOLID, 1, RGB(0, 160, 255));
	m_tAgent.pRotation = new CPen(PS_SOLID, 1, RGB(255, 255, 255));

	m_tTeammate.pBrush = new CBrush(RGB(240, 240, 0));
	m_tTeammate.pBase = new CPen(PS_SOLID, 1, RGB(240, 240, 0));
	m_tTeammate.pSpeed = new CPen(PS_SOLID, 1, RGB( 0, 0, 220));
	m_tTeammate.pAccel = new CPen(PS_SOLID, 1, RGB( 128, 128, 128));
	m_tTeammate.pHeadBrush = new CBrush(RGB(0, 160, 255));
	m_tTeammate.pHeadPen = new CPen(PS_SOLID, 1, RGB(0, 160, 255));
	m_tTeammate.pRotation = new CPen(PS_SOLID, 1, RGB(255, 255, 255));

	m_tEnemy.pBrush = new CBrush(RGB(140, 0, 0));
	m_tEnemy.pBase = new CPen(PS_SOLID, 1, RGB(140, 0, 0));
	m_tEnemy.pSpeed = new CPen(PS_SOLID, 1, RGB(0, 0, 220));
	m_tEnemy.pAccel = new CPen(PS_SOLID, 1, RGB(128, 128, 128));
	m_tEnemy.pHeadBrush = new CBrush(RGB(0, 160, 255));
	m_tEnemy.pHeadPen = new CPen(PS_SOLID, 1, RGB(0, 160, 255));
	m_tEnemy.pRotation = new CPen(PS_SOLID, 1, RGB(255, 255, 255));

	m_tThis.pBrush = new CBrush(RGB(0, 100, 255));
	m_tThis.pBase = new CPen(PS_SOLID, 1, RGB(0, 100, 255));
	m_tThis.pSpeed = new CPen(PS_SOLID, 1, RGB(0, 0, 220));
	m_tThis.pAccel = new CPen(PS_SOLID, 1, RGB(128, 128, 128));
	m_tThis.pHeadBrush = new CBrush(RGB(0, 160, 255));
	m_tThis.pHeadPen = new CPen(PS_SOLID, 1, RGB(0, 160, 255));
	m_tThis.pRotation = new CPen(PS_SOLID, 1, RGB(255, 255, 255));

	return KPDL_SUCCESS;
}

// Vizualizacia vnemov
KPDL_RESULT CVisualView::DrawSeeMsg(
	CDC &cDC, 
	CSeeMsg &cSeeMsg,
	CAgent &cThis
)
{
	int iCount;
	int iIndex;

	if((eVisSetup & VIS_SETUP_PERCEPT) == 0)
		return KPDL_SUCCESS;

	if(!cThis.cPosition.IsValid())
		return KPDL_UNSUCCESS;

	iCount = cSeeMsg.cFlags.size();
	for(iIndex = 0; iIndex < iCount; iIndex++)
		DrawSeenFlag(cDC, cSeeMsg.cFlags[iIndex], cThis);

	iCount = cSeeMsg.cPlayers.size();
	for(iIndex = 0; iIndex < iCount; iIndex++)
		DrawSeenPlayer(cDC, cSeeMsg.cPlayers[iIndex], cThis);

	if(!cSeeMsg.cBall.IsValid())
		return KPDL_ERROR;
	
	DrawSeenBall(cDC, cSeeMsg.cBall, cThis);
	return KPDL_SUCCESS;
}

KPDL_RESULT CVisualView::DrawSeenFlag(
	CDC &cDC, 
	CSeenFlag &cFlag,
	CAgent &cThis
)
{
	int iFlagSize = VIS_PERC_FLAG_MAGNIFIER * m_iViewPixelRatio;
	C2DVector dFlagPos = cThis.cPosition + CAngular2DVector(
		cFlag.dDirection + cThis.dHeadDirection,
		cFlag.dDistance
	);

	return DrawCircle(
		cDC, 
		ConvertCoordX(dFlagPos.GetX()),
		ConvertCoordY(dFlagPos.GetY()), 
		iFlagSize, 
		*m_tField.pFlag, 
		*m_tField.pFlagBrush
	);
}

KPDL_RESULT CVisualView::DrawSeenPlayer(
	CDC &cDC, 
	CSeenPlayer &cAgent,
	CAgent &cThis
)
{
	int iAgentSize = VIS_PERC_PLAYER_MAGNIFIER * m_iViewPixelRatio;
	C2DVector dAgentPos = cThis.cPosition + CAngular2DVector(
		cAgent.dDirection + cThis.dHeadDirection,
		cAgent.dDistance
	);

	SObjectDrawSet *pSet;

	switch(cAgent.eSide)
	{
		case S_None:
			// Unknown
			pSet = &m_tAgent;
			break;

		case S_Home:
			// Teammate
			pSet = &m_tTeammate;
			break;

		case S_Opponent:
			// Enemy
			pSet = &m_tEnemy;
			break;

		default:
			return KPDL_ERROR;
	}

	return DrawCircle(
		cDC, 
		ConvertCoordX(dAgentPos.GetX()),
		ConvertCoordY(dAgentPos.GetY()),
		iAgentSize,
		*(pSet->pBase), 
		*(pSet->pBrush)
	);
}

KPDL_RESULT CVisualView::DrawSeenBall(
	CDC &cDC,
	CSeenObject &cBall,
	CAgent &cThis
)
{
	int iBallSize = VIS_PERC_BALL_MAGNIFIER * m_iViewPixelRatio;
	C2DVector dBallPos = cThis.cPosition + CAngular2DVector(
		cBall.dDirection + cThis.dHeadDirection,
		cBall.dDistance
	);
	
	return DrawCircle(
		cDC, 
		ConvertCoordX(dBallPos.GetX()), 
		ConvertCoordY(dBallPos.GetY()), 
		iBallSize, 
		*m_tBall.pBase, 
		*m_tBall.pBrush
	);
}

KPDL_RESULT CVisualView::DrawWorld(CDC &cDC, CWorldModel &cWorld)
{
	DrawTextInfo(cDC, cWorld);
	DrawField(cDC, cWorld.cField);

	TAgentVectorIterator pIter = cWorld.cUnIdentifiedAgents.begin();
	TAgentVectorIterator pLast = cWorld.cUnIdentifiedAgents.end();
	for(; pIter != pLast; pIter++)
		DrawPlayer(cDC, *(*pIter));

	pIter = cWorld.cIdentifiedAgents.begin();
	pLast = cWorld.cIdentifiedAgents.end();
	for(; pIter != pLast; pIter++)
		DrawPlayer(cDC, *(*pIter));

	// lopta podla vnutorneho sveta
	DrawBall(cDC, cWorld.cBall);

	// vlastny agent
	DrawThis(cDC, cWorld.cThis);
	return KPDL_SUCCESS;
}

KPDL_RESULT CVisualView::DrawTextInfo(
	CDC &cDC,
	CWorldModel &cWorld
)
{
	CString sBuffer;
	int iTextY = VIS_TOP_OFFSET;
	int iTextX = m_iViewBorderX + m_iPgSizeX +
		2 * m_iPgBorder + VIS_TEXT_INFO_BORDER;

	sBuffer.Format(
		"Local time: %s", 
		cWorld.cTime.GetAttribute("allCycles").c_str()
	);
	cDC.TextOut(iTextX, iTextY, sBuffer);
	iTextY += VIS_TEXT_FONT_SIZE;
	
	sBuffer.Format(
		"Play time:  %s",
		cWorld.cTime.GetAttribute("playTime").c_str()
	);
	cDC.TextOut(iTextX, iTextY, sBuffer);
	iTextY += VIS_TEXT_FONT_SIZE;

	sBuffer.Format(
		"Play mode:  %s", 
		cWorld.cTime.GetAttribute("playMode").c_str()
	);
	cDC.TextOut(iTextX, iTextY, sBuffer);
	iTextY += 2*VIS_TEXT_FONT_SIZE;

	sBuffer.Format(
		"Number:     %s",
		cWorld.cThis.GetAttribute("number").c_str()
	);
	cDC.TextOut(iTextX, iTextY, sBuffer);
	iTextY += VIS_TEXT_FONT_SIZE;

	sBuffer.Format(
		"Effort:     %s",
		cWorld.cThis.GetAttribute("effort").c_str()
	);
	cDC.TextOut(iTextX, iTextY, sBuffer);
	iTextY += VIS_TEXT_FONT_SIZE;

	sBuffer.Format(
		"Stamina:    %s", 
		cWorld.cThis.GetAttribute("stamina").c_str()
	);
	cDC.TextOut(iTextX, iTextY, sBuffer);
	iTextY += VIS_TEXT_FONT_SIZE;

	sBuffer.Format(
		"Body dir:   %s",
		cWorld.cThis.GetAttribute("bodyDir").c_str()
	);
	cDC.TextOut(iTextX, iTextY, sBuffer);
	iTextY += VIS_TEXT_FONT_SIZE;

	sBuffer.Format(
		"Head dir:   %s",
		cWorld.cThis.GetAttribute("headDir").c_str()
	);
	cDC.TextOut(iTextX, iTextY, sBuffer);
	iTextY += VIS_TEXT_FONT_SIZE;

	sBuffer.Format(
		"Position:   %s",
		cWorld.cThis.GetAttribute("position").c_str()
	);
	cDC.TextOut(iTextX, iTextY, sBuffer);
	iTextY += VIS_TEXT_FONT_SIZE;

	sBuffer.Format(
		"Speed:      %s", 
		cWorld.cThis.GetAttribute("speed").c_str()
	);
	cDC.TextOut(iTextX, iTextY, sBuffer);
	iTextY += VIS_TEXT_FONT_SIZE;

	return KPDL_SUCCESS;
}

KPDL_RESULT CVisualView::DrawField(
	CDC &cDC,
	CStaticField &cField,
	const bool bStaticFlags /* = true */
)
{
	RECT tFieldRect;

	cDC.SetBkMode(TRANSPARENT);

	tFieldRect.left = m_iViewBorderX;
	tFieldRect.right = m_iViewBorderX + m_iPgSizeX + 2 * m_iPgBorder;
	tFieldRect.bottom = m_iViewBorderY;
	tFieldRect.top = m_iViewBorderY + m_iPgSizeY + 2 * m_iPgBorder;

	// plocha ihriska + okraje
	cDC.FillRect(&tFieldRect, m_tField.pBackground);
	
	tFieldRect.left += m_iPgBorder;
	tFieldRect.top -= m_iPgBorder;
	tFieldRect.bottom += m_iPgBorder;
	tFieldRect.right -= m_iPgBorder;

	// plochu ihriska zobraz odlisnopu farbou
	cDC.FillRect(&tFieldRect, m_tField.pPlayground);

	// kreslenie ciar
	// zobrazenie obvodovych ciar ihriska
	DrawLine(
		cDC, 
		tFieldRect.left, 
		tFieldRect.top, 
		tFieldRect.right + 1, 
		tFieldRect.top, 
		*m_tField.pLine
	);
	
	DrawLine(
		cDC, 
		tFieldRect.left, 
		tFieldRect.bottom, 
		tFieldRect.right + 1, 
		tFieldRect.bottom, 
		*m_tField.pLine
	);

	DrawLine(
		cDC, 
		tFieldRect.left, 
		tFieldRect.top, 
		tFieldRect.left, 
		tFieldRect.bottom, 
		*m_tField.pLine
	);
	
	DrawLine(
		cDC, 
		tFieldRect.right, 
		tFieldRect.top, 
		tFieldRect.right, 
		tFieldRect.bottom, 
		*m_tField.pLine
	);

	// zobrazenie stredovej ciary
	DrawLine(
		cDC, 
		m_iPgCenterX, 
		tFieldRect.top, 
		m_iPgCenterX, 
		tFieldRect.bottom, 
		*m_tField.pLine
	);

	DrawCircleLine(
		cDC, 
		m_iPgCenterX, 
		m_iPgCenterY, 
		gpConfig->center_circle_r * m_iViewPixelRatio, 
		*m_tField.pLine
	);

	// zobrazenie pokutoveho uzemia
	double dM0X, dM0Y, dM1X, dM1Y;
	int iP0X, iP0Y, iP1X, iP1Y;

	// najskor lave pokutove uzemie
	dM0X = dM1X = -cField.dFieldLength + cField.dPenaltyLength;
	dM0Y = -cField.dPenaltyWidth;
	dM1Y = -dM0Y;

	iP0X = iP1X = ConvertCoordX(dM0X);
	iP0Y = ConvertCoordY(dM0Y);
	iP1Y = ConvertCoordY(dM1Y);

	DrawLine(cDC, iP0X, iP0Y, iP1X, iP1Y, *m_tField.pLine);
	DrawLine(cDC, tFieldRect.left, iP1Y, iP1X, iP1Y, *m_tField.pLine);
	DrawLine(cDC, tFieldRect.left, iP0Y, iP0X, iP0Y, *m_tField.pLine);

	// prave pokutove uzemie, y-ove suradnice sa nemenia!
	dM0X = dM1X = cField.dFieldLength - cField.dPenaltyLength;

	iP0X = iP1X =ConvertCoordX(dM0X);

	DrawLine(cDC, iP0X, iP0Y, iP1X, iP1Y, *m_tField.pLine);
	DrawLine(cDC, tFieldRect.right, iP1Y, iP1X, iP1Y, *m_tField.pLine);
	DrawLine(cDC, tFieldRect.right, iP0Y, iP0X, iP0Y, *m_tField.pLine);

	// zozbrazenie branok
	// y-ove suradnice pre obe branky su rovnake
	int iHalfGoalPixelSize = int(cField.dGoalSize * m_iViewPixelRatio);
	int iGoalTopY = m_iPgCenterY + iHalfGoalPixelSize;
	int iGoalBottomY = m_iPgCenterY - iHalfGoalPixelSize;

	// lava branka
	DrawLine(
		cDC, 
		tFieldRect.left - VIS_FIELD_GOAL_LINE_WIDTH / 2, 
		iGoalBottomY, 
		tFieldRect.left - VIS_FIELD_GOAL_LINE_WIDTH / 2, 
		iGoalTopY, 
		*m_tField.pGoal
	);

	// prava branka
	DrawLine(
		cDC, 
		tFieldRect.right + VIS_FIELD_GOAL_LINE_WIDTH / 2, 
		iGoalBottomY,
		tFieldRect.right + VIS_FIELD_GOAL_LINE_WIDTH / 2, 
		iGoalTopY, 
		*m_tField.pGoal
	);

	if(!bStaticFlags)
		return KPDL_SUCCESS;

	// zobrazenie vsetkych vlajok na ihrisku (nulty flag je neplatny)
	for(int iFlag = 0; iFlag < F_Count; iFlag++)
		DrawFlag(cDC, cField.GetFlag((EFlagType)iFlag));

	return KPDL_SUCCESS;
}

KPDL_RESULT CVisualView::DrawFlag(CDC &cDC, const C2DVector &cFlag)
{
	int iX = ConvertCoordX(cFlag.GetX());
	int iY = ConvertCoordY(cFlag.GetY());
	int iFlagSize = VIS_FLAG_MAGNIFIER * m_iViewPixelRatio;

	return DrawCross(cDC, iX, iY, iFlagSize, *m_tField.pFlag);
}

KPDL_RESULT CVisualView::DrawBall(CDC &cDC, CMovingObject &cBall)
{
	int iBallSize = VIS_BALL_MAGNIFIER * m_iViewPixelRatio;
	if(!cBall.cPosition.IsValid())
		return KPDL_UNSUCCESS;

	int iBallX = ConvertCoordX(cBall.cPosition.GetX());
	int iBallY = ConvertCoordY(cBall.cPosition.GetY());
	int iX, iY;

	// vizualizacia udaje o lopte
	if((eVisSetup & VIS_SETUP_BALL) != 0 && cBall.cPosition.IsValid())
	{
		// zozbrazernie rychlosti
		if(cBall.cSpeed.IsValid())
		{
			iX = ConvertCoordX(cBall.cPosition.GetX() + 
				cBall.cSpeed.GetX() * VIS_SPEED_MAGNIFIER);
			iY = ConvertCoordY(cBall.cPosition.GetY() + 
				cBall.cSpeed.GetY() * VIS_SPEED_MAGNIFIER);
			DrawLine(cDC, iBallX, iBallY, iX, iY, *m_tBall.pSpeed);
		}

		// zozbrazenie zrychlenia
		if(cBall.cAcceleration.IsValid())
		{
			iX = ConvertCoordX(cBall.cPosition.GetX() + 
				cBall.cAcceleration.GetX() * VIS_ACCEL_MAGNIFIER);
			iY = ConvertCoordY(cBall.cPosition.GetY() + 
				cBall.cAcceleration.GetY() * VIS_ACCEL_MAGNIFIER);
			DrawLine(cDC, iBallX, iBallY, iX, iY, *m_tBall.pAccel);
		}
	}

	// samotna lopta
	return DrawCircle(
		cDC, 
		iBallX, 
		iBallY, 
		iBallSize, 
		*m_tBall.pBase, 
		*m_tBall.pBrush
	);
}

KPDL_RESULT CVisualView::DrawPlayer(CDC &cDC, CAgent &cAgent)
{
	int iAgentSize = VIS_PLAYER_MAGNIFIER * m_iViewPixelRatio;
	int iAgentX;
	int iAgentY;
	int iX;
	int iY;
	double dX;
	double dY;
	SPlayerDrawSet *pSet = NULL;
	bool bDrawInfo;

	switch(cAgent.eSide)
	{
		case S_None:
			pSet = &m_tAgent;
			bDrawInfo = (eVisSetup & VIS_SETUP_ENEMY) != 0;
			break;

		case S_Home:
			pSet = &m_tTeammate;
			bDrawInfo = (eVisSetup & VIS_SETUP_TEAMMATE) != 0;
			break;

		case S_Opponent:
			pSet = &m_tEnemy;
			bDrawInfo = (eVisSetup & VIS_SETUP_ENEMY) != 0;
			break;

		default :
			return KPDL_ERROR;
	}

	// poloha hraca
	if(cAgent.cPosition.IsValid())
	{
		// if zobrazuj info o spoluhracoch alebo protihracoch
		iAgentX = ConvertCoordX(cAgent.cPosition.GetX());
		iAgentY = ConvertCoordY(cAgent.cPosition.GetY());

		// neurcitost pozicie
		if(bDrawInfo)
		{
			if(IsValidCount(cAgent.iNumber))
			{
				CString sBuffer;

				sBuffer.Format("%d", cAgent.iNumber);
				cDC.TextOut(iAgentX, iAgentY + iAgentSize, sBuffer);
			}

			// zozbrazernie rychlosti
			if(cAgent.cSpeed.IsValid())
			{
				iX = ConvertCoordX(cAgent.cPosition.GetX() + 
					cAgent.cSpeed.GetX() * VIS_SPEED_MAGNIFIER);
				iY = ConvertCoordY(cAgent.cPosition.GetY() + 
					cAgent.cSpeed.GetY() * VIS_SPEED_MAGNIFIER);
				DrawLine(cDC, iAgentX, iAgentY, iX, iY, *(pSet->pSpeed));
			}

			// zozbrazenie zrychlenia
			if(cAgent.cAcceleration.IsValid())
			{
				iX = ConvertCoordX(cAgent.cPosition.GetX() + 
					cAgent.cAcceleration.GetX() * VIS_ACCEL_MAGNIFIER);
				iY = ConvertCoordY(cAgent.cPosition.GetY() + 
					cAgent.cAcceleration.GetY() * VIS_ACCEL_MAGNIFIER);
				DrawLine(cDC, iAgentX, iAgentY, iX, iY, *(pSet->pAccel));
			}
		}

		// zobrazenie pozicie, na konci
		DrawCircle(
			cDC, 
			iAgentX, 
			iAgentY, 
			iAgentSize, 
			*(pSet->pBase), 
			*(pSet->pBrush)
		);

		// zobrazenie natocenia tela
		if(IsValidAngle(cAgent.dBodyDirection))
		{
			Relocate(iAgentSize, cAgent.dBodyDirection, dX, dY);
			DrawLine(
				cDC, 
				iAgentX, 
				iAgentY, 
				iAgentX + dX,
				iAgentY + dY, 
				*(pSet->pRotation)
			);
		}

		// zobrazenie hlavy
		DrawCircle(
			cDC, 
			iAgentX, 
			iAgentY, 
			iAgentSize / 2, 
			*(pSet->pHeadPen), 
			*(pSet->pHeadBrush)
		);

		// zobrazenie natocenia krku
		if(IsValidAngle(cAgent.dHeadDirection))
		{
			// rotacia hlavy
			Relocate(iAgentSize / 2, cAgent.dHeadDirection, dX, dY);
			DrawLine(
				cDC, 
				iAgentX, 
				iAgentY, 
				iAgentX + dX,
				iAgentY + dY, 
				*(pSet->pRotation)
			);
		}
		return KPDL_SUCCESS;
	}

	// nevieme, kde sa nachadza takze ho nekreslime
	return KPDL_UNSUCCESS;
}

KPDL_RESULT CVisualView::DrawThis(CDC &cDC, CAgent &cThis)
{
	int iAgentSize = VIS_PLAYER_MAGNIFIER * m_iViewPixelRatio;
	if(cThis.cPosition.IsValid())
	{
		int iAgentX = ConvertCoordX(cThis.cPosition.GetX()); 
		int iAgentY = ConvertCoordY(cThis.cPosition.GetY());
		int iX, iY;
		double dX, dY;

		if((eVisSetup & VIS_SETUP_PLAYER) != 0)
		{
			// zozbrazernie rychlosti
			if(cThis.cSpeed.IsValid())
			{
				iX = ConvertCoordX(cThis.cPosition.GetX() + 
					cThis.cSpeed.GetX() * VIS_SPEED_MAGNIFIER);
				iY = ConvertCoordY(cThis.cPosition.GetY() + 
					cThis.cSpeed.GetY() * VIS_SPEED_MAGNIFIER);
				DrawLine(cDC, iAgentX, iAgentY, iX, iY, *m_tThis.pSpeed);
			}

			// zozbrazenie zrychlenia
			if(cThis.cAcceleration.IsValid())
			{
				iX = ConvertCoordX(cThis.cPosition.GetX() + 
					cThis.cAcceleration.GetX() * VIS_ACCEL_MAGNIFIER);
				iY = ConvertCoordY(cThis.cPosition.GetY() + 
					cThis.cAcceleration.GetY() * VIS_ACCEL_MAGNIFIER);
				DrawLine(cDC, iAgentX, iAgentY, iX, iY, *m_tThis.pAccel);
			}
		}

		if(IsValidAngle(cThis.dHeadDirection) && (eVisSetup & VIS_SETUP_PLAYER) != 0)
		{
			double dX0, dY0, dX1, dY1;
			double dViewAngle = 0.5 * cThis.dViewAngle;
			int iViewDistance = 
				gpConfig->team_too_far_length * m_iViewPixelRatio;

			Relocate(iViewDistance, cThis.dHeadDirection + dViewAngle, dX0, dY0);
			DrawLine(
				cDC, 
				iAgentX, 
				iAgentY, 
				iAgentX + dX0,
				iAgentY + dY0,
				*m_tThis.pRotation
			);

			Relocate(iViewDistance, cThis.dHeadDirection - dViewAngle, dX1, dY1);
			DrawLine(
				cDC, 
				iAgentX, 
				iAgentY, 
				iAgentX + dX1,
				iAgentY + dY1, 
				*m_tThis.pRotation
			);
			
			DrawArc(
				cDC, 
				iAgentX - iViewDistance, 
				iAgentY - iViewDistance,
				iAgentX + iViewDistance, 
				iAgentY + iViewDistance, 
				iAgentX + dX0, 
				iAgentY + dY0, 
				iAgentX + dX1,
				iAgentY + dY1, 
				*m_tThis.pRotation
			);
		}

		// telo
		DrawCircle(
			cDC, 
			iAgentX, 
			iAgentY, 
			iAgentSize, 
			*m_tThis.pBase, 
			*m_tThis.pBrush
		);

		// rotacia tela
		if(IsValidAngle(cThis.dBodyDirection))
		{
			Relocate(iAgentSize, cThis.dBodyDirection, dX, dY);
			DrawLine(
				cDC, 
				iAgentX, 
				iAgentY, 
				iAgentX + dX,
				iAgentY + dY,
				*m_tThis.pRotation
			);
		}

		// hlava
		DrawCircle(
			cDC, 
			iAgentX, 
			iAgentY, 
			iAgentSize / 2, 
			*m_tThis.pHeadPen, 
			*m_tThis.pHeadBrush
		);

		// rotacia hlavy
		if(IsValidAngle(cThis.dHeadDirection))
		{
			Relocate(iAgentSize / 2, cThis.dHeadDirection, dX, dY);
			DrawLine(
				cDC, 
				iAgentX, 
				iAgentY, 
				iAgentX + dX,
				iAgentY + dY,
				*m_tThis.pRotation
			);
		}

		return KPDL_SUCCESS;
	}

	// kedze neviem poziciu hraca nektreslim ho
	return KPDL_UNSUCCESS;
}

KPDL_RESULT CVisualView::DrawOrientedLine(
	CDC &cDC, 
	const int iStartX, 
	const int iStartY, 
	const int iTargetX, 
	const int iTargetY, 
	CPen &cPen 
)
{
	DrawLine(cDC, iStartX, iStartY, iTargetX, iTargetY, cPen);

	int iX = iTargetX - iStartX;
	int iY = iTargetY - iStartY;
	double dLen = sqrt(double(iX * iX + iY * iY));

	iX = VIS_ORIENTED_LINE_SIZE * iX / dLen;
	iY = VIS_ORIENTED_LINE_SIZE * iY / dLen;

	int iArrowX = iTargetX - iY - 2 * iX;
	int iArrowY = iTargetY + iX - 2 * iY;

	DrawLine(cDC, iTargetX, iTargetY, iArrowX, iArrowY, cPen);

	iArrowX = iTargetX + iY - 2 * iX;
	iArrowY = iTargetY - iX - 2 * iY;

	DrawLine(cDC, iTargetX, iTargetY, iArrowX, iArrowY, cPen);

	return KPDL_SUCCESS;
}

KPDL_RESULT CVisualView::DrawCircle( 
	CDC &cDC, 
	const int iCenterX, 
	const int iCenterY, 
	const int iRadius, 
	CPen &cPen, 
	CBrush &cBrush 
)
{
	cDC.SelectObject(&cPen);
	cDC.SelectObject(&cBrush);
	
	cDC.Ellipse(
		iCenterX - iRadius, 
		iCenterY - iRadius, 
		iCenterX + iRadius, 
		iCenterY + iRadius
	);

	return KPDL_SUCCESS;
}

KPDL_RESULT CVisualView::DrawCircleLine( 
	CDC &cDC, 
	const int iCenterX, 
	const int iCenterY, 
	const int iRadius, 
	CPen &cPen 
)
{
	// create & select transparent brush
	LOGBRUSH tLogBrush;
	tLogBrush.lbStyle = BS_HOLLOW;
	CBrush cBrush;
	cBrush.CreateBrushIndirect(&tLogBrush);

	cDC.SelectObject(&cBrush);
	cDC.SelectObject(&cPen);

	cDC.Ellipse(
		iCenterX - iRadius, 
		iCenterY - iRadius, 
		iCenterX + iRadius, 
		iCenterY + iRadius
	);

	return KPDL_SUCCESS;
}

KPDL_RESULT CVisualView::DrawLine( 
	CDC &cDC, 
	const int iPosX0, 
	const int iPosY0, 
	const int iPosX1, 
	const int iPosY1, 
	CPen &cPen
)
{
	cDC.SelectObject(cPen);

	cDC.MoveTo(iPosX0, iPosY0);
	cDC.LineTo(iPosX1, iPosY1);

	return KPDL_SUCCESS;
}

KPDL_RESULT CVisualView::DrawCross( 
	CDC &cDC, 
	const int iCenterX, 
	const int iCenterY, 
	const int iSize, 
	CPen &cPen 
) 
{
	int iHalfSize = iSize / 2;
	
	DrawLine(cDC, 
		iCenterX - iHalfSize, 
		iCenterY - iHalfSize, 
		iCenterX + iHalfSize, 
		iCenterY + iHalfSize + 1, 
		cPen
	);
	DrawLine(
		cDC, 
		iCenterX - iHalfSize,
		iCenterY + iHalfSize, 
		iCenterX + iHalfSize, 
		iCenterY - iHalfSize - 1, 
		cPen
	);

	return KPDL_SUCCESS;
}

KPDL_RESULT CVisualView::DrawArc( 
	CDC &cDC, 
	const int iPosX0, 
	const int iPosY0, 
	const int iPosX1, 
	const int iPosY1, 
	const int iArcX0, 
	const int iArcY0, 
	const int iArcX1, 
	const int iArcY1, 
	CPen &cPen 
)
{
	cDC.SelectObject(cPen);
	cDC.Arc(iPosX0, iPosY0, iPosX1, iPosY1, iArcX0, iArcY0, iArcX1, iArcY1);
	return KPDL_SUCCESS;
}

/////////////////////////////////////////////////////////////////////////////
// CVisualView message handlers

BEGIN_MESSAGE_MAP(CVisualView,CWnd )
	//{{AFX_MSG_MAP(CVisualView)
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CVisualView::OnPaint() 
{
	// device context for painting
	CPaintDC cDevice(this);
	CRect cRectUpdate;

	cDevice.GetClipBox(&cRectUpdate);
	m_cDevice.SelectObject(&m_cBitmap);
	m_cDevice.SelectClipRgn(0);
	m_cDevice.IntersectClipRect(&cRectUpdate);

	CBrush cBkgndBrush((COLORREF)::GetSysColor(COLOR_WINDOW));
	m_cDevice.SelectObject(&cBkgndBrush);
	m_cDevice.PatBlt(
		cRectUpdate.left, 
		cRectUpdate.top, 
		cRectUpdate.Width(), 
		cRectUpdate.Height(), 
		PATCOPY
	);

	if(m_pWorld != NULL)
	{
		DrawWorld(m_cDevice, *m_pWorld);
		DrawSeeMsg(m_cDevice, cSeeMsg, m_pWorld->cThis);
	}

	cDevice.BitBlt(
		cRectUpdate.left, 
		cRectUpdate.top, 
		cRectUpdate.Width(), 
		cRectUpdate.Height(), 
		&m_cDevice, 
		cRectUpdate.left, 
		cRectUpdate.top, 
		SRCCOPY
	);
}

BOOL CVisualView::OnEraseBkgnd(CDC* pDC) 
{
	return FALSE;
}

KPDL_RESULT CVisualView::ViewResize(const int iWidth, const int iHeight)
{
	if(m_pWorld == NULL)
		return KPDL_UNSUCCESS;

	double dVisWidth = iWidth - VIS_LEFT_OFFSET - VIS_TEXT_INFO_BORDER -
		VIS_TEXT_COLUMN_WIDTH;
	double dVisHeight = iHeight - VIS_TOP_OFFSET - VIS_BOTTOM_BORDER;
	double dBorder = 2.0 * m_pWorld->cField.dBoundary;
	double dFieldWidth = 2.0 * (m_pWorld->cField.dFieldLength + dBorder);
	double dFieldHeight = 2.0 * (m_pWorld->cField.dFieldWidth + dBorder);

	m_iViewPixelRatio = min(
		dVisWidth / dFieldWidth,
		dVisHeight / dFieldHeight
	);
	m_iPgBorder = dBorder * m_iViewPixelRatio;

	m_iPgSizeX = 2 * m_pWorld->cField.dFieldLength * m_iViewPixelRatio;
	m_iPgSizeY = 2 * m_pWorld->cField.dFieldWidth * m_iViewPixelRatio;
	m_iPgCenterX = m_iViewBorderX + m_iPgBorder + m_iPgSizeX / 2;
	m_iPgCenterY = m_iViewBorderY + m_iPgBorder + m_iPgSizeY / 2;

	return KPDL_SUCCESS;
}

void CVisualView::OnSize(UINT uiType, int iX, int iY) 
{
	ViewResize(iX, iY);

	CWnd::OnSize(uiType, iX, iY);
}
