// VisualView.h : interface of the CVisualView class
//
/////////////////////////////////////////////////////////////////////////////

#ifndef _VISUAL_VIEW_
#define _VISUAL_VIEW_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/////////////////////////////////////////////////////////////////////////////
// Sada nastrojov na vizualizaciu ihriska
struct SFieldDrawSet
{
	// Vypln okolia ihriska
	CBrush *pBackground;
	// Vypln ihriska
	CBrush *pPlayground; 
	// Vypln vlajky
	CBrush *pFlagBrush;
	// Pero na vykreslenie vlajky
	CPen *pFlag;
	// Pero na vykreslenie brany
	CPen *pGoal;
	// Pero na vykreslenie ciary
	CPen *pLine;

	SFieldDrawSet()
	:	pBackground(NULL),
		pPlayground(NULL),
		pFlagBrush(NULL),
		pFlag(NULL),
		pGoal(NULL),
		pLine(NULL)
	{
	}

	~SFieldDrawSet()
	{
		if(pBackground != NULL)
		{
			delete pBackground;
			pBackground = NULL;
		}

		if(pPlayground != NULL)
		{
			delete pPlayground;
			pPlayground = NULL;
		}

		if(pFlagBrush != NULL)
		{
			delete pFlagBrush;
			pFlagBrush = NULL;
		}

		if(pFlag != NULL)
		{
			delete pFlag;
			pFlag = NULL;
		}

		if(pGoal != NULL)
		{
			delete pGoal;
			pGoal = NULL;
		}

		if(pLine != NULL)
		{
			delete pLine;
			pLine = NULL;
		}
	}

};

// Sada nastrojov na vizualizaciu objektu
struct SObjectDrawSet
{
	// Zakladne pero
	CPen* pBase;
	// Vypln objektu
	CBrush* pBrush;
	// Vizualizacia rychlosti
	CPen* pSpeed;
	// Vizualizacia zrychlenia
	CPen* pAccel;

	SObjectDrawSet()
	:	pBase(NULL),
		pBrush(NULL),
		pSpeed(NULL),
		pAccel(NULL)
	{
	}

	~SObjectDrawSet()
	{
		if(pBase != NULL)
		{
			delete pBase;
			pBase = NULL;
		}

		if(pBrush != NULL)
		{
			delete pBrush;
			pBrush = NULL;
		}

		if(pSpeed != NULL)
		{
			delete pSpeed;
			pSpeed = NULL;
		}

		if(pAccel != NULL)
		{
			delete pAccel;
			pAccel = NULL;
		}
	}
};

// Sada nastrojov na vizualizaciu hraca
struct SPlayerDrawSet : public SObjectDrawSet
{
	// pero na vykreslenie hlavy
	CPen* pHeadPen;
	// vypln hlavy agenta
	CBrush* pHeadBrush;
	// pero na vizualizaciu rotacie
	CPen* pRotation;

	SPlayerDrawSet()
	:	SObjectDrawSet(),
		pHeadPen(NULL),
		pHeadBrush(NULL),
		pRotation(NULL)
	{
	}

	~SPlayerDrawSet()
	{
		if(pHeadPen != NULL)
		{
			delete pHeadPen;
			pHeadPen = NULL;
		}

		if(pHeadBrush != NULL)
		{
			delete pHeadBrush;
			pHeadBrush = NULL;
		}

		if(pRotation != NULL)
		{
			delete pRotation;
			pRotation = NULL;
		}
	}
};

/////////////////////////////////////////////////////////////////////////////
// CVisualView window

class CVisualView : public CWnd
{
// Attributes
public:
	// Nastavenie zobrazovania
	EVisSetup eVisSetup;
	// vnemy zo servera
	CSeeMsg cSeeMsg;

private:
	// zobrazovany model sveta
	CWorldModel *m_pWorld;

	// nastroje a objekty na vykreslovanie
	// pamatove zariadenie
	CDC m_cDevice;
	// bitmapa v pamati, do ktorej sa zobrazuje
	CBitmap m_cBitmap;
	// font zobrazovaneho textu
	CFont m_cFont;

	// sada nastrojov pre zobrazenie ihriska
	SFieldDrawSet m_tField;
	// sada nastrojov pre loptu
	SObjectDrawSet m_tBall;
	// sada nastrojov pre hraca neznamej strany
	SPlayerDrawSet m_tAgent;
	// sada nastrojov pre spoluhraca
	SPlayerDrawSet m_tTeammate;
	// sada nastrojov pre spoluhraca
	SPlayerDrawSet m_tEnemy;
	// sada nastrojov pre vlastneho hraca
	SPlayerDrawSet m_tThis;

	// staticke parametre vizualizovaneho ihriska
	// prevod medzi jednotkami sveta (metrami) a pixelmi
	int m_iViewPixelRatio;
	// x-ova suradnica laveho okraja ihriska v pixeloch
	int m_iViewBorderX;
	// y-ova suradnica horneho okraja ihriska v pixeloch
	int m_iViewBorderY;

	// dlzka ihriska v pixeloch
	int m_iPgSizeX;
	// sirka ihriska v pixeloch
	int m_iPgSizeY;
	// x-ova suradnica stredu ihriska v pixeloch
	int m_iPgCenterX;
	// y-ova suradnica stredu ihriska v pixeloch
	int m_iPgCenterY;
	// velkost okraja ihriska v pixeloch
	int m_iPgBorder;

public:
	CVisualView();
	virtual ~CVisualView();

	KPDL_RESULT SetWorldModel(CWorldModel *pWorld);

	//{{AFX_VIRTUAL(CVisualView)
	public:
	virtual BOOL Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext = NULL);
	protected:
	virtual BOOL PreCreateWindow(CREATESTRUCT& tCS);
	//}}AFX_VIRTUAL

private:
	KPDL_RESULT CreatePensAndBrushes();

	// vizualizacia vnemov
	KPDL_RESULT CVisualView::DrawSeeMsg(
		CDC &cDC, 
		CSeeMsg &cSeeMsg,
		CAgent &cThis
	);

	KPDL_RESULT DrawSeenFlag(
		CDC &cDC, 
		CSeenFlag &cFlag, 
		CAgent &cThis
	);
	
	KPDL_RESULT DrawSeenPlayer(
		CDC &cDC, 
		CSeenPlayer &cPlayer, 
		CAgent &cThis
	);
	
	KPDL_RESULT DrawSeenBall(
		CDC &cDC, 
		CSeenObject &cBall, 
		CAgent &cThis
	);

	// vizualizacia modelu sveta
	KPDL_RESULT DrawWorld(CDC &cDC, CWorldModel &cWorld);

	KPDL_RESULT DrawTextInfo(
		CDC &cDC,
		CWorldModel &cWorld
	);

	KPDL_RESULT DrawField(
		CDC &DC,
		CStaticField &cField,
		const bool bStaticFlags = true
	);
	KPDL_RESULT DrawFlag(CDC &cDC, const C2DVector &cFlag);
	KPDL_RESULT DrawBall(CDC &cDC, CMovingObject &cBall);
	KPDL_RESULT DrawPlayer(CDC &cDC, CAgent &cPlayer);
	KPDL_RESULT DrawThis(CDC &cDC, CAgent &cVisThis);

	// funkcie na vykreslenie zakladnych elementov sceny
	KPDL_RESULT DrawOrientedLine(
		CDC &cDC, 
		const int iStartX, 
		const int iStartY, 
		const int iTargetX, 
		const int iTargetY, 
		CPen &cPen 
	);
	
	KPDL_RESULT DrawCircle( 
		CDC &cDC, 
		const int iCenterX, 
		const int iCenterY, 
		const int iRadius, 
		CPen &cPen, 
		CBrush &cBrush 
	);
	
	KPDL_RESULT DrawCircleLine( 
		CDC &cDC, 
		const int iCenterX, 
		const int iCenterY, 
		const int iRadius, 
		CPen &cPen 
	);
	
	KPDL_RESULT DrawLine( 
		CDC &cDC, 
		const int iPosX0, 
		const int iPosY0, 
		const int iPosX1, 
		const int iPosY1, 
		CPen &cPen
	);
	
	KPDL_RESULT DrawCross( 
		CDC &cDC, 
		const int iCenterX, 
		const int iCenterY, 
		const int iSize, 
		CPen &cPen 
	); 
	
	KPDL_RESULT DrawArc( 
		CDC &cDC, 
		const int iPosX0, 
		const int iPosY0, 
		const int iPosX1, 
		const int iPosY1, 
		const int iArcX0, 
		const int iArcY0, 
		const int iArcX1, 
		const int iArcY1, 
		CPen &cPen 
	);

	int ConvertCoordX(const double dCoordX)
	{
		return dCoordX * m_iViewPixelRatio + m_iPgCenterX;
	}

	int ConvertCoordY(const double dCoordY)
	{
		int iY = dCoordY * m_iViewPixelRatio;
		return m_iPgCenterY + iY;
	}

protected:

	KPDL_RESULT ViewResize(const int iWidth, const int iHeight);

	//{{AFX_MSG(CVisualView)
	afx_msg void OnPaint();
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnSize(UINT uiType, int iX, int iY);
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
//{{AFX_INSERT_LOCATION}}

#endif // _VISUAL_VIEW_
