#ifndef _2D_LINE_
#define _2D_LINE_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class C2DLine;

typedef vector<C2DLine> T2DLineVector;
typedef T2DLineVector::iterator T2DLineVectorIterator;

class C2DLine
{
protected:
	double m_dAngle;
	double m_dShift;

public:
	inline C2DLine();
	inline C2DLine(const C2DLine &cSource);
	inline C2DLine(const double dAngle, const double dShift);
	inline C2DLine(const C2DVector &cStart, const C2DVector &cEnd);

	inline double GetAngle() const;
	inline double GetShift() const;

	inline C2DLine &SetAngle(const double dAngle);
	inline C2DLine &SetShift(const double dShift);
	inline C2DLine &Set(const C2DVector &cStart, const C2DVector &cEnd);

};

C2DLine::C2DLine()
:	m_dAngle(0.0),
	m_dShift(0.0)
{
}

C2DLine::C2DLine(const C2DLine &cSource)
:	m_dAngle(cSource.m_dAngle),
	m_dShift(cSource.m_dShift)
{
}

C2DLine::C2DLine(const double dAngle, const double dShift)
:	m_dAngle(dAngle),
	m_dShift(dShift)
{
}

C2DLine::C2DLine(const C2DVector &cStart, const C2DVector &cEnd)
{
	double dX = cEnd.GetX() - cStart.GetX();
	double dY = cEnd.GetY() - cStart.GetY();
	
	m_dAngle = atan2(dY , dX);
	//! Pozor! t0t0 nesuhlasi s vypoctom v metode Set 
	m_dShift = cStart.GetX() * dY - cStart.GetY() * dX;
}

double C2DLine::GetAngle() const
{
	return m_dAngle;
}

double C2DLine::GetShift() const
{
	return m_dShift;
}

C2DLine &C2DLine::SetAngle(double dAngle)
{
	m_dAngle = dAngle;
	return *this;
}

C2DLine &C2DLine::SetShift(double dShift)
{
	m_dShift = dShift;
	return *this;
}

C2DLine &C2DLine::Set(const C2DVector &cStart, const C2DVector &cEnd)
{
	double dX = cEnd.GetX() - cStart.GetX();
	double dY = cEnd.GetY() - cStart.GetY();

	m_dAngle = atan2(dY , dX);
	m_dShift = - cStart.GetX() * dY + cStart.GetY() * dX;
}

#endif // _2D_LINE_
