#ifndef _2D_VECTOR_
#define _2D_VECTOR_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class C2DVector;
class CAngular2DVector;

typedef vector<C2DVector> T2DVectorVector;
typedef T2DVectorVector::iterator T2DVectorVectorIterator;

typedef vector<CAngular2DVector> TAngular2DVectorVector;
typedef TAngular2DVectorVector::iterator TAngular2DVectorVectorIterator;

//////////////////////////////////////////////////////////////////////////////
// deklaracia triedy C2DVector reprezentujucej dvojrozmerny vektor 
// v zlozkovom tvare
class C2DVector
{
	friend C2DVector operator * (double dCoeff, const C2DVector &cVect);

// Members
private:
	bool m_bValid;
	double m_dX;
	double m_dY;

// Methods
public:
	inline C2DVector();
	inline C2DVector(const C2DVector &cSource);
	inline C2DVector(const CAngular2DVector &cSource);
	inline C2DVector(const double dX, const double dY, const bool bValid = true);
	
	inline C2DVector &SetX(const double dX);
	inline C2DVector &SetY(const double dY);
	inline C2DVector &SetXY(const double dX, const double dY);
	inline C2DVector &Validate(const bool bValid = true);
	
	inline C2DVector &SetAngleDistance(
		const double dAngle, 
		const double dDistance
	);
	inline C2DVector &SetAngle(const double dAngle);
	inline C2DVector &SetDistance(const double dDistance);

	inline double GetX() const;
	inline double GetY() const;
    inline double GetAngle() const;
	inline double GetDistance() const;
	inline bool IsValid() const;

	inline const C2DVector operator *(const double dCoeff) const;
	inline const C2DVector operator /(const double dCoeff) const;
	inline const C2DVector operator +(const C2DVector &cVect) const;
	inline const C2DVector operator -(const C2DVector &cVect) const;
	inline double operator *(const C2DVector &cVect) const;
	inline double operator &(const C2DVector &cVect) const;

	inline const C2DVector &operator =(const C2DVector &cValue);
	inline const C2DVector &operator =(const CAngular2DVector &cValue);
	inline const C2DVector &operator +=(const C2DVector &cValue);
	inline const C2DVector &operator -=(const C2DVector &cValue);
	inline const C2DVector &operator *=(const double dCoeff);
	inline const C2DVector &operator /=(const double dCoeff);

	virtual string GetAttribute(const string &sName = "") const;
};

//////////////////////////////////////////////////////////////////////////////
// deklaracia triedy CAngular2DVector reprezentujucej dvojrozmerny vektor 
// v polarnej suradnej sustave :)
class CAngular2DVector
{
private:
	bool m_bValid;
	double m_dAngle;
	double m_dDistance;

public:
	inline CAngular2DVector();
	inline CAngular2DVector(const CAngular2DVector &cSource);
	inline CAngular2DVector(
		const double dAngle, 
		const double dDistance, 
		const bool bValid = true
	);
	inline CAngular2DVector(const C2DVector &cSource);

	inline CAngular2DVector &SetAngle(const double dAngle);
	inline CAngular2DVector &SetDistance(const double dDistance);
	inline CAngular2DVector &SetXY(const double dX, const double dY);
	inline CAngular2DVector &Validate(const bool bValid = true);

	inline double GetAngle() const;
	inline double GetDistance() const;
	inline double GetX() const;
	inline double GetY() const;
	inline bool IsValid() const;

	inline const CAngular2DVector &operator =(const CAngular2DVector &cSource);
	inline const CAngular2DVector &operator =(const C2DVector &cSource);

	virtual string GetAttribute(const string &sName = "") const;
};

/*
inline const CAngular2DVector Rot(const C2DVector &cVector, const double dAngle)
{
	return CAngular2DVector(
		cVector.GetAngle() + dAngle, 
		cVector.GetDistance(), 
		cVector.IsValid()
	);
	
	// ina verzia toho isteho
	double dSinAngle = sin(dAngle);
	double dCosAngle = cos(dAngle);
	return C2DVector(
		cVector.GetX() * dCosAngle - cVector.GetY() * dSinAngle, 
		cVector.GetX() * dSinAngle + cVector.GetY() * dCosAngle,
		cVector.IsValid()
	);
}
*/

inline const CAngular2DVector Rot(CAngular2DVector cVector, const double dAngle)
{
	return cVector.SetAngle(cVector.GetAngle() + dAngle);
}

//////////////////////////////////////////////////////////////////////////////
// Implementacia triedy C2DVector
inline C2DVector::C2DVector()
:	m_bValid(false),
	m_dX(0.0),
	m_dY(0.0)
{
}

inline C2DVector::C2DVector(const C2DVector &cSource)
:	m_bValid(cSource.m_bValid),
	m_dX(cSource.m_dX),
	m_dY(cSource.m_dY)
{
}

inline C2DVector::C2DVector(const CAngular2DVector &cSource)
:	m_bValid(cSource.IsValid()),
	m_dX(cSource.GetX()),
	m_dY(cSource.GetY())
{
}

inline C2DVector::C2DVector(
	const double dX, 
	const double dY,
	const bool bValid /* = true */
)
:	m_bValid(true),
	m_dX(dX),
	m_dY(dY)
{
}

inline C2DVector &C2DVector::SetX(const double dX)
{
	m_dX = dX;
	return *this;
}

inline C2DVector &C2DVector::SetY(const double dY)
{
	m_dY = dY;
	return *this;
}

inline C2DVector &C2DVector::SetXY(const double dX, const double dY) 
{
	m_dX = dX;
	m_dY = dY;
	m_bValid = true;
	return *this;
}

inline C2DVector &C2DVector::SetAngleDistance(
	const double dAngle, 
	const double dDistance
)
{
	m_dX = dDistance * cos(dAngle);
	m_dY = dDistance * sin(dAngle);
	return *this;
}

inline C2DVector &C2DVector::SetAngle(const double dAngle)
{
	return SetAngleDistance(dAngle, GetDistance());
}

inline C2DVector &C2DVector::SetDistance(const double dDistance)
{
	return SetAngleDistance(GetAngle(), dDistance);
}

inline C2DVector &C2DVector::Validate(const bool bValid /* = true */)
{
	m_bValid = bValid;
	return *this;
}

inline double C2DVector::GetX() const
{
	// Co ak vektor nie je platny ?? chcelo by to vynimku
	return m_dX;
}

inline double C2DVector::GetY() const
{
	// Co ak vektor nie je platny ?? chcelo by to vynimku
	return m_dY;
}

inline double C2DVector::GetAngle() const
{
	// Co ak vektor nie je platny ?? chcelo by to vynimku
	return atan2(m_dY, m_dX);
}

inline double C2DVector::GetDistance() const
{
	// Co ak vektor nie je platny ?? chcelo by to vynimku
	return sqrt(m_dX * m_dX + m_dY * m_dY);
}

inline bool C2DVector::IsValid() const
{
	return m_bValid;
}

inline const C2DVector C2DVector::operator *(const double dCoeff) const
{
	return C2DVector(m_dX * dCoeff, m_dY * dCoeff, m_bValid);
}

inline const C2DVector C2DVector::operator /(const double dCoeff) const
{
	return C2DVector(m_dX / dCoeff, m_dY / dCoeff, m_bValid);
}

inline const C2DVector C2DVector::operator +(const C2DVector &cVect) const
{
	return C2DVector(
		m_dX + cVect.m_dX, 
		m_dY + cVect.m_dY, 
		m_bValid && cVect.m_bValid
	);
}

inline const C2DVector C2DVector::operator -(const C2DVector &cVect) const
{
	return C2DVector(
		m_dX - cVect.m_dX, 
		m_dY - cVect.m_dY,
		m_bValid && cVect.m_bValid
	);
}

inline double C2DVector::operator *(const C2DVector &cVect) const
{
	// Co ak vektor nie je platny ?? chcelo by to vynimku
	return m_dX * cVect.m_dX + m_dY * cVect.m_dY;
}

inline double C2DVector::operator &(const C2DVector &cVect) const
{
	// Co ak vektor nie je platny ?? chcelo by to vynimku
	return m_dX * cVect.m_dY - m_dY * cVect.m_dX;
}

inline const C2DVector &C2DVector::operator =(const CAngular2DVector &cValue)
{
	m_bValid = cValue.IsValid();
	m_dX = cValue.GetX();
	m_dY = cValue.GetY();
	return *this;
}

inline const C2DVector &C2DVector::operator =(const C2DVector &cValue)
{
	m_bValid = cValue.m_bValid;
	m_dX = cValue.m_dX;
	m_dY = cValue.m_dY;
	return *this;
}

inline const C2DVector &C2DVector::operator +=(const C2DVector &cValue)
{
	m_bValid = m_bValid && cValue.m_bValid;
	m_dX += cValue.m_dX;
	m_dY += cValue.m_dY;
	return *this;
}

inline const C2DVector &C2DVector::operator -=(const C2DVector &cValue)
{
	m_bValid = m_bValid && cValue.m_bValid;
	m_dX -= cValue.m_dX;
	m_dY -= cValue.m_dY;
	return *this;
}

inline const C2DVector &C2DVector::operator *=(const double dCoeff)
{
	m_dX *= dCoeff;
	m_dY *= dCoeff;
	return *this;
}

inline const C2DVector &C2DVector::operator /=(const double dCoeff)
{
	m_dX /= dCoeff;
	m_dY /= dCoeff;
	return *this;
}

inline C2DVector operator *(const double dCoeff, C2DVector cVect)
{
	return cVect * dCoeff;
}

//////////////////////////////////////////////////////////////////////////////
// Implementacia triedy CAngular2DVector
inline CAngular2DVector::CAngular2DVector()
:	m_bValid(false),
	m_dAngle(0.0),
	m_dDistance(0.0)
{
}

inline CAngular2DVector::CAngular2DVector(
	const double dAngle, 
	const double dDistance,
	const bool bValid /* = true */
)
:	m_bValid(bValid),
	m_dAngle(dAngle),
	m_dDistance(dDistance)
{
}

inline CAngular2DVector::CAngular2DVector(const CAngular2DVector &cSource)
:	m_bValid(cSource.m_bValid),
	m_dAngle(cSource.m_dAngle),
	m_dDistance(cSource.m_dDistance)
{
}

inline CAngular2DVector::CAngular2DVector(const C2DVector &cSource)
:	m_bValid(cSource.IsValid()),
	m_dAngle(cSource.GetAngle()),
	m_dDistance(cSource.GetDistance())
{
}

inline CAngular2DVector &CAngular2DVector::SetAngle(const double dAngle)
{
	m_dAngle = dAngle;
	return *this;
}

inline CAngular2DVector &CAngular2DVector::SetDistance(const double dDistance)
{
	m_dDistance = dDistance;
	return *this;
}

inline CAngular2DVector &CAngular2DVector::SetXY(
	const double dX, 
	const double dY
)
{
	m_dAngle = atan2(dY, dX);
	m_dDistance = sqrt(dX * dX + dY * dY);
	return *this;
}

inline CAngular2DVector &CAngular2DVector::Validate(
	const bool bValid /* = true */
)
{
	m_bValid = bValid;
	return *this;
}

inline double CAngular2DVector::GetAngle() const
{
	// Co ak vektor nie je platny ?? chcelo by to vynimku
	return m_dAngle;
}

inline double CAngular2DVector::GetDistance() const
{
	// Co ak vektor nie je platny ?? chcelo by to vynimku
	return m_dDistance;
}

inline double CAngular2DVector::GetX() const
{
	// Co ak vektor nie je platny ?? chcelo by to vynimku
	return m_dDistance * cos(m_dAngle);
}

inline double CAngular2DVector::GetY() const
{
	// Co ak vektor nie je platny ?? chcelo by to vynimku
	return m_dDistance * sin(m_dAngle);
}

inline bool CAngular2DVector::IsValid() const
{
	return m_bValid;
}

inline const CAngular2DVector &CAngular2DVector::operator =(
	const CAngular2DVector &cValue
)
{
	m_bValid = cValue.m_bValid;
	m_dAngle = cValue.m_dAngle;
	m_dDistance = cValue.m_dDistance;
	return *this;
}

inline const CAngular2DVector &CAngular2DVector::operator =(
	const C2DVector &cValue
)
{
	m_bValid = cValue.IsValid();
	m_dAngle = cValue.GetAngle();
	m_dDistance = cValue.GetDistance();
	return *this;
}

#endif // _2D_VECTOR_
