//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	Agent.cpp
 *  \brief Implementacia triedy agenta modelu sveta
 *  \author Mato Spal
 *  \date 19.3.2002 11:15:40
 *  \version 1.0
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_TERM_MAP(CAgent)
	ADD_TERM("sightingBall", SightingBall)
	ADD_TERM("hasBall", HasBall)
	ADD_TERM("ballKickable", BallKickable)
	ADD_TERM("ballNear", BallNear)
	ADD_TERM("ballInterceptPosition", BallInterceptPosition)
	ADD_TERM("nearGoal", NearGoal)
	ADD_TERM("freePathToGoal", FreePathToGoal)
	ADD_TERM("hasFreeTeamMate", HasFreeTeamMate)
	ADD_TERM("hasBetterLocatedTeamMate", HasBetterLocatedTeamMate)
	ADD_TERM("teamMateHasBall", TeamMateHasBall)
	ADD_TERM("teamMateNearBall", TeamMateNearBall)
	ADD_TERM("canPass", CanPass)
	ADD_TERM("canGetPass", CanGetPass)
	ADD_TERM("locationForPass", LocationForPass)
	ADD_TERM("bestLocatedTeamMate", BestLocatedTeamMate)
	ADD_TERM("isFree", IsFree)
	ADD_TERM("freePosition", FreePosition)
END_TERM_MAP

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAgent::CAgent(CWorldModel &cWorld)
:	CMovingObject(
		gpConfig->player_accel_max, 
		gpConfig->player_speed_max, 
		gpConfig->player_decay
	),
	m_cWorld(cWorld),
	eSide(S_None),
	iNumber(INVALID_COUNT),
	eViewQuality(VQ_None),
	dViewAngle(INVALID_ANGLE),
	dEffort(INVALID_AMOUNT),
	dStamina(INVALID_AMOUNT),
	dHeadDirection(INVALID_ANGLE),
	dBodyDirection(INVALID_ANGLE),
	iLastCatch(INVALID_COUNT)
{
}

CAgent::CAgent(const CAgent &cSource, CWorldModel &cWorld)
:	CMovingObject(cSource),
	m_cWorld(cWorld),
	eSide(cSource.eSide),
	iNumber(cSource.iNumber),
	eViewQuality(cSource.eViewQuality),
	dViewAngle(cSource.dViewAngle),
	dEffort(cSource.dEffort),
	dStamina(cSource.dStamina),
	dHeadDirection(cSource.dHeadDirection),
	dBodyDirection(cSource.dBodyDirection),
	iLastCatch(cSource.iLastCatch)
{
}

CAgent &CAgent::operator =(const CAgent &cSource)
{
	CMovingObject::operator =(cSource);
	eSide = cSource.eSide;
	iNumber = cSource.iNumber;
	eViewQuality = cSource.eViewQuality;
	dViewAngle = cSource.dViewAngle;
	dEffort = cSource.dEffort;
	dStamina = cSource.dStamina;
	dHeadDirection = cSource.dHeadDirection;
	dBodyDirection = cSource.dBodyDirection;
	iLastCatch = cSource.iLastCatch;

	return *this;
}

//! t0n0: tuto funkciu bude treba v buducnosti prezriet
// a mozno aj reimplementovat :(
// fcia by mala vracat mieru podobnosti k vnemu cPlayer z pohladu
// hraca cThis v rozmedzi 0.0 az 1.0 (1 = jedna sa o ten isty objekt)
double CAgent::Similarity(
	const CSeenPlayer &cPlayer,
	const CAgent &cThis
)
{
	if(cPlayer.eSide != S_None && eSide != S_None && 
		(eSide == cThis.eSide) && (eSide != cPlayer.eSide)
	)
		return KPDL_UNSUCCESS;

	if(IsValidCount(cPlayer.iNumber) && IsValidCount(iNumber) &&
		iNumber != cPlayer.iNumber
	)
		return KPDL_UNSUCCESS;

	if(cPlayer.eSide != S_None && eSide != S_None && 
		IsValidCount(cPlayer.iNumber) && IsValidCount(iNumber)
	)
		return KPDL_SUCCESS;

	if(IsValidAmount(cPlayer.dDirection) && IsValidAngle(cPlayer.dDistance))
	{
		C2DVector cSeePosition = 
			cThis.cPosition + 
			CAngular2DVector (
				cPlayer.dDirection + cThis.dHeadDirection,
				cPlayer.dDistance
			);

		C2DVector cDelta = cPosition - cSeePosition;

		if(cDelta.GetDistance() > 20.0)
			return KPDL_UNSUCCESS;
		
		//! nejaka rozumna ohodnocujuca funkcia
		// return 0.9 * exp(-cDelta.GetDistance());
	}

	return KPDL_UNSUCCESS;
}

KPDL_RESULT CAgent::Percept(
	const CSeenPlayer &cPlayer,
	const CAgent &cThis
)
{
	if(cPlayer.eSide != S_None)
		eSide = cPlayer.eSide;

	if(IsValidCount(cPlayer.iNumber))
		iNumber=cPlayer.iNumber;

	if(IsValidAngle(cPlayer.dBodyDirection))
		dBodyDirection = AngNormalise(
			cPlayer.dBodyDirection + cThis.dHeadDirection
		);

	if(IsValidAngle(cPlayer.dHeadDirection))
		dHeadDirection = AngNormalise(
			cPlayer.dHeadDirection + cThis.dHeadDirection
		);

	return CMovingObject::Percept(cPlayer, cThis);
}

/******************************************************************************
** Termy
******************************************************************************/

string CAgent::SightingBall()
{
	//! tato funkcia sluzi na odhad, ci nejaky iny hrac moze videt loptu
	// na otestovanie, ci cThis vidi loptu treba vyuzit dotaz priamo na
	// loptu (ball.visible)

	// ak nevidim loptu, nepoznam jej polohu alebo zorny uhol hraca ...
	if(!m_cWorld.cBall.IsValid() ||
		!m_cWorld.cBall.cPosition.IsValid() ||
		!cPosition.IsValid() ||
		!IsValidAngle(dHeadDirection) ||
		!IsValidAngle(dViewAngle)
	)
		// ... sanca je tak 50 na 50
		return string("0.50");

	C2DVector dBallRelPos = RelVectorTo(m_cWorld.cBall.cPosition);
	
	// ak je lopta v zornom uhle hraca alebo jeho viditelnej oblasti ...
	if(Abs(dBallRelPos.GetAngle()) < dViewAngle ||
		dBallRelPos.GetDistance() < gpConfig->visible_distance)
		// ... hrac by mohol vidiet loptu :)
		return string("1.00");

	// ... v oacnom pripade ju nevidi
	return string("0.00");
}

string CAgent::HasBall()
{
	char sBuffer[QUERY_MAX_BUFFER];

	// musime poznat polohu lopty aj hraca
	if(!m_cWorld.cBall.IsValid() ||
		!m_cWorld.cBall.cPosition.IsValid() ||
		!cPosition.IsValid()
	)
		return string("0.50");

	CAgent *pOpponent = NULL;
	double dBallDist = RelVectorTo(m_cWorld.cBall.cPosition).GetDistance();
	double dBestOpponentBallDist = WORLD_MAX_BALL_DIST;
	double dOpponentBallDist;
	double dStepDist;
	double dOpponentStepDist;

	if(dBallDist < gpConfig->kickable_area)
		return string("1.00");

	TAgentVectorIterator pIter = m_cWorld.cIdentifiedAgents.begin();
	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();

	// vzdialenost lopty od superov
	for(; pIter != pLast; pIter ++)
	{
		pOpponent = (*pIter);
		if(pOpponent->eSide == eSide)
			// spoluhracov preskocime ...
			continue;

		// tu by sa hodilo pocitat s intercept polohou lopty
		// resp. s poctom cyklov, za ktoru hrac loptu dobehne
		dOpponentBallDist = 
			(m_cWorld.cBall.cPosition - cPosition).GetDistance();

		if(dBestOpponentBallDist < dOpponentBallDist)
			dBestOpponentBallDist = dOpponentBallDist;
	}

	// dalej predpokladajme, ze hrac sa bude k lopte priblizovat priamo,
	// maximalnou rychlostou, predpokladame, ze sa vsetci hrci pohybuju
	// rovnako rychlo, co je iste zjedodusenie
	dStepDist = dBallDist / gpConfig->player_speed_max;
	dOpponentStepDist = dBestOpponentBallDist / gpConfig->player_speed_max;

	// ak je blizsie super
	if(dOpponentStepDist < dStepDist && 
		dOpponentStepDist < WORLD_MAX_BALL_CONTROL_DIST)
		return "0.00";

	// ak je blizsie tento hrac
	if(dOpponentStepDist > dStepDist && 
		dStepDist < WORLD_MAX_BALL_CONTROL_DIST)
		return "1.00";

	// nejaka funkcia vhodne ohodnocujuca zvysne pripady
	// zatial linearna, pouvazovat o exponencialnej ohodnocovacej funkcii
	//! mozno by bolo vhodne pridat este clen - cim dalej je lopta od hraca,
	// tym mensia kontrola
	sprintf(
		sBuffer, 
		"%.2f", 
		dOpponentStepDist / (dOpponentStepDist + dStepDist)
	);

	return string(sBuffer);
}

string CAgent::BallKickable()
{
	// loptu treba v kazdom pripade vidiet; V podstate by tu mal byt test,
	// ci je lopta este platna, ale ak hrac loptu vidi, potom je
	// tato podmienka splnena
	if(m_cWorld.cBall.iLastSeen > 0 || 
		!m_cWorld.cBall.cPosition.IsValid() ||
		!cPosition.IsValid()
	) 
		return string("0.00");

	double dBallDist = RelVectorTo(m_cWorld.cBall.cPosition).GetDistance();
	if(dBallDist < gpConfig->kickable_area)
		return string("1.00");

	return string("0.00");
}

string CAgent::BallNear()
{
	// test na vzdialenost od lopty nie je taky kriticky ako test, ci je
	// mozne kopnut do lopty; nemusim vyzadovat, aby bola lopta viditelna
	if(!m_cWorld.cBall.IsValid() ||
		!m_cWorld.cBall.cPosition.IsValid() ||
		!cPosition.IsValid()
	)
		return string("0.50");
	
	char sBuffer[QUERY_MAX_BUFFER];
	double dBallDist = (m_cWorld.cBall.cPosition - cPosition).GetDistance();
	double dNormBallDist = 1.0 - dBallDist / WORLD_MAX_BALL_DIST;
	
	if(dNormBallDist < 0.0)
		// lopta je prilis daleko
		return string("0.00");

	sprintf(sBuffer, "%.2f", (1.0 - dNormBallDist));
	return string(sBuffer);
}

string CAgent::BallInterceptPosition()
{
	//! momentalne dotaz vrati aktualnu poziciu lopty
	//  (to iste co dotaz "ball.position")
	//  samozreme musim poznat polohu a ryclost hraca aj lopty
	//  a udaje o lopte este musia byt platne ...
	//  (vlastne aj o hracovi ...)
	char sBuffer[QUERY_MAX_BUFFER];
	C2DVector cBallPos = m_cWorld.cBall.cPosition;

	sprintf(
		sBuffer, 
		"%.2f %.2f", 
		cBallPos.GetX(), 
		cBallPos.GetY()
	);

	return string(sBuffer);
}

string CAgent::NearGoal()
{
	if(eSide == S_None)
		// ... sanca je 50 na 50 ked nepoznam dres hraca 
		return string("0.50");

	char sBuffer[QUERY_MAX_BUFFER];
	EGoalType eGoal = 
		(eSide == S_Home) ? G_Home : G_Opponent;

	//! bolo by dobre uvazovat aj vertikalnu vzdialenost od brany
	double dResult = 
		((cPosition.GetX() - m_cWorld.cField.GetGoal(eGoal).GetX()) /
		 (2 * m_cWorld.cField.dFieldLength));

	sprintf(sBuffer, "%.2f", dResult);
	return string(sBuffer);
}

string CAgent::FreePathToGoal()
{
/*
	char sBuffer[QUERY_MAX_BUFFER];
	EGoalType eGoal = 
		(eSide == S_Home) ? G_Home : 
		(eSide == S_Opponent) ? G_Opponent :
		G_None;

	C2DVector cUpperGoalFlag = m_cWorld.cField.GetGoal(eGoal);
	C2DVector cLowerGoalFlag = cUpperGoalFlag;
	CAgent *pOpponent = NULL;

	cUpperGoalFlag.SetY(- cField.GetGoalSize());
	cLowerGoalFlag.SetY(+ cField.GetGoalSize());
*/
	return string("0.90");
}

string CAgent::HasFreeTeamMate()
{
	if(eSide == S_None)
		// ... sanca je 50 na 50 ked nepoznam dres hraca 
		return string("0.50");

	char sBuffer[QUERY_MAX_BUFFER];
	double dOpponentDist;
	double dNormOpponentDist;
	double dWorstNormOpponentDist = 0.0;
	double dBestNormOpponentDist = 1.0;
	CAgent *pHome = NULL;
	CAgent *pOpponent = NULL;

	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();
	TAgentVectorIterator pHomeIter = m_cWorld.cIdentifiedAgents.begin();
	TAgentVectorIterator pOpponentIter;

	for(; pHomeIter != pLast; pHomeIter ++)
	{
		pHome = (*pHomeIter);

		if(pHome->eSide != eSide || pHome->iNumber == iNumber)
			// protihacov a seba preskakujeme 
			//! (treba test na platnost dresu)
			continue;

		dBestNormOpponentDist = 1.0;
		
		//! v podstate sa tu jedna o test, ci je volny hrac *pHome
		// dalo by sa to zjednodusit :(
		//! museli by sme vytvorit mechanizmus na navratove hodnoty
		// funkcii (AMOUNT, VECTOR, BOOL, COUNT) a ich konverziu
		// na string pre potreby ruleenginu
		//! rovnako by sa dali analyzovat vzdialenosti k protihracom
		// a uhly, ktore blokujuci protihraci vykryvaju
		pOpponentIter = m_cWorld.cIdentifiedAgents.begin();
		for(; pOpponentIter != pLast; pOpponentIter ++)
		{
			pOpponent = (*pOpponentIter);

			if(pOpponent->eSide == eSide)
				// spoluhracov preskocime :)
				continue;

			dOpponentDist = RelVectorTo(pOpponent->cPosition).GetDistance();
			dNormOpponentDist = dOpponentDist / WORLD_MAX_OPPONENT_DIST;

			if(dBestNormOpponentDist < dNormOpponentDist)
				dBestNormOpponentDist = dNormOpponentDist;
		}

		// vlastne sa jedna o akysi minimax - berieme najlepsi vysledok
		// z najhorsich :)
		if(dWorstNormOpponentDist < dBestNormOpponentDist)
			dWorstNormOpponentDist = dBestNormOpponentDist;
	}

	sprintf(sBuffer, "%.2f", 1.0 - dWorstNormOpponentDist);
	return string(sBuffer);
}

string CAgent::HasBetterLocatedTeamMate()
{
	return string("0.00");
}

string CAgent::TeamMateHasBall()
{
	if(eSide == S_None ||
		!m_cWorld.cBall.IsValid() ||
		!m_cWorld.cBall.cPosition.IsValid()
	)
		// ... sanca je 50 na 50 ked nepoznam dres hraca
		// alebo polohu lopty
		return string("0.50");

	char sBuffer[QUERY_MAX_BUFFER];
	CAgent *pAgent = NULL;

	double dResult;
	double dOpponentBallDist = WORLD_MAX_BALL_DIST;
	double dHomeBallDist = WORLD_MAX_BALL_DIST;
	double dBallDist;
	double dStepDist;

	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();
	TAgentVectorIterator pIter = m_cWorld.cIdentifiedAgents.begin();

	for(; pIter != pLast; pIter ++)
	{
		pAgent = (*pIter);

		if(pAgent->iNumber == iNumber || !pAgent->cPosition.IsValid())
			//! seba preskakujeme (treba test na platnost dresu)
			// a tiez agentov, ktorych polohu nepozname
			continue;

		dBallDist = 
			(m_cWorld.cBall.cPosition - pAgent->cPosition).GetDistance();

		// predpokladajme, ze hrac sa bude k lopte priblizovat priamo,
		// maximalnou rychlostou (a ze vsetci hraci behaju rovnako rychlo)
		dStepDist = dBallDist / gpConfig->player_speed_max;

		if(dBallDist < gpConfig->kickable_area)
			dStepDist = 0.0;

		if(pAgent->eSide == eSide)
		{
			// ide o spoluhraca
			if(dHomeBallDist < dStepDist)
				dHomeBallDist = dStepDist;
		}
		else
		{
			// ide o protihraca
			if(dOpponentBallDist < dStepDist)
				dOpponentBallDist = dStepDist;
		}
	}

	// spracovanie vysledkov :)
	if(dOpponentBallDist < dHomeBallDist && 
		dOpponentBallDist < WORLD_MAX_BALL_CONTROL_DIST
	)
	{	
		// blizsie je super
		dResult = 0.0;
	}
	else if(dOpponentBallDist > dHomeBallDist && 
		dHomeBallDist < WORLD_MAX_BALL_CONTROL_DIST
	)
	{
		// blizsie je spoluhrac
		dResult = 1.0;
	}
	else
	{
		// nejaka funkcia vhodne ohodnocujuca zvysne pripady
		// ! mozno vhodne pridat este clen - cim dalej je lopta od hraca,
		// tym mensia kontrola
		dResult = dOpponentBallDist / (dOpponentBallDist + dHomeBallDist);
	}

	sprintf(sBuffer, "%.2f", dResult);
	return string(sBuffer);
}	

string CAgent::TeamMateNearBall()
{
	if(eSide == S_None ||
		!m_cWorld.cBall.IsValid() ||
		!m_cWorld.cBall.cPosition.IsValid()
	)
		// ... sanca je 50 na 50 ked nepoznam dres hraca alebo polohu lopty
		//! mala by sa tu pocitat ball intercept position, takze mozno budeme
		// potrebovat aj rychlost lopty ...
		return string("0.50");

	char sBuffer[QUERY_MAX_BUFFER];
	CAgent *pHome = NULL;
	double dBestNormBallDist = 1.0;
	double dBallDist;
	double dNormBallDist;

	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();
	TAgentVectorIterator pIter = m_cWorld.cIdentifiedAgents.begin();

	for(; pIter != pLast; pIter ++)
	{
		pHome = (*pIter);

		if(pHome->eSide != eSide ||	pHome->iNumber == iNumber ||
			!pHome->cPosition.IsValid()
		)
			// preskakujeme protihracov, seba a tych, ktorym nevieme
			// urcit polohu ...
			continue;

		dBallDist = 
			(m_cWorld.cBall.cPosition - pHome->cPosition).GetDistance();
		dNormBallDist = dBallDist / WORLD_MAX_BALL_DIST;

		if(dBestNormBallDist > dNormBallDist)
			dBestNormBallDist = dNormBallDist;
	}

	sprintf(sBuffer, "%.2f", (1.0 - dBestNormBallDist));
	return string(sBuffer);
}	

string CAgent::CanPass()
{
	if(eSide == S_None)
		return string("0.50");

	char sBuffer[QUERY_MAX_BUFFER];
	double dOpponentDist;
	double dNormOpponentDist;
	double dWorstNormOpponentDist = 0.0;
	double dBestNormOpponentDist = 1.0;
	CAgent *pOpponent = NULL;
	CAgent *pHome = NULL;

	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();
	TAgentVectorIterator pHomeIter = m_cWorld.cIdentifiedAgents.begin();
	TAgentVectorIterator pOpponentIter;

	for(; pHomeIter != pLast; pHomeIter ++)
	{
		pHome = (*pHomeIter);
		if(pHome->eSide != eSide || 
			pHome->iNumber == iNumber ||
			!pHome->cPosition.IsValid()
		)
			//! seba preskakujeme (treba test na platnost dresu)
			// a tiez agentov, ktorych polohu nepozname
			continue;

		dBestNormOpponentDist = 1.0;
		
		pOpponentIter = m_cWorld.cIdentifiedAgents.begin();
		for(; pOpponentIter != pLast; pOpponentIter ++)
		{
			pOpponent = (*pOpponentIter);

			if(pOpponent->eSide == eSide || 
				!pOpponent->cPosition.IsValid()
			)
				// teraz preskakujeme spoluhracov
				continue;

			dOpponentDist = 
				(pOpponent->cPosition - pHome->cPosition).GetDistance();
			dNormOpponentDist = dOpponentDist / WORLD_MAX_OPPONENT_DIST;

			if(dBestNormOpponentDist < dNormOpponentDist)
				dBestNormOpponentDist = dNormOpponentDist;
		}

		if(dWorstNormOpponentDist < dBestNormOpponentDist)
			// ! tu treba este skontrolovat, nakolko je volna draha lopty
			// k najdenemu spoluhracovi
			// (vlastne aj v tomto pripade robime akysi minimax -
			// najhorsi z najlepsich pripadov pre supera)
			dWorstNormOpponentDist = dBestNormOpponentDist;
	}

	sprintf(sBuffer, "%.2f", 1.0 - dWorstNormOpponentDist);
	return string(sBuffer);
}

string CAgent::CanGetPass()
{
	if(eSide == S_None || !cPosition.IsValid())
		// ... sanca je 50 na 50 ked nepoznam dres alebo polohu hraca
		return string("0.50");

	char sBuffer[QUERY_MAX_BUFFER];
	CAgent *pOpponent = NULL;
	double dBestNormOpponentDist = 1.0;
	double dOpponentDist;
	double dNormOpponentDist;

	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();
	TAgentVectorIterator pIter = m_cWorld.cIdentifiedAgents.begin();

	for(; pIter != pLast; pLast ++)
	{
		pOpponent = (*pIter);
		if(pOpponent->eSide == eSide || !pOpponent->cPosition.IsValid())
			// preskakujeme spoluhracov a hracov s neznamou polohou
			continue;

		dOpponentDist = (pOpponent->cPosition - cPosition).GetDistance();
		dNormOpponentDist = dOpponentDist / WORLD_MAX_OPPONENT_DIST;

		if(dBestNormOpponentDist < dNormOpponentDist)
			dBestNormOpponentDist = dNormOpponentDist;
	}

	//! potialto je to IsFree
	//! ak som volny, treba este skontrolovat, ci medzi mnou 
	// a spoluhracom s loptou je volna cesta

	sprintf(sBuffer, "%f",(1.0 - dBestNormOpponentDist));
	return string(sBuffer);
}

string CAgent::LocationForPass()
{
	//! treba este dorobit
	return "0.00 0.00";
}

string CAgent::BestLocatedTeamMate()
{
	//! treba este dorobit
	return "1";
}

string CAgent::IsFree()
{
	if(eSide == S_None || !cPosition.IsValid())
		// ... sanca je 50 na 50 ked nepoznam dres alebo polohu hraca
		return string("0.50");

	char sBuffer[QUERY_MAX_BUFFER];
	CAgent *pOpponent = NULL;
	double dBestNormOpponentDist = 1.0;
	double dOpponentDist;
	double dNormOpponentDist;

	TAgentVectorIterator pLast = m_cWorld.cIdentifiedAgents.end();
	TAgentVectorIterator pIter = m_cWorld.cIdentifiedAgents.begin();

	for(; pIter != pLast; pLast ++)
	{
		pOpponent = (*pIter);
		if(pOpponent->eSide == eSide || !pOpponent->cPosition.IsValid())
			// preskakujeme spoluhracov a hracov s neznamou polohou
			continue;

		dOpponentDist = (pOpponent->cPosition - cPosition).GetDistance();
		dNormOpponentDist = dOpponentDist / WORLD_MAX_OPPONENT_DIST;

		if(dBestNormOpponentDist < dNormOpponentDist)
			dBestNormOpponentDist = dNormOpponentDist;
	}

	sprintf(sBuffer, "%.2f", (1.0 - dBestNormOpponentDist));
	return string(sBuffer);
}

string CAgent::FreePosition()
{
	//! treba este dorobit
	return "0.00 0.00";
}

//======================================================
// CAgent : implementacia CQuery
//======================================================
string CAgent::GetAttribute(const string &sName)
{
	char sBuffer[QUERY_MAX_BUFFER] = QUERY_NOT_AVAILABLE;

	//-------------------------------------------------------
	// stamina
	//-------------------------------------------------------
	if(sName == "stamina")
	{
		if(IsValidAmount(dStamina))
			sprintf(sBuffer, "%.2f", dStamina);
	}
	//-------------------------------------------------------
	// stamina
	//-------------------------------------------------------
	else if(sName == "number")
	{
		if(IsValidCount(iNumber))
			sprintf(sBuffer, "%02d", iNumber);
	}
	//-------------------------------------------------------
	// headDirection
	//-------------------------------------------------------
	else if(sName == "headDir")
	{
		if(IsValidAngle(dHeadDirection))
			sprintf(sBuffer, "%.2f", RadToDeg(dHeadDirection));
	}
	//-------------------------------------------------------
	// bodyDirection
	//-------------------------------------------------------
	else if(sName == "bodyDir")
	{
		if(IsValidAngle(dBodyDirection))
			sprintf(sBuffer, "%.2f", RadToDeg(dBodyDirection));
	}
	//-------------------------------------------------------
	// effort
	//-------------------------------------------------------
	else if(sName == "effort")
	{
		if(IsValidAmount(dEffort))
			sprintf(sBuffer, "%.2f", dEffort);
	}
	//-------------------------------------------------------
	// viewAngle
	//-------------------------------------------------------
	else if(sName == "viewAngle")
	{
		if(IsValidAngle(dViewAngle))
			sprintf(sBuffer, "%.2f", RadToDeg(dViewAngle));
	}
	else
		return CMovingObject::GetAttribute(sName);

	return string(sBuffer);
}
