//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	MovingObject.cpp
 *  \brief Implementacia triedy pohybujuceho sa objektu v modeli sveta
 *  \author Mato Spal
 *  \date 19.3.2002 10:46:58
 *  \version 1.0
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMovingObject::CMovingObject(
	const double dMaxAcceleration,
	const double dMaxSpeed,
	const double dSpeedDecay
)
:	CQuery(),
	// konfiguracia
	m_dMaxAcceleration(dMaxAcceleration),
	m_dMaxSpeed(dMaxSpeed),
	m_dSpeedDecay(dSpeedDecay),
	// predchadzajuca pozicia
	cPrevPosition(),
	// aktualny stav
	cPosition(),
	cSpeed(),
	cAcceleration(),
	// pocet cyklov od posledneho zhliadnutia objektu
	iLastSeen(WORLD_MAX_LAST_SEEN)
{
}

CMovingObject::CMovingObject(const CMovingObject& cSource)
:	CQuery(),
	// konfiguracia
	m_dMaxAcceleration(cSource.m_dMaxAcceleration),
	m_dMaxSpeed(cSource.m_dMaxSpeed),
	m_dSpeedDecay(cSource.m_dSpeedDecay),
	// predchadzajuca pozicia
	cPrevPosition(cSource.cPrevPosition),
	// aktualny stav
	cPosition(cSource.cPosition),
	cSpeed(cSource.cSpeed),
	cAcceleration(cSource.cAcceleration),
	// pocet cyklov od posledneho zhliadnutia objektu
	iLastSeen(cSource.iLastSeen)
{
}

CMovingObject &CMovingObject::operator =(const CMovingObject &cSource)
{
	m_dMaxAcceleration = cSource.m_dMaxAcceleration;
	m_dMaxSpeed = cSource.m_dMaxSpeed;
	m_dSpeedDecay = cSource.m_dSpeedDecay;
	cPrevPosition = cSource.cPrevPosition;
	cPosition = cSource.cPosition;
	cSpeed = cSource.cSpeed;
	cAcceleration = cSource.cAcceleration;
	iLastSeen = cSource.iLastSeen;

	return *this;
}


KPDL_RESULT CMovingObject::Precompute()
{
	// zapametanie predchadzajucej pozicie
	cPrevPosition = cPosition;

	// zrychlenie
	if(cAcceleration.GetDistance() > m_dMaxAcceleration)
		cAcceleration.SetDistance(m_dMaxAcceleration);
	cSpeed += cAcceleration;
	
	// presun
	if(cSpeed.GetDistance() > m_dMaxSpeed)
		cSpeed.SetDistance(m_dMaxSpeed);
	cPosition += cSpeed;

	// spomalenie
	cSpeed *= m_dSpeedDecay;
	cAcceleration.SetXY(0.0, 0.0);

	iLastSeen ++;
	return KPDL_SUCCESS;
}

KPDL_RESULT CMovingObject::Percept(
	const CSeenObject &cObject,
	const CAgent &cThis
)
{
	// predpocitame rychlost ...
	cSpeed = (cPosition - cPrevPosition) * m_dSpeedDecay;
	// ... odlozime staru polohu ...
	cPrevPosition = cPosition;
	// a aktualnu polohu zrusime ...
	cPosition.Validate(false);

	if(! cObject.IsValid())
		return KPDL_UNSUCCESS;

	if(IsValidAmount(cObject.dDistance) && 
		IsValidAngle(cObject.dDirection)
	)
	{
		cPosition = 
			cThis.cPosition + 
			CAngular2DVector(
				cObject.dDirection + cThis.dHeadDirection,
				cObject.dDistance
			);

		if(IsValidAmount(cObject.dDistanceChange) &&
			IsValidAngle(cObject.dDirectionChange)
		)
		{
			// ak su zname hodnoty o zmenach parametrov zo servera,
			// mame sancu vypocitat rychlost presnejsie ...
			CAngular2DVector cNormal(
				cObject.dDirection + cThis.dHeadDirection,
				cObject.dDistanceChange
			);

			CAngular2DVector cTangent(
				cObject.dDirection + cThis.dHeadDirection + 0.5 * PI,
				cObject.dDirectionChange * cObject.dDistance
			);

			cSpeed = cThis.cSpeed + cNormal + cTangent;
		}
	}

	// preskusame ohranicenie rychlosti
	if(cSpeed.IsValid() && cSpeed.GetDistance() > m_dMaxSpeed)
		cSpeed.SetDistance(m_dMaxSpeed);

	iLastSeen = 0;
	return KPDL_SUCCESS;
}

//======================================================
// CMovingObject : implementacia CQuery
//======================================================
string CMovingObject::GetAttribute(const string &sName)
{
	if(sName == "visible")
		// bud som objekt videl v tomto cykle alebo nie
		// atribut visible bude dostupny vzdy
		return string((iLastSeen == 0)? "1.00" : "0.00");

	if(sName == "valid")
		return string((IsValid())? "1.00" : "0.00");

	if(!IsValid())
		return string(QUERY_NOT_AVAILABLE);

	string sHeader = sName;
	string sTail = CQuery::SplitTail(sHeader);

	if(sHeader == "position")
		return cPosition.GetAttribute(sTail);
	
	if(sHeader == "speed")
		return cSpeed.GetAttribute(sTail);

	if(sHeader == "acceleration")
		return cAcceleration.GetAttribute(sTail);

	return string(QUERY_NOT_AVAILABLE);
}
