// StaticField.cpp: implementation of the CStaticField class.
//
//////////////////////////////////////////////////////////////////////

#include "stdAfx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStaticField::CStaticField(
	const double dFieldLength,
	const double dFieldWidth,
	const double dPenaltyLength,
	const double dPenaltyWidth,
	const double dGoalSize,
	const double dBoundary
)
:	dFieldLength(dFieldLength),
	dFieldWidth(dFieldWidth),
	dPenaltyLength(dPenaltyLength),
	dPenaltyWidth(dPenaltyWidth),
	dGoalSize(dGoalSize),
	dBoundary(dBoundary)
{
	CreateFlags();
    CreateGoals();
    CreateLines();
}

CStaticField::~CStaticField()
{
	m_cFlags.clear();
	m_cGoals.clear();
	m_cLines.clear();
}

KPDL_RESULT CStaticField::CreateFlags()
{
	m_cFlags.reserve(F_Count);

	// stred ihriska
	m_cFlags[F_Center].SetXY(0.0, 0.0);

	// kraje hracej plochy
	m_cFlags[F_HomeTop].SetXY(-dFieldLength, -dFieldWidth);
	m_cFlags[F_HomeBottom].SetXY(-dFieldLength, dFieldWidth);
	m_cFlags[F_OpponentTop].SetXY(dFieldLength, -dFieldWidth);
	m_cFlags[F_OpponentBottom].SetXY(dFieldLength, dFieldWidth);
	m_cFlags[F_CenterTop].SetXY(0.0, -dFieldWidth);
	m_cFlags[F_CenterBottom].SetXY(0.0, dFieldWidth);
	
	// penaltove uzemie
	m_cFlags[F_PenaltyHomeTop].
		SetXY(-dFieldLength + dPenaltyLength, -dPenaltyWidth);
	m_cFlags[F_PenaltyHomeCenter].
		SetXY(-dFieldLength + dPenaltyLength, 0.0);
	m_cFlags[F_PenaltyHomeBottom].
		SetXY(-dFieldLength + dPenaltyLength, dPenaltyWidth);
	m_cFlags[F_PenaltyOpponentTop].
		SetXY(dFieldLength - dPenaltyLength, -dPenaltyWidth);
	m_cFlags[F_PenaltyOpponentCenter].
		SetXY(dFieldLength - dPenaltyLength, 0.0);
	m_cFlags[F_PenaltyOpponentBottom].
		SetXY(dFieldLength - dPenaltyLength, dPenaltyWidth);

	// branky
	m_cFlags[F_GoalHomeTop].SetXY(-dFieldLength, -dGoalSize);
	m_cFlags[F_GoalHomeBottom].SetXY(-dFieldLength, dGoalSize);
	m_cFlags[F_GoalOpponentTop].SetXY(dFieldLength, -dGoalSize);
	m_cFlags[F_GoalOpponentBottom].SetXY(dFieldLength, dGoalSize);

	// pomocne zastavky ihriska
	// horizontalne
	m_cFlags[F_TopHome50].SetXY(-50.0, -dFieldWidth - dBoundary);
	m_cFlags[F_TopHome40].SetXY(-40.0, -dFieldWidth - dBoundary);
	m_cFlags[F_TopHome30].SetXY(-30.0, -dFieldWidth - dBoundary);
	m_cFlags[F_TopHome20].SetXY(-20.0, -dFieldWidth - dBoundary);
	m_cFlags[F_TopHome10].SetXY(-10.0, -dFieldWidth - dBoundary);
	m_cFlags[F_Top0].SetXY(0.0, -dFieldWidth - dBoundary);
	m_cFlags[F_TopOpponent10].SetXY(10.0, -dFieldWidth - dBoundary);
	m_cFlags[F_TopOpponent20].SetXY(20.0, -dFieldWidth - dBoundary);
	m_cFlags[F_TopOpponent30].SetXY(30.0, -dFieldWidth - dBoundary);
	m_cFlags[F_TopOpponent40].SetXY(40.0, -dFieldWidth - dBoundary);
	m_cFlags[F_TopOpponent50].SetXY(50.0, -dFieldWidth - dBoundary);
	m_cFlags[F_BottomHome50].SetXY(-50.0, dFieldWidth + dBoundary);
	m_cFlags[F_BottomHome40].SetXY(-40.0, dFieldWidth + dBoundary);
	m_cFlags[F_BottomHome30].SetXY(-30.0, dFieldWidth + dBoundary);
	m_cFlags[F_BottomHome20].SetXY(-20.0, dFieldWidth + dBoundary);
	m_cFlags[F_BottomHome10].SetXY(-10.0, dFieldWidth + dBoundary);
	m_cFlags[F_Bottom0].SetXY(0.0, dFieldWidth + dBoundary);
	m_cFlags[F_BottomOpponent10].SetXY(10.0, dFieldWidth + dBoundary);
	m_cFlags[F_BottomOpponent20].SetXY(20.0, dFieldWidth + dBoundary);
	m_cFlags[F_BottomOpponent30].SetXY(30.0, dFieldWidth + dBoundary);
	m_cFlags[F_BottomOpponent40].SetXY(40.0, dFieldWidth + dBoundary);
	m_cFlags[F_BottomOpponent50].SetXY(50.0, dFieldWidth + dBoundary);

	// vertikalne
	m_cFlags[F_HomeTop30].SetXY(-dFieldLength - dBoundary, -30.0);
	m_cFlags[F_HomeTop20].SetXY(-dFieldLength - dBoundary, -20.0);
	m_cFlags[F_HomeTop10].SetXY(-dFieldLength - dBoundary, -10.0);
	m_cFlags[F_Home0].SetXY(-dFieldLength - dBoundary, 0.0);
	m_cFlags[F_HomeBottom10].SetXY(-dFieldLength - dBoundary, 10.0);
	m_cFlags[F_HomeBottom20].SetXY(-dFieldLength - dBoundary, 20.0);
	m_cFlags[F_HomeBottom30].SetXY(-dFieldLength - dBoundary, 30.0);
	m_cFlags[F_OpponentTop30].SetXY(dFieldLength + dBoundary, -30.0);
	m_cFlags[F_OpponentTop20].SetXY(dFieldLength + dBoundary, -20.0);
	m_cFlags[F_OpponentTop10].SetXY(dFieldLength + dBoundary, -10.0);
	m_cFlags[F_Opponent0].SetXY(dFieldLength + dBoundary, 0.0);
	m_cFlags[F_OpponentBottom10].SetXY(dFieldLength + dBoundary, 10.0);
	m_cFlags[F_OpponentBottom20].SetXY(dFieldLength + dBoundary, 20.0);
	m_cFlags[F_OpponentBottom30].SetXY(dFieldLength + dBoundary, 30.0);

	return KPDL_SUCCESS;
}

KPDL_RESULT CStaticField::CreateGoals()
{
	m_cGoals.reserve(G_Count);
	m_cGoals[G_Home].SetXY(-dFieldLength, 0.0);
	m_cGoals[G_Opponent].SetXY(dFieldLength, 0.0);

	return KPDL_SUCCESS;
}

KPDL_RESULT CStaticField::CreateLines()
{
	m_cLines.reserve(L_Count);

	m_cLines[L_Home] = C2DLine(
		C2DVector(-dFieldLength, dFieldWidth), 
		C2DVector(-dFieldLength, -dFieldWidth)
	);
	
	m_cLines[L_Opponent] = C2DLine(
		C2DVector(dFieldLength, dFieldWidth), 
		C2DVector(dFieldLength, -dFieldWidth)
	);
	
	m_cLines[L_Bottom] = C2DLine(
		C2DVector(-dFieldLength, -dFieldWidth), 
		C2DVector(dFieldLength, -dFieldWidth)
	);
    
	m_cLines[L_Top] = C2DLine(
		C2DVector(-dFieldLength, dFieldWidth), 
		C2DVector(dFieldLength, dFieldWidth)
	);

	return KPDL_SUCCESS;
}
