//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	WorldModel.cpp
 *  \brief Implementacia triedy modelu sveta
 *  \author Mato Spal
 *  \date 19.3.2002 11:30:39
 *  \version 1.0
 *  \bug 
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Historia zmien:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include "stdafx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_TERM_MAP(CWorldModel)
	ADD_TERM("someoneHasBall", SomeoneHasBall)
	ADD_TERM("opponentGoal", OpponentGoal)
	ADD_TERM("isPassMessage", IsPassMessage)
END_TERM_MAP

#define EPS (1.0e-10)

inline double WeightFn(const double dAmount, double dQuantizeStep)
{
	// t0n0: vychadzal som z quantizacneho vztahu v sserveri
	// tim4 tu mal podla mna chybne vztahy
	dQuantizeStep *= 0.5;
	double dLogAmount = log(dAmount + EPS);
	double dDiffMax = exp(dLogAmount + dQuantizeStep);
	double dDiffMin = exp(dLogAmount - dQuantizeStep);
	double dDiffusion = 0.5 * (dDiffMax - dDiffMin + 0.1);

	// t0n0: hodnotu 0.01 do nasledujuceho vypoctu vlozil tim4,
	// pravdepodobne riesila pripad, ked dDiffusion je 0.
	// (podla predchadzajuceho vypoctu bude dDiffusion vzdy aspon 0.02 :)
	return 1.0 / (0.01 + dDiffusion);
}

inline double minQ(const double dAmount, const double dQuantizeStep)
{
	return dAmount - 0.5 * dQuantizeStep;
}

inline double maxQ(const double dAmount, const double dQuantizeStep)
{
	return dAmount + 0.5 * dQuantizeStep;
}

inline double minEQ(const double dAmount, const double dQuantizeStep)
{
	return minQ(exp(minQ(log(dAmount + EPS), dQuantizeStep)), 0.1);
}

inline double maxEQ(const double dAmount, const double dQuantizeStep)
{
	return maxQ(exp(maxQ(log(dAmount + EPS), dQuantizeStep)), 0.1);
}
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWorldModel::CWorldModel(CStaticField &cField)
:	CQuery(),
	cTime(),
	cField(cField),
	cBall(
		gpConfig->ball_accel_max, 
		gpConfig->ball_speed_max, 
		gpConfig->ball_decay
	),
	cIdentifiedAgents(),
	cUnIdentifiedAgents(),
	cThis(*this)
{
}

CWorldModel::CWorldModel(CWorldModel& cSource)
:	CQuery(),
	cTime(cSource.cTime),
	cField(cSource.cField),
	cBall(cSource.cBall),
	cIdentifiedAgents(),
	cUnIdentifiedAgents(),
	cThis(cSource.cThis, *this)
{
	TAgentVectorIterator pIter = cSource.cIdentifiedAgents.begin();
	TAgentVectorIterator pLast = cSource.cIdentifiedAgents.end();
	for(; pIter != pLast ; pIter++)
		cIdentifiedAgents.push_back(new CAgent(**pIter, *this));

	pIter = cSource.cUnIdentifiedAgents.begin();
	pLast = cSource.cUnIdentifiedAgents.end();
	for(; pIter != pLast ; pIter++)
		cUnIdentifiedAgents.push_back(new CAgent(**pIter, *this));
}

CWorldModel::~CWorldModel()
{
	TAgentVectorIterator pIter = cIdentifiedAgents.begin();
	TAgentVectorIterator pLast = cIdentifiedAgents.end();

	for(; pIter != pLast ; pIter++)
		delete (*pIter);

	pIter = cUnIdentifiedAgents.begin();
	pLast = cUnIdentifiedAgents.end();

	for(; pIter != pLast ; pIter++)
		delete (*pIter);

	cIdentifiedAgents.clear();
	cUnIdentifiedAgents.clear();
}

CWorldModel &CWorldModel::operator =(CWorldModel& cSource)
{
	ClearCache();
	cTime = cSource.cTime;
	cBall = cSource.cBall;
	cThis = cSource.cThis;
	
	// zrusime starych hracov
	TAgentVectorIterator pIter = cIdentifiedAgents.begin();
	TAgentVectorIterator pLast = cIdentifiedAgents.end();
	for(; pIter != pLast ; pIter++)
		delete (*pIter);

	pIter = cUnIdentifiedAgents.begin();
	pLast = cUnIdentifiedAgents.end();
	for(; pIter != pLast ; pIter++)
		delete (*pIter);

	cIdentifiedAgents.clear();
	cUnIdentifiedAgents.clear();

	// skopirujeme novych
	pIter = cSource.cIdentifiedAgents.begin();
	pLast = cSource.cIdentifiedAgents.end();
	for(; pIter != pLast ; pIter++)
		cIdentifiedAgents.push_back(new CAgent(**pIter, *this));

	pIter = cSource.cUnIdentifiedAgents.begin();
	pLast = cSource.cUnIdentifiedAgents.end();
	for(; pIter != pLast ; pIter++)
		cUnIdentifiedAgents.push_back(new CAgent(**pIter, *this));

	return *this;
}

KPDL_RESULT CWorldModel::Precompute()
{
	TAgentVectorIterator pIter = cIdentifiedAgents.begin();
	TAgentVectorIterator pLast = cIdentifiedAgents.end();

	for(; pIter != pLast; pIter++)
		(*pIter)->Precompute();

	pIter = cUnIdentifiedAgents.begin();
	pLast = cUnIdentifiedAgents.end();

	for(; pIter != pLast; pIter++)
		(*pIter)->Precompute();

	cThis.Precompute();
	cBall.Precompute();

	return KPDL_SUCCESS;
}

//! t0n0: HeadDirection
KPDL_RESULT CWorldModel::ProcessSenseBody(CSenseBody &cSenseBody)
{
	if(cSenseBody.dSpeed == 0.0)
	{
		cThis.dHeadDirection = -cSenseBody.dSpeedDirection;
	}
	cThis.dBodyDirection = cThis.dHeadDirection - cSenseBody.dHeadDirection;

	cThis.dViewAngle = cSenseBody.dViewWidth;
	cThis.cSpeed.SetAngleDistance(
		cSenseBody.dSpeedDirection + cThis.dHeadDirection, 
		cSenseBody.dSpeed
	).Validate();

	cThis.dStamina = cSenseBody.dStamina;
	cThis.dEffort = cSenseBody.dEffort;
//	cThis.iLastSeen = 0;

	cTime.iAllCycles ++;
	cTime.iPlayTime = cSenseBody.iPlayTime;
	gpBSkillLog->Write(
		true, "Sense Body, pocet trnov: %d, iLastSeen: %d", 
		cSenseBody.iTurnCount, 
		cThis.iLastSeen
	);


	return KPDL_SUCCESS;
}

KPDL_RESULT CWorldModel::ProcessInit(CInitMsg &cInitMsg)
{
	cThis.iNumber = cInitMsg.iNumber;
	cTime.ePlayMode = cInitMsg.ePlayMode;

	return KPDL_SUCCESS;
}

KPDL_RESULT CWorldModel::ProcessHear(CHearMsg &cHearMsg)
{
	if(cHearMsg.eHearMsg == HM_Referee)
	{
		cTime.ePlayMode = cHearMsg.ePlayMode;
		cTime.iPlayTime = cHearMsg.iPlayTime;
	} 
	else 
		cHear = cHearMsg;

	return KPDL_SUCCESS;
}

KPDL_RESULT CWorldModel::Recompute(CSeeMsg &cSeeMsg)
{
	UpdatePlayerPosition(cSeeMsg.cFlags, cSeeMsg.cGoals, cSeeMsg.cLines);

	// Update polohy lopty
	cBall.Percept(cSeeMsg.cBall, cThis); 

	// zmaz existujuce zoznamy agentov
	TAgentVectorIterator pIter = cIdentifiedAgents.begin();
	TAgentVectorIterator pLast = cIdentifiedAgents.end();

	for(; pIter != pLast; pIter++)
		delete (*pIter);

	pIter = cUnIdentifiedAgents.begin();
	pLast = cUnIdentifiedAgents.end();

	for(; pIter != pLast; pIter++)
		delete (*pIter);

	cIdentifiedAgents.clear();
	cUnIdentifiedAgents.clear();

	// aktualizuj hracov podla vnemu
	TSeenPlayerVectorIterator pPlayerIter = cSeeMsg.cPlayers.begin();
	TSeenPlayerVectorIterator pPlayerLast = cSeeMsg.cPlayers.end();

	for(; pPlayerIter != pPlayerLast; pPlayerIter++)
	{
		CSeenPlayer& cSeenPlayer = (*pPlayerIter);
		CAgent* pAgent = new CAgent(*this);

		pAgent->Percept(cSeenPlayer, cThis);
		if(IsValidCount(pAgent->iNumber))
			cIdentifiedAgents.push_back(pAgent);
		else
			cUnIdentifiedAgents.push_back(pAgent);
	}

	cTime.iPlayTime = cSeeMsg.iPlayTime;

/*
	//CSeenPlayersVect my_seen_vect=Perception->GetSeenPlayers();
	int seen_count=cSeeMsg.cPlayers.size();
	int unid_count=cUnIdentifiedAgents.size();
	int id_count=cIdentifiedAgents.size();
	int i,j;

	for(i=0;i<id_count;i++)
	{
		cIdentifiedAgents[i]->SetSeeStatus(cIdentifiedAgents[i]->iSeeStatus|STATE_IS_VISIBLE, 0);
        cIdentifiedAgents[i]->cLastVisible = local.cTime;
	}

	for(i=0;i<unid_count;i++)
	{
		cUnIdentifiedAgents[i]->SetSeeStatus (cUnIdentifiedAgents[i]->iSeeStatus|STATE_IS_VISIBLE, 0);
        cUnIdentifiedAgents[i]->cLastVisible = local.cTime;
	}

	double **id_vals;
	double **unid_vals;
	bool *matched_unid;
	bool *matched_seen;
	CSeenPlayer **act_seen;

	if(unid_count)
	{
		matched_unid=new bool   [unid_count];
		memset(matched_unid,0,unid_count*sizeof(bool));
	}
	
	if(seen_count)
	{
		CSeenPlayer *tmp_player=new CSeenPlayer;
		int tmp_poz=cSeeMsg.GetPlayer(*tmp_player);

		act_seen = new CSeenPlayer *[seen_count];

		if(id_count)
			id_vals = new double*[id_count];

		if(unid_count)
			unid_vals = new double*[unid_count];

		matched_seen=new bool[seen_count];
		memset(matched_seen,0,seen_count*sizeof(bool));

        act_seen[0]=tmp_player;
		for(i=1;i<seen_count;i++)
		{
			tmp_player=new CSeenPlayer;
			tmp_poz=cSeeMsg.GetPlayer(*tmp_player,tmp_poz);
            act_seen[i]=tmp_player;
		}

		for(i=0;i<id_count;i++)
		{
			id_vals[i]=new double[seen_count];

			for(j=0;j<seen_count;j++)
				id_vals[i][j]=cIdentifiedAgents[i]->TestOwnership(*(act_seen[j]),cThis);
		}

		for(i=0;i<unid_count;i++)
		{
			unid_vals[i]=new double[seen_count];

			for(j=0;j<seen_count;j++)
				unid_vals[i][j]=cUnIdentifiedAgents[i]->TestOwnership(*(act_seen[j]),cThis);
		}
	}

	int i_id=-1,j_id=-1;
	double max_id=-1;
	int seen_rem=seen_count;
	bool not_hotovo=true;

	while(not_hotovo)
	{
		int i_unid=-1,j_unid=-1;
		double max_unid=-1;
		int i_id=-1,j_id=-1;
		double max_id=-1;

		for(i=0;i<unid_count;i++)
			for(j=0;j<seen_count;j++)
				if(max_unid<unid_vals[i][j])
				{
					i_unid=i;
					j_unid=j;
					max_unid=unid_vals[i][j];
				}

		for(i=0;i<id_count;i++)
			for(j=0;j<seen_count;j++)
				if(max_id<id_vals[i][j])
				{
					i_id=i;
					j_id=j;
					max_id=id_vals[i][j];
				}

		if(max_id>=max_unid)
		{
			if(max_id>0)
			{
				cIdentifiedAgents[i_id]->Update(*(act_seen[j_id]),cThis);
				matched_seen[j_id]=true;

				for(j=0;j<seen_count;j++)
					id_vals[i_id][j]=-1;

				for(i=0;i<id_count;i++)
					id_vals[i][j_id]=-1;
			}
			else
				not_hotovo=false;
		}
		else if(max_unid>0)
		{
			cUnIdentifiedAgents[i_unid]->Update(*(act_seen[j_unid]),cThis);
			matched_unid[i_unid]=true;
			matched_seen[j_unid]=true;

			for(j=0;j<seen_count;j++)
				unid_vals[i_unid][j]=-1;

			for(i=0;i<unid_count;i++)
				unid_vals[i][j_unid]=-1;
		}
		else
			not_hotovo=false;
	}

	for(i=0;i<unid_count;i++)
		if(matched_unid[i])
		{
			if((cUnIdentifiedAgents[i]->iSeeStatus & SEE_SIDE) &&(cUnIdentifiedAgents[i]->iSeeStatus & SEE_NUMBER))
			{
				for(j=0;j<id_count;j++)
					if(	cIdentifiedAgents[j]->eSide==cUnIdentifiedAgents[i]->eSide &&
						cIdentifiedAgents[j]->iNumber==cUnIdentifiedAgents[i]->iNumber
					)
						break;

				if(j!=id_count)
				{
					delete cIdentifiedAgents[j];
					cIdentifiedAgents[j]=cUnIdentifiedAgents[i];
				}
				else
					cIdentifiedAgents.insert(cIdentifiedAgents.end(),cUnIdentifiedAgents[i]);

				cUnIdentifiedAgents[i]=0;
			}
		}
		else
		{
			delete cUnIdentifiedAgents[i];
			cUnIdentifiedAgents[i]=0;
		}

	bool one_zero=true;
	while(one_zero)
	{
		one_zero=false;

		for(i=0;i<cUnIdentifiedAgents.size();i++)
			if(!cUnIdentifiedAgents[i])
			{
				cUnIdentifiedAgents.erase(&(cUnIdentifiedAgents[i]));
				one_zero=true;
				break;
			}
	}

	for(i=0;i<seen_count;i++)
	{
		if(!matched_seen[i])
		{
			CAgent *new_agent=new CAgent;
			new_agent->Update(*(act_seen[i]),cThis);

			if((new_agent->iSeeStatus &SEE_SIDE) &&(new_agent->iSeeStatus &SEE_NUMBER))
				cIdentifiedAgents.insert(cIdentifiedAgents.end(),new_agent);
			else
				cUnIdentifiedAgents.insert(cUnIdentifiedAgents.end(),new_agent);
		}
	}

	if(seen_count)
	{
		for(i=0;i<id_count;i++)
			delete []id_vals[i];

		if(id_count)
			delete []id_vals;

		for(i=0;i<unid_count;i++)
			delete []unid_vals[i];

		if(unid_count)
			delete []unid_vals;

		for(i=0;i<seen_count;i++)
			delete act_seen[i];

		delete []act_seen;
		delete []matched_seen;
	}

	if(unid_count) delete []matched_unid;
*/
	return KPDL_SUCCESS;
}

/******************************************************************************
** Update modelu sveta
******************************************************************************/
/*
void Update(
	CGameTime cGameTime, 
	CPerceptionSeq *pPerceptionSeq,
	CGameCommandSeq *pSentCommandSeq,
	CGameCommandSeq *pGameCommandSeq
)
// vstupy:
//   cGameTime -> cas ku ktoremu sa ma agent rozhodovat
//   pPerceptionSeq -> postupnost vnemy, ktore od posledneho Deliberate prisli
//   pSentCommandSeq -> postupnost povelov, ktore boli poslane serveru 
// vystup:
//   pGameCommandSeq -> postupnost povelov, ktore sa maju odoslat serveru
{
    // poznacime si GameCommandSeq do kontextu kvoli skillsom
    actionseq = pGameCommandSeq;

    CPerception *pPerception;
	CGameCommand *pGameCommand;

    static CPerception world_perception;
    static CPerception rPerception0;  // FIXMEEE: pouzivane len na vizualizaciu 

    int received_info = 0;
    int received_info0 = 0;

    // QUICK FIX NA OTACANIE - ZRUSIT KED BUDE DOBRY WORLDMODEL (SO ZAPOCITANIM POSLEDNEJ HRACOVEJ AKCIE)
    double lastplrdir = dDirection;
    double lastneckdir = dNeckDirection;

	bool bIsCmd, bIsPerc;
	bIsPerc = bIsCmd = false;

    // ak by sme sa mali rozhodovat pre cas v minulosti, alebo do prilis 
    //   dalekej buducnosti, tak asi nemame spravny cas a potrebujeme si ho nastavit
    if ((cGameTime <= cTime) || (cGameTime >= cTime.m_nAllCycles + MAXPRECOMPUTE))
        cTime = cGameTime;

	// potrebujeme prepocitat svet az do casu cGameTime
	while (cGameTime >= cTime) 
	{

        ///fprintf(m_ff, "%d. [%d] before_perc x:%f, y:%f, d:%f, nd:%f\n", cGameTime.m_nAllCycles, cTime.m_nAllCycles, (float)cPosition.GetX(), (float)cPosition.GetY(), (float)RadToDeg(dDirection), (float)RadToDeg(dNeckDirection));

		// nacitaj prvy vnem ---------------------------------------------------------
		if (!bIsPerc)
			bIsPerc = pPerceptionSeq->GetPerception(&pPerception);

		// vyhod vsetky moc stare vnemy
		while (bIsPerc && (pPerception->cGameTime < cTime)
		) 
		{
			delete pPerception;
			bIsPerc = pPerceptionSeq->GetPerception(&pPerception);
		}

		// aplikuj vsetky aktualne vnemy
		received_info =  0;
		while (bIsPerc && (pPerception->cGameTime == cTime)) 
		{
			switch(pPerception->eType)
			{
			case MT_See:
				world_perception = *pPerception;  // brrrrr
				received_info |= GOT_SEE;
				
				UpdatePlayerPosition(pPerception->cSeenFlags, pPerception->cSeenGoals, pPerception->cSeenLines);
				
                // niekedy uz je pointer nastaveny, ale okno este neexistuje
				if(gpVisualView != NULL)
				{
					rPerception0 = *pPerception;  // brrrrr
					gpVisualView->pPerception = &rPerception0;
                    gpVisualView->Invalidate();
					
//					CChildView *vv = m_pReadyView;
					
//					vv->m_pAgent->SetPosition(cPosition);
//					vv->m_pAgent->SetDirection(dDirection);
//					vv->m_pAgent->dNeckDirection = dNeckDirection;
//	                    m_pReadyView = NULL;  // FIXMEEEE
				}
				break;

			case MT_Hear:
				received_info |= GOT_HEAR;
				//fprintf(m_ff, "<hear>\n");
				
				EPlayMode playmode = pPerception->GetPlayMode();
				if (playmode != PM_NONE)  SetPlayMode(playmode);  // FIXME
				break;

			case MT_SenseBody:
				received_info |= GOT_SENSE_BODY;
				CSenseBody my_body;
				
				pPerception->GetSenseBody(&my_body);
				UpdatePlayerProperties(&my_body);
				UpdatePlayerPosition(pPerception->cSeenFlags, pPerception->cSeenGoals, pPerception->cSeenLines);
				
				if (gpVisualView != NULL)
				{
					rPerception0 = *pPerception;
					gpVisualView->pPerception = &rPerception0;
						
//					CVisualView *vv = gpVisualView;
//					vv->m_pAgent->SetDirection(dDirection);
				}
				break;

			case MT_Init:
				CInitMsg cInitMsg;
					
				pPerception->GetInitMsg(&cInitMsg);
				eSide = cInitMsg.eSide;
				iNumber = cInitMsg.iUniformNum;
					
				// FIXME:
				m_bGoalie = g_bCoordinatorInitGoalie;

                // FIXME
                field = new CLocalStaticField(eSide, ::field);
                fdata = new CFormationData();
					
					EPlayMode playmode = pPerception->GetPlayMode();
				if (playmode != PM_NONE) SetPlayMode(playmode);  // FIXME

				// "resetneme" systemovy cas
				cTime = cGameTime;
				break;
			}

			delete pPerception;
			bIsPerc = pPerceptionSeq->GetPerception(&pPerception);
		}

		if (received_info & GOT_SEE)
			UpdatePlayerSeenObjects(world_perception);

		received_info0 |= received_info;

		// v aktualnom kroku vyskocime -----------------------------------------------
		if (cGameTime == cTime) 
			break;

		// nacitaj prvy command ------------------------------------------------------
		if (!bIsCmd)
			bIsCmd = pSentCommandSeq->GetGameCommand(&pGameCommand);

		while (bIsCmd && (pGameCommand->cGameTime < cTime)) 
		{
			delete pGameCommand;
			bIsCmd = pSentCommandSeq->GetGameCommand(&pGameCommand);
		}

		while (bIsCmd && (pGameCommand->cGameTime == cTime))
		{
			switch(pGameCommand->eCmdType)
			{
			case CT__TURN:
				double zmena=DegToRad((pGameCommand->GetDArg(1))/(1.0+5.0*cSpeed.GetDistance()*0.9));
				dDirection+=zmena;
				dPrevNeckDirection=dNeckDirection;
				dNeckDirection+=zmena;
				dAngularSpeed=zmena;
				iSeeStatus|=PREV_NECK_DIRECTION|EXPECT_ANGULAR_SPEED;
				break;

			case CT__NECK:
				double zmena=DegToRad(pGameCommand->GetDArg(1));
				dPrevNeckDirection=dNeckDirection;
				dNeckDirection+=zmena;
				dAngularSpeed=zmena;
				iSeeStatus|=PREV_NECK_DIRECTION|EXPECT_ANGULAR_SPEED;
				break;

			case CT__KICK:
  // Rado Otipka 16. 12. 2001 13:16:39
				C2DVector poz = cWorld.cBall.cPosition - cPosition;
				double direct = pGameCommand->GetDArg(1);

				double pwr=0.027*(1-0.25*(poz.GetAngle ()-dDirection)/PI-0.25*(poz.GetDistance ()-0.3)/0.7);
				C2DVector acca(pwr* pGameCommand->GetDArg(1),dDirection+pGameCommand->GetDArg(2));
				cWorld.cBall.cAcceleration = acca;
  // Rado Otipka 16. 12. 2001 13:16:49
				break;

			case CT__DASH:
				double pwr=dEffort*pGameCommand->GetDArg(1);
				cAcceleration=0.06*pwr*C2DVector(CAngular2DVector(1,dDirection));
				dStamina=__max(0,dStamina-pwr);
				break;

			case CT__CHANGEVIEW: 
				string width = pGameCommand->GetSArg(1);
				string res = pGameCommand->GetSArg(2);
				if(strcmp(width.data(),"normal")==0) 
					iVisualSettings = VISUAL_RANGE_NORMAL; 
				else if(strcmp(width.data(),"narrow")==0)
					iVisualSettings = VISUAL_RANGE_NARROW; 
				else if(strcmp(width.data(),"wide")==0)
					iVisualSettings = VISUAL_RANGE_WIDE;
				break;

			case CT__MOVE:
				//iSeeStatus=0;
				cPosition=C2DVector(pGameCommand->GetDArg(1),pGameCommand->GetDArg(2));
				cSpeed=C2DVector(0.0,0.0);
				cAcceleration=C2DVector(0.0,0.0);
				break;
			}

			delete pGameCommand;
			bIsCmd=pSentCommandSeq->GetGameCommand(&pGameCommand);
		}

		Precompute();
	}

	if(bIsPerc)
	{
		delete pPerception;
		bIsPerc = false;
	}

	if(bIsCmd)
	{
		delete pGameCommand;
		bIsCmd = false;
	}

    // nastavime presny cas, ku ktoremu sme prepocitali svet
	cTime = cGameTime;

    // FIXME
    AngNormalise(dDirection);
    AngNormalise(dNeckDirection);
}
*/

KPDL_RESULT CWorldModel::UpdatePlayerPosition(
	TSeenFlagVector &cFlags, 
	TSeenGoalVector &cGoals, 
	TSeenLineVector &cLines
)
{
	int iIndex;

	//-------------------------------------------------------------------------
	// algoritmus urcenia polohy pomocou zastaviek
	double dSpotWTotal;
	double dSpotError;
	double dSpotAngle = INVALID_ANGLE;
	C2DVector cSpotPosition;

	T2DVectorVector cVisSpots;
	T2DVectorVector cDefSpots;
	TAngular2DVectorVector cAngSpots;
	int iSpotSize = cFlags.size() + cGoals.size();

	cVisSpots.resize(iSpotSize);
	cDefSpots.resize(iSpotSize);
	cAngSpots.resize(iSpotSize);

	//-------------------------------------------------------------------------
	// spocitame relativne suradnice bodov
	EFlagType eFlag = F_None;
	int iFlagSize = 0;

	for(iIndex = 0; iIndex < cFlags.size(); iIndex++)
		if((eFlag = cFlags[iIndex].eFlag) != F_None)
		{
			CAngular2DVector cFlagPos(
				cFlags[iIndex].dDirection, 
				cFlags[iIndex].dDistance
			);
			
			cDefSpots[iFlagSize] = cField.GetFlag(eFlag);
			cVisSpots[iFlagSize] = cAngSpots[iFlagSize] = cFlagPos;
			iFlagSize++;
		}

	EGoalType eGoal = G_None;
	int iGoalSize = 0;

	for(iIndex = 0; iIndex < cGoals.size(); iIndex++)
		if((eGoal = cGoals[iIndex].eGoal) != G_None)
		{
			CAngular2DVector cGoalPos(
				cGoals[iIndex].dDirection, 
				cGoals[iIndex].dDistance
			);
			
			cDefSpots[iGoalSize + iFlagSize] = cField.GetGoal(eGoal);
			cVisSpots[iGoalSize + iFlagSize] = 
				cAngSpots[iGoalSize + iFlagSize] = cGoalPos;
			iGoalSize++;
		}

	iSpotSize = iFlagSize + iGoalSize;

	//-------------------------------------------------------------------------
	// vypocet uhla pohladu hraca pomocou vlajok
	double dWeight;
	C2DVector cDefTotal;
	C2DVector cVisTotal;

	// vahovany aritmeticky priemer 
	dSpotWTotal = 0.0;
	for(iIndex = 0 ; iIndex < iSpotSize ; iIndex++)
	{
		dWeight = WeightFn(
			cAngSpots[iIndex].GetDistance(), 
			gpConfig->quantize_step_l
		);

		cDefTotal += dWeight * cDefSpots[iIndex];
		cVisTotal += dWeight * cVisSpots[iIndex];
		dSpotWTotal += dWeight;
	}

	// upravena metoda najmensich stvorcov (Marian Bobrik)
	if(dSpotWTotal > 0)
	{
		cDefTotal /= dSpotWTotal;
		cVisTotal /= dSpotWTotal;

		double A = 0.0;
		double B = 0.0;
		double C = 0.0;

		for(iIndex = 0; iIndex < iSpotSize; iIndex++)
		{
			dWeight = WeightFn(
				cAngSpots[iIndex].GetDistance(), 
				gpConfig->quantize_step_l
			);
			
			C2DVector cRemainder = cDefTotal - cDefSpots[iIndex];
			A += dWeight * (cVisSpots[iIndex] & cVisTotal);
			B += dWeight * (cVisSpots[iIndex] & cRemainder);
			C += dWeight * (cVisSpots[iIndex] * cRemainder);
		}

		double D = sqrt(B * B + C * C);
		if(D != 0.0)
			dSpotAngle = acosf(A / D) + atan2(C , B);
	}

	//-------------------------------------------------------------------------
	// algoritmus urcenia polohy pomocou ciar (ciarovy algoritmus)
	int iLineWTotal = 0;
	double dLineError;
	double dLineAngle = INVALID_ANGLE;
	C2DVector cLinePosition;
	int iLineSize = cLines.size();
	
	//-------------------------------------------------------------------------
	// vypocet uhla pohladu
	double dAngleTotal = 0.0;
	for(iIndex = 0; iIndex < iLineSize; iIndex++)
	{
		double dAngDiff = 
			cLines[iIndex].dDirection - 
			cField.GetLine(cLines[iIndex].eLine).GetAngle();
		
		if(IsValidAngle(dSpotAngle))
		{
			if(fabs(dAngDiff - dSpotAngle) > 0.5 * PI)
				// hrac stoji na druhej strane ciary
				dAngDiff = AngNormalise(dAngDiff + PI);

			dAngleTotal += dAngDiff;
			iLineWTotal++;
		}
		else if(IsValidAngle(cThis.dHeadDirection))
		{
			if(fabs(dAngDiff - cThis.dHeadDirection) > 0.5 * PI)
				// hrac stoji na druhej strane ciary
				dAngDiff = AngNormalise(dAngDiff + PI);

			dAngleTotal += dAngDiff;
			iLineWTotal++;
		}
	}
	
	if(iLineWTotal > 0)
		dLineAngle = dAngleTotal / iLineWTotal;
	
	//-------------------------------------------------------------------------
	// vyberieme uhol pohladu hraca - z ciaroveho algoritmu je vraj presnejsi
	double dAngle = INVALID_ANGLE;

	if(IsValidAngle(dLineAngle))
		dAngle = dLineAngle;
	else if(IsValidAngle(dSpotAngle))
		dAngle = dSpotAngle;
	
	//-------------------------------------------------------------------------
	// vypocet polohy - vlajkovy algoritmus
	if(IsValidAngle(dAngle) && dSpotWTotal > 0.0)
	{
		// toto je nieco ako algoritmus preratania pozicii 2
		cSpotPosition = cDefTotal - Rot(cVisTotal, -dAngle);

        // a na zaver zratame svoju vlastnu neurcitost
		dSpotError = 0.0;
		for(iIndex = 0; iIndex < iSpotSize; iIndex++)
		{
			C2DVector cDiffError = 
				cVisSpots[iIndex] - 
				Rot(cDefSpots[iIndex] - cSpotPosition, dAngle);

			dSpotError += WeightFn(
				cAngSpots[iIndex].GetDistance(), 
				gpConfig->quantize_step_l
			) * (cDiffError * cDiffError);
		}

		dSpotError = sqrt(dSpotError / dSpotWTotal);
	}

	//-------------------------------------------------------------------------
	// vypocet polohy - ciarovy algoritmus
	if(IsValidAngle(dAngle) && iLineSize > 1)
	{
		double A = 0.0;
		double B = 0.0;
		double C = 0.0;
		double G = 0.0;
		double H = 0.0;
		
		//! t0n0: problem je, ze server quantizuje aj vzdialenost od ciary
		iLineWTotal = 0;
		
		for(iIndex = 0; iIndex < iLineSize; iIndex++)
		{
			double an = cField.GetLine(cLines[iIndex].eLine).GetAngle();
			double dn = cField.GetLine(cLines[iIndex].eLine).GetShift();
			double di = cLines[iIndex].dDistance;
			
			double SCL = (-cos(dAngle) * sin(an) + sin(dAngle) * cos(an));
			if(SCL != 0.0)
			{
				SCL = 1.0 / SCL;
				double EX = -sin(an) * SCL;
				double EY = cos(an) * SCL;
				double ED = dn * SCL;
	
				A += EX * EX;
				B += EX * EY;
				C += EY * EY;
				
				G = (dn - di) * EX;
				H = (dn - di) * EY;
				
				iLineWTotal ++;
			}
		}
		
		double det = (B * B - A * C);

		if(iLineWTotal > 0 && det != 0.0)
		{
			cLinePosition = 
				(1.0 / det) * C2DVector(H * B - G * C, G * B - H * A);
			
			for(iIndex = 0; iIndex < iLineSize; iIndex++)
			{
				double an = cField.GetLine(cLines[iIndex].eLine).GetAngle();
				double dn = cField.GetLine(cLines[iIndex].eLine).GetShift();
				double di = cLines[iIndex].dDistance;
				
				dLineError = 0.0;
				
				double SCL = -cos(dAngle) * sin(an) + sin(dAngle) * cos(an);
				if(SCL != 0.0)
				{
					SCL = 1.0 / SCL;
					double EX = -sin(an) * SCL;
					double EY = cos(an) * SCL;
					double ED = dn * SCL;
					
					double p = 
						cLinePosition.GetX() * EX + 
						cLinePosition.GetY() * EY + 
						dn - di;
					
					dLineError += p * p;
				}
			}

			dLineError = sqrt(dLineError / iLineWTotal);
		}
	}

	double dFieldError;
	C2DVector cFieldPosition;

	if(!cSpotPosition.IsValid() || 
		(cLinePosition.IsValid() && dLineError < 3 * dSpotError)
	) 
	{
		// dSpotError je 2 * sqrt(3) krat nespolahlivejsi :)
		dFieldError = dLineError;
		cFieldPosition = cLinePosition;
	}
	else
	{
		dFieldError = dSpotError;
		cFieldPosition = cSpotPosition;
	}
	
	//-------------------------------------------------------------------------
	// t0t0 vyzera ako algoritmus vypoctu polohy hraca 
	// pomocou prieniku intervalov
	double xmin = -INVALID_COORD;
	double ymin = -INVALID_COORD;
	double xmax = INVALID_COORD;
	double ymax = INVALID_COORD;

	for(int i = 0; i < iSpotSize; i++)
	{
		for(int j = i + 1; j < iSpotSize; j += 1)
		{
			C2DVector cOrig = cDefSpots[i];
			C2DVector cDiff = cDefSpots[j] - cOrig;
			double d12s = cDiff.GetDistance();
			double d12 = d12s * d12s;
			
			double d1L = minEQ(cAngSpots[i].GetDistance(), gpConfig->quantize_step_l);
			double d1H = maxEQ(cAngSpots[i].GetDistance(), gpConfig->quantize_step_l);
			double d2L = minEQ(cAngSpots[j].GetDistance(), gpConfig->quantize_step_l);
			double d2H = maxEQ(cAngSpots[j].GetDistance(), gpConfig->quantize_step_l);

			if((d12s > d1L + d2L) || (d1H > d12s + d2L) || (d2H > d12s + d1L))
				continue;

			double xp[4],yp[4];

			d1L *= d1L;
			d1H *= d1H;
			d2L *= d2L;
			d2H *= d2H;

			double tm;
			double t;
			C2DVector cM;

			bool minflag = (cAngSpots[j].GetAngle() - cAngSpots[i].GetAngle()) < 0.0;

			tm = 0.5 * ((d1L - d2L) / d12 + 1.0);
			cM = cOrig + cDiff * tm;
			t = sqrt(d1L / d12 - tm * tm);
			if(minflag)
				t = -t;
			xp[0] = cM.GetX() - cDiff.GetY() * t;
			yp[0] = cM.GetY() + cDiff.GetX() * t;

			tm = 0.5 * ((d1H - d2L) / d12 + 1.0);
			cM = cOrig + cDiff * tm;
			t = sqrt(d1H / d12 - tm * tm);
			if(minflag) 
				t = -t;
			xp[1] = cM.GetX() - cDiff.GetY() * t;
			yp[1] = cM.GetY() + cDiff.GetX() * t;

			tm = 0.5 * ((d1L - d2H) / d12 + 1.0);
			cM = cOrig + cDiff * tm;
			t = sqrt(d1L / d12 - tm * tm);
			if(minflag)
				t = -t;
			xp[2] = cM.GetX() - cDiff.GetY() * t;
			yp[2] = cM.GetY() + cDiff.GetX() * t;

			tm = 0.5 * ((d1H - d2H) / d12 + 1.0);
			cM = cOrig + cDiff * tm;
			t = sqrt(d1H / d12 - tm * tm);
			if(minflag)
				t = -t;
			xp[3] = cM.GetX() - cDiff.GetY() * t;
			yp[3] = cM.GetY() + cDiff.GetX() * t;

			double xl = INVALID_COORD;
			double xh = -INVALID_COORD;
			double yl = INVALID_COORD;
			double yh = -INVALID_COORD;

			for(int l = 0; l < 4; l++)
			{
				if(xl > xp[l])
					xl = xp[l];
				if(xh < xp[l])
					xh = xp[l];
				if(yl > yp[l])
					yl = yp[l];
				if(yh < yp[l])
					yh = yp[l];
			}

			double k = 1.2; //! ?? clovek nevychadza z udivu

			double xln = ((1.0 + k) * xl + (1.0 - k) * xh) * 0.5;
			double xhn = ((1.0 - k) * xl + (1.0 + k) * xh) * 0.5;
			double yln = ((1.0 + k) * yl + (1.0 - k) * yh) * 0.5;
			double yhn = ((1.0 - k) * yl + (1.0 + k) * yh) * 0.5;

			xmin = __max(xln, xmin);
			xmax = __min(xhn, xmax);
			ymin = __max(yln, ymin);
			ymax = __min(yhn, ymax);
		}
	}
	
	// vysledok intervaloveho algoritmu :)
	C2DVector cIntervalPos((xmin + xmax) * 0.5, (ymin + ymax) * 0.5);
	// a jeho neurcitost :(
	double dIntervalError = __max(fabs(xmax - xmin) * 0.5, fabs(ymax - ymin) * 0.5);
	// if algorithm works, it never will be higher than cca 7.6!!!
	bool bIntervalValid = dIntervalError < 10.0;

	//-------------------------------------------------------------------------
	// vyber konecnej polohy hraca
	//! t0n0: na ladiace ucely !!!
	cThis.cPosition.SetXY(INVALID_COORD, INVALID_COORD).Validate(false);
	cThis.dHeadDirection = cThis.dBodyDirection = INVALID_ANGLE;

	if(cSpotPosition.IsValid() || cLinePosition.IsValid())
	{
		if(bIntervalValid && (dIntervalError < 10 * dFieldError))
			cThis.cPosition = cIntervalPos;
		else
			cThis.cPosition = cFieldPosition;
	}
	else if(bIntervalValid)
		cThis.cPosition = cIntervalPos;

	//-------------------------------------------------------------------------
	// urcenie uhla pohladu hraca (t0n0)
	if(cThis.cPosition.IsValid() && iSpotSize > 0)
	{
		int iSpotCount = 0;
		double dAngleSum = 0.0;

		//! jednoducho nastavit HeadDirection
		// mozno uvazit pouzitie viacerych bodov
		for(iIndex = 0; iIndex < iSpotSize; iIndex++)
		{
			if(cAngSpots[iIndex].GetDistance() > 0)
			{
				dAngleSum += 
					(cDefSpots[iIndex] - cThis.cPosition).GetAngle() - 
					cAngSpots[iIndex].GetAngle();
				iSpotCount ++;
			}
		}
		if(iSpotCount > 0)
			cThis.dHeadDirection = cThis.dBodyDirection 
				= dAngleSum / iSpotCount;

		//! docasne riesenie pre Rada :)
		cThis.iLastSeen = 0;
	}

	return KPDL_SUCCESS;
}


/******************************************************************************
** Termy
******************************************************************************/

string CWorldModel::SomeoneHasBall()
{
	if(!cBall.IsValid() ||
		!cBall.cPosition.IsValid()
	)
		// ... sanca je 50 na 50 ked nepoznam polohu lopty
		//! (neskuor aj jej rychlost ...)
		return string("0.50");

	char sBuffer[QUERY_MAX_BUFFER];
	CAgent *pAgent = NULL;

	double dBestStepDist = WORLD_MAX_BALL_DIST; 
	double dBallDist;
	double dStepDist;

	TAgentVectorIterator pLast = cIdentifiedAgents.end();
	TAgentVectorIterator pIter = cIdentifiedAgents.begin();

	// znami hraci
	for(; pIter != pLast; pIter ++)
	{
		pAgent = (*pIter);

		if(!pAgent->cPosition.IsValid())
			// preskakujeme hracov, ktorych polohu nepozname
			continue;

		dBallDist = 
			(cBall.cPosition - pAgent->cPosition).GetDistance();

		// predpokladajme, ze hrac sa bude k lopte priblizovat priamo,
		// maximalnou rychlostou (a ze vsetci hraci behaju rovnako rychlo)
		dStepDist = dBallDist / gpConfig->player_speed_max;

		if(dBallDist < gpConfig->kickable_area ||
			dStepDist < WORLD_MAX_BALL_CONTROL_DIST
		)
			return string("1.00");

		if(dBestStepDist < dStepDist)
			dBestStepDist = dStepDist;
	}

	pLast = cUnIdentifiedAgents.end();
	pIter = cUnIdentifiedAgents.begin();

	// neznami hraci
	for(; pIter != pLast; pIter ++)
	{
		pAgent = (*pIter);

		if(!pAgent->cPosition.IsValid())
			// preskakujeme hracov, ktorych polohu nepozname
			continue;

		dBallDist = 
			(cBall.cPosition - pAgent->cPosition).GetDistance();

		// predpokladajme, ze hrac sa bude k lopte priblizovat priamo,
		// maximalnou rychlostou (a ze vsetci hraci behaju rovnako rychlo)
		dStepDist = dBallDist / gpConfig->player_speed_max;

		if(dBallDist < gpConfig->kickable_area ||
			dStepDist < WORLD_MAX_BALL_CONTROL_DIST
		)
			return string("1.00");

		if(dBestStepDist < dStepDist)
			dBestStepDist = dStepDist;
	}

	sprintf(sBuffer, "%.2f", (1.0 - dBestStepDist / WORLD_MAX_BALL_DIST));
	return string(sBuffer);
}

string CWorldModel::OpponentGoal()
{
	return cField.GetGoal(G_Opponent).GetAttribute();
}

string CWorldModel::IsPassMessage()
{
	static char tmp[100];
	int iNumber;
	double dDir;
	sscanf(cHear.sMessage.c_str(),"%s %d %.2f",tmp, &iNumber, &dDir);
	if(cHear.iPlayTime != cTime.iPlayTime	||
	   cHear.eMsgType == HM_Self			||
	   cHear.eMsgType != HM_Other			||
	   cThis.iNumber  != iNumber			||
	   strcmp(tmp,"nahravka:") != 0
	) 
		return string("0.0");
	else 
		return string("1.0");
}

//======================================================
// CWorldModel : implementacia CQuery
//======================================================
string CWorldModel::GetAttribute(const string &sName)
{
	string sHeader = sName;
	string sTail = SplitTail(sHeader);

	if(sHeader == "ball")
		return cBall.GetAttribute(sTail); 

	if(sHeader == "me")
		return cThis.Evaluate(sTail);

	if(sHeader == "time")
		return cTime.GetAttribute(sTail);

	return string(QUERY_NOT_AVAILABLE);
}

void CWorldModel::ClearCache()
{
	CQuery::ClearCache();
	cBall.ClearCache();
	cThis.ClearCache();
}
